#!/bin/bash

function GOTO
{
	label=$1
	cmd=$(sed -n "/:$label:/{:a;n;p;ba};" $0 | grep -v ':$')
	eval "$cmd"
	exit
}

# Definicio variables 
export PATH_ACTUAL=$(eval "pwd")
export RUTA_SELECTS=${PATH_ACTUAL}/scripts/selects
export ORACLE_HOME=
export SYSTEM_PASSWORD=
export USUARI_BD=
export SID_BD=
export PARTICIONAR_TAULES=N
export SCRIPT_CREACIO_TAULES=${PATH_ACTUAL}/scripts/taules
export SCRIPT_CREACIO_PAQUETS=${PATH_ACTUAL}/scripts/paquets/scripts_paquets.sql
export SCRIPT_DIR_BD=${PATH_ACTUAL}/scripts/directoris_bd/directoris_bd.sql
export SCRIPT_PARAMETRES=${PATH_ACTUAL}/scripts/parametres/ins_parametres.sql
export SCRIPT_CREACIO_VISTES=${PATH_ACTUAL}/scripts/vistes/scripts_vistes.sql
export SCRIPT_RECOMPILAR_PAQUETS=${PATH_ACTUAL}/scripts/paquets/recompilar.sql
export SCRIPT_CREACIO_USUARI=${PATH_ACTUAL}/scripts/usuari/usuari.sql
export SCRIPT_CREACIO_DISPARADORS=${PATH_ACTUAL}/scripts/disparadors/scripts_disparadors.sql
export TABLESPACE_TAULES=
export TABLESPACE_INDEXS=
export LOG_INSTALACIO=${PATH_ACTUAL}/scripts/instalacio.log

cd $PATH_ACTUAL

# Aspectes configurables per part de l' usuari 
echo -n "Indiqui la ruta d' instal.lacio base del client Oracle (ORACLE_HOME): "
read ORACLE_HOME
# Es verifica si en la ruta informada existeix el SQLPLUS
export DIR_SQLPLUS=${ORACLE_HOME}/bin/sqlplus
ls $DIR_SQLPLUS >> $LOG_INSTALACIO
if [ -f $DIR_SQLPLUS ];
then
	echo "ORACLE_HOME correcte"
else
	GOTO SQLPLUS_NO_EXISTEIX
fi

export NLS_LANG=CATALAN_CATALONIA.WE8MSWIN1252

# Es pregunten les dades de connexio a l' usuari 
export RETORN_PREGUNTAR_CONNEXIO_BD=POST_PREGUNTAR_CONNEXIO_BD
GOTO PREGUNTAR_CONNEXIO_BD
:POST_PREGUNTAR_CONNEXIO_BD:

# Es verifica la connectivitat amb la base de dades 
export RETORN_VERIFICAR_CONNEXIO_BD=POST_VERIFICAR_CONNEXIO_BD
GOTO VERIFICAR_CONNEXIO_BD
:POST_VERIFICAR_CONNEXIO_BD:

# Es verifica el propietari dels objectes 
export RETORN_PROPIETARI_OBJECTES=POST_PROPIETARI_OBJECTES
GOTO PROPIETARI_OBJECTES
:POST_PROPIETARI_OBJECTES:

# Es verifica la existencia del tablespace de taules 
export RETORN_VERIFICAR_TABLESPACE_TAB=POST_VERIFICAR_TABLESPACE_TAB
GOTO VERIFICAR_TABLESPACE_TAB
:POST_VERIFICAR_TABLESPACE_TAB:

# Es verifica la existencia del tables de indexs
export RETORN_VERIFICAR_TABLESPACE_IND=POST_VERIFICAR_TABLESPACE_IND
GOTO VERIFICAR_TABLESPACE_IND
:POST_VERIFICAR_TABLESPACE_IND:

# Es revisa si la base de dades te activada la opcio de particionament 
:PARTICIONAMENT:
export RETORN_VERIFICAR_BD_PARTICIONADA=REVISAR_PARTICIONAMENT
GOTO VERIFICAR_BD_PARTICIONADA

:REVISAR_PARTICIONAMENT:
if [ "${BD_PARTICIONADA}" == "TRUE" ];
then
	GOTO PREGUNTAR_PARTICIONAMENT
fi

:POST_REVISAR_PARTICIONAMENT:
if [ "${PARTICIONAR_TAULES}" == "S"];
then 
	export SCRIPT_CREACIO_TAULES=${PATH_ACTUAL}/scripts/taules/scripts_taules_particionades.sql
else
	export SCRIPT_CREACIO_TAULES=${PATH_ACTUAL}/scripts/taules/scripts_taules_no_particionades.sql
fi

# Preparacio de l' usuari 
echo "Preparant usuari de base de dades ${USUARI_BD}"
${DIR_SQLPLUS} -s -l ${CONEXIO_BD} @${SCRIPT_CREACIO_USUARI} ${USUARI_BD} ${TABLESPACE_TAULES} >> ${LOG_INSTALACIO}

# Directoris de BD
echo "Creant directoris de Base de dades"
${DIR_SQLPLUS} -s -l ${CONEXIO_BD} @${SCRIPT_DIR_BD} ${PATH_ACTUAL}/Dades ${USUARI_BD} >> ${LOG_INSTALACIO}

# Creacio de taules  
echo "Creant taules"
${DIR_SQLPLUS} -s -l ${CONEXIO_BD} @${SCRIPT_CREACIO_TAULES} ${PATH_ACTUAL/scripts/taules ${USUARI_BD} ${TABLESPACE_TAULES} ${TABLESPACE_INDEXS} >> ${LOG_INSTALACIO}

# Creacio de vistes
echo "Creant vistes "
${DIR_SQLPLUS} -s -l ${CONEXIO_BD} @${SCRIPT_CREACIO_VISTES} ${PATH_ACTUAL}/scripts/vistes ${USUARI_BD} >> ${LOG_INSTALACIO}

# Creacio de disparadors  
echo "Creant disparadors"
${DIR_SQLPLUS} -s -l ${CONEXIO_BD} @${SCRIPT_CREACIO_DISPARADORS} ${PATH_ACTUAL}/scripts/disparadors ${USUARI_BD} >> ${LOG_INSTALACIO}

# Creacio de paquets 
echo "Carregant codi PL/SQL"
${DIR_SQLPLUS} -s -l ${CONEXIO_BD} @${SCRIPT_CREACIO_PAQUETS} ${PATH_ACTUAL}/scripts/paquets ${USUARI_BD} >> ${LOG_INSTALACIO}
${DIR_SQLPLUS} -s -l ${CONEXIO_BD} @${SCRIPT_RECOMPILAR_PAQUETS} ${USUARI_BD} >> ${LOG_INSTALACIO}
${DIR_SQLPLUS} -s -l ${CONEXIO_BD} @${SCRIPT_RECOMPILAR_PAQUETS} ${USUARI_BD} >> ${LOG_INSTALACIO}
${DIR_SQLPLUS} -s -l ${CONEXIO_BD} @${SCRIPT_RECOMPILAR_PAQUETS} ${USUARI_BD} >> ${LOG_INSTALACIO}

# Carregant parametres
echo "Carregant parametres"
${DIR_SQLPLUS} -s -l ${CONEXIO_BD} @${SCRIPT_PARAMETRES} ${PATH_ACTUAL}/scripts/parametres ${USUARI_BD} >> ${LOG_INSTALACIO}

GOTO SORTIR

:SQLPLUS_NO_EXISTEIX:
echo "No s' ha trobat el SQLPLUS en la ruta especificada. ORACLE_HOME/bin=${ORACLE_HOME}/bin"
GOTO SORTIR

:PREGUNTAR_PARTICIONAMENT:
echo "Vol aplicar particionament a les taules? (S/N):"
read PARTICIONAR_TAULES
if [ "${PARTICIONAR_TAULES}" == "S" ];
then
	GOTO POST_REVISAR_PARTICIONAMENT
fi
if [ "${PARTICIONAR_TAULES}" == "N" ];
then
	GOTO POST_REVISAR_PARTICIONAMENT
fi

GOTO REVISAR_PARTICIONAMENT

:VERIFICAR_BD_PARTICIONADA:
${DIR_SQLPLUS} -s -l ${CONEXIO_BD} @${RUTA_SELECTS}/bd_particionada.sql ${RUTA_SELECTS}/bd_particionada.out >> ${LOG_INSTALACIO}
if [ -f ${RUTA_SELECTS}/bd_particionada.out ];
then
	BD_PARTICIONADA=$(<${RUTA_SELECTS}/bd_particionada.out)
	rm ${RUTA_SELECTS}/bd_particionada.out
fi
GOTO ${RETORN_VERIFICAR_BD_PARTICIONADA}

:VERIFICAR_TABLESPACE_TAB:
echo "Indiqui el nom del tablespace on han de residir les taules a crear: "
read TABLESPACE_TAULES
if [ "${TABLESPACE_TAULES}" == ""];
then
	GOTO VERIFICAR_TABLESPACE_TAB
fi
echo "Verificant existencia del tablespace ${TABLESPACE_TAULES}"
${DIR_SQLPLUS} -s -l ${CONEXIO_BD} @${RUTA_SELECTS}/tablespace_ok.sql ${RUTA_SELECTS}/tablespace_ok.out ${TABLESPACE_TAULES} >> ${LOG_INSTALACIO}
if [ -f ${RUTA_SELECTS}/tablespace_ok.out ];
then
	TABLESPACE_TAB_OK=$(<${RUTA_SELECTS}/tablespace_ok.out)
	rm ${RUTA_SELECTS}/tablespace_ok.out
fi

if [ "${TABLESPACE_TAB_OK}" == "S" ];
then
	echo "Validacio superada"
else
	echo "No existeix cap tablespace amb el nom ${TABLESPACE_TAULES}"
	GOTO VERIFICAR_TABLESPACE_TAB
)
GOTO ${RETORN_VERIFICAR_TABLESPACE_TAB}

:VERIFICAR_TABLESPACE_IND:
echo "Indiqui el nom del tablespace on han de residir els indexs a crear: "
read TABLESPACE_INDEXS
if [ "${TABLESPACE_INDEXS}" == "" ];
then
	GOTO VERIFICAR_TABLESPACE_IND
fi
echo "Verificant existencia del tablespace ${TABLESPACE_INDEXS}"
${DIR_SQLPLUS} -s -l ${CONEXIO_BD} @${RUTA_SELECTS}/tablespace_ok.sql ${RUTA_SELECTS}/tablespace_ok.out ${TABLESPACE_INDEXS} >> ${LOG_INSTALACIO}
if [ -f ${RUTA_SELECTS}/tablespace_ok.out ];
then
	TABLESPACE_IND_OK=$(<${RUTA_SELECTS}/tablespace_ok.out)
	rm ${RUTA_SELECTS}/tablespace_ok.out
fi

if [ "${TABLESPACE_IND_OK}" == "S" ];
then 
	echo "Validacio superada"
else
	echo "No existeix cap tablespace amb el nom ${TABLESPACE_INDEXS}"
	GOTO VERIFICAR_TABLESPACE_IND
fi
GOTO ${RETORN_VERIFICAR_TABLESPACE_IND}

:PROPIETARI_OBJECTES:
echo "Indiqui l' usuari en el que s' han de crear els objectes: "
read USUARI_BD
if [ "$USUARI_BD" == ""  ];
then
	GOTO PROPIETARI_OBJECTES
fi
echo "Verificant existencia de l' usuari ${USUARI_BD}"
${DIR_SQLPLUS} -s -l ${CONEXIO_BD} @${RUTA_SELECTS}/usuari_bd_ok.sql ${RUTA_SELECTS}/usuari_bd_ok.out ${USUARI_BD} >> ${LOG_INSTALACIO}
if [ -f ${RUTA_SELECTS}/usuari_bd_ok.out ];
then
	USUARI_BD_OK=$(<${RUTA_SELECTS}/usuari_bd_ok.out)
	rm ${RUTA_SELECTS}/usuari_bd_ok.out
fi
if [ "${USUARI_BD_OK}" == "S" ];
then
	echo "Validacio superada"
else
	echo "No existeix cap usuari amb el nom ${USUARI_BD}"
	GOTO PROPIETARI_OBJECTES
fi
GOTO ${RETORN_PROPIETARI_OBJECTES}

:VERIFICAR_CONNEXIO_BD:
echo "Verificant connectivitat amb la base de dades ${SID_BD}"
export CONNEXIO_BD_OK=
${DIR_SQLPLUS} -s -l ${CONEXIO_BD} @${RUTA_SELECTS}/connexio_bd.sql ${RUTA_SELECTS}/connexio_bd.out >> ${LOG_INSTALACIO}
if [ -f ${RUTA_SELECTS}/connexio_bd.out ];
then
	CONNEXIO_BD_OK=$(<${RUTA_SELECTS}/connexio_bd.out)
	rm ${RUTA_SELECTS}/connexio_bd.out
fi

if [ "$CONNEXIO_BD_OK" == "S" ]; 
then
	echo "Validacio superada"
else
	echo "ERROR: Les dades de connexio no son correctes o be la base de dades ${SID_BD} no es accessible."
	export RETORN_PREGUNTAR_CONNEXIO_BD=VERIFICAR_CONNEXIO_BD
	GOTO PREGUNTAR_CONNEXIO_BD
fi
GOTO ${RETORN_VERIFICAR_CONNEXIO_BD}

:PREGUNTAR_CONNEXIO_BD:
echo "Indiqui la connexio a la BD on vol fer la instal.lacio: "
read SID_BD
echo "Introdueixi la contrasenya de l' usuari SYS:"
read SYSTEM_PASSWORD
export CONEXIO_BD=SYS/"${SYSTEM_PASSWORD}"@${SID_BD} AS SYSDBA
GOTO ${RETORN_PREGUNTAR_CONNEXIO_BD}

:SORTIR:
echo "Instal.lacio finalitzada correctament"
