create or replace package TFC_ACA is
  -- Procediment que dna d' alta un registre a la taula TFC_AGR_CIU_ANY 
  -- Parmetres d' entrada:
  --   P_NUMANY => Any per el que es vol buscar els indicadors 
  --   P_IDECIU => Identificador de la ciutat per la que es vol calcular els indicadors de ms i menys vendes 
  --   P_BENNET => Benefici net de la botiga indicada.
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE ALT_ACA(P_NUMANY TFC_AGR_CIU_ANY.INA_NUMANY%TYPE,
                    P_IDECIU TFC_AGR_CIU_ANY.CIU_IDECIU%TYPE,
                    P_BENNET TFC_AGR_CIU_ANY.BENNET%TYPE,
                    RSP      OUT VARCHAR2);

  -- Procediment que modifica un registre a la taula TFC_AGR_CIU_ANY 
  -- Parmetres d' entrada:
  --   P_NUMANY => Any per el que es vol buscar els indicadors 
  --   P_IDECIU => Identificador de la ciutat per la que es vol calcular els indicadors de ms i menys vendes 
  --   P_BENNET => Benefici net de la botiga indicada.
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE MOD_ACA(P_NUMANY TFC_AGR_CIU_ANY.INA_NUMANY%TYPE,
                    P_IDECIU TFC_AGR_CIU_ANY.CIU_IDECIU%TYPE,
                    P_BENNET TFC_AGR_CIU_ANY.BENNET%TYPE,
                    RSP      OUT VARCHAR2);

  -- Procediment que decrementa un registre de la taula TFC_AGR_CIU_ANY  
  -- Parmetres d' entrada:
  --   P_NUMANY     => Any per el que es vol buscar els indicadors 
  --   P_IDECIU     => Identificador de la ciutat per la que es vol calcular els indicadors de ms i menys vendes 
  --   P_BENNET_DEC => Benefici a decrementar 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE SUP_ACA(P_NUMANY     TFC_AGR_CIU_ANY.INA_NUMANY%TYPE,
                    P_IDECIU     TFC_AGR_CIU_ANY.CIU_IDECIU%TYPE,
                    P_BENNET_DEC TFC_AGR_CIU_ANY.BENNET%TYPE,
                    RSP          OUT VARCHAR2);

  -- Procediment que elimina un registre de la taula TFC_AGR_CIU_ANY  
  -- Parmetres d' entrada:
  --   P_NUMANY => Any per el que es vol buscar els indicadors 
  --   P_IDECIU => Identificador de la ciutat per la que es vol calcular els indicadors de ms i menys vendes 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.                    
  PROCEDURE SUP_ACA(P_NUMANY TFC_AGR_CIU_ANY.INA_NUMANY%TYPE,
                    P_IDECIU TFC_AGR_CIU_ANY.CIU_IDECIU%TYPE,
                    RSP      OUT VARCHAR2);
                    
  -- Procediment que recalcula els indicadors de vendes per ciutat i any a la taula TFC_AGR_CIU_ANY 
  -- Parmetres d' entrada:
  --   P_NUMANY => Any per el que es vol buscar els indicadors                                  
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE RECALCULAR_IND_VEN(P_NUMANY TFC_AGR_CIU_ANY.INA_NUMANY%TYPE,
                               RSP      OUT VARCHAR2);
end TFC_ACA;
/
create or replace package body TFC_ACA is
  AnyTractat TFC_AGR_BOT_ANY.INA_NUMANY%Type;

  -- Variable global privada que contindr el benefici net mnim. S' utilitza per millorar el rendiment i evitar clculs. 
  Benefici_Net_Minim TFC_AGR_CIU_ANY.BENNET%type;
  -- Variable global privada que contindr el benefici net mxim. S' utilitza per millorar el rendiment i evitar clculs.
  Benefici_Net_Maxim TFC_AGR_CIU_ANY.BENNET%type;

  -- Variable global privada que contindr el benefici net mnim histric. S' utilitza per millorar el rendiment i evitar clculs. 
  Benefici_Net_Minim_HIS TFC_AGR_CIU_ANY.BENNET%type;
  -- Variable global privada que contindr el benefici net mxim histric. S' utilitza per millorar el rendiment i evitar clculs.
  Benefici_Net_Maxim_HIS TFC_AGR_CIU_ANY.BENNET%type;


  -- Funci que retorna el benefici net mnim de l' any passat com a parmetre.
  -- Parmetres d' entrada:
  --   P_NUMANY => Any per el que es vol buscar el benefici net mnim.
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  -- Retorn de la funci: Benefici net mnim de l' any passat com a parmetre 
  FUNCTION CALC_BENNET_MIN(P_NUMANY TFC_AGR_CIU_ANY.INA_NUMANY%TYPE,
                           RSP      OUT VARCHAR2) RETURN TFC_AGR_CIU_ANY.BENNET%type IS
    vBenNet TFC_AGR_CIU_ANY.BENNET%type;
  BEGIN
    -- El benefini net de les botigues i any est calculat en la taula d' agregat TFC_AGR_CIU_ANY 
    Select min(BENNET)
      into vBenNet 
      from TFC_AGR_CIU_ANY aca
      where aca.INA_NUMANY = P_NUMANY and 
            aca.FLGVEN = 'S';
      
    RSP := TFC_UTL.RSP_OK;
    return nvl(vBenNet, 0);
  EXCEPTION -- Tractament d' excepcions 
    WHEN OTHERS THEN -- Si es produeix un error no controlat es retorna el codi i el missatge.
      RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
  END;

  -- Funci que retorna el benefici net mnim histric de l' any passat com a parmetre.
  -- Parmetres d' entrada:
  --   P_NUMANY => Any per el que es vol buscar el benefici net mnim.
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  -- Retorn de la funci: Benefici net mnim histric de l' any passat com a parmetre 
  FUNCTION CALC_BENNET_MIN_HIS(P_NUMANY TFC_AGR_CIU_ANY.INA_NUMANY%TYPE,
                               RSP      OUT VARCHAR2) RETURN TFC_AGR_CIU_ANY.BENNET_HIS%type IS
    vBenNet TFC_AGR_CIU_ANY.BENNET_HIS%type;
  BEGIN
    -- El benefini net de les botigues i any est calculat en la taula d' agregat TFC_AGR_CIU_ANY 
    Select min(BENNET_HIS)
      into vBenNet 
      from TFC_AGR_CIU_ANY aca
      where aca.INA_NUMANY = P_NUMANY;
      
    RSP := TFC_UTL.RSP_OK;
    return nvl(vBenNet, 0);
  EXCEPTION -- Tractament d' excepcions 
    WHEN OTHERS THEN -- Si es produeix un error no controlat es retorna el codi i el missatge.
      RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
  END;

  -- Funci que retorna el benefici net mxim de l' any passat com a parmetre.
  -- Parmetres d' entrada:
  --   P_NUMANY => Any per el que es vol buscar el benefici net mxim.
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  -- Retorn de la funci: Benefici net mxim de l' any passat com a parmetre 
  FUNCTION CALC_BENNET_MAX(P_NUMANY TFC_AGR_CIU_ANY.INA_NUMANY%TYPE,
                           RSP      OUT VARCHAR2) RETURN TFC_AGR_CIU_ANY.BENNET%type IS 
    vBenNet TFC_AGR_CIU_ANY.BENNET%type;
  BEGIN
    -- El benefini net de les botigues i any est calculat en la taula d' agregat TFC_AGR_CIU_ANY 
    Select max(BENNET)
      into vBenNet 
      from TFC_AGR_CIU_ANY aca
      where aca.INA_NUMANY = P_NUMANY and 
            aca.FLGVEN     = 'S';

    RSP := TFC_UTL.RSP_OK;
    return nvl(vBenNet, 0);
  EXCEPTION -- Tractament d' excepcions
    WHEN OTHERS THEN -- Si es produeix un error no controlat es retorna el codi i el missatge.
      RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
  END;

  -- Funci que retorna el benefici net mxim histric de l' any passat com a parmetre.
  -- Parmetres d' entrada:
  --   P_NUMANY => Any per el que es vol buscar el benefici net mxim.
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  -- Retorn de la funci: Benefici net mxim histric de l' any passat com a parmetre 
  FUNCTION CALC_BENNET_MAX_HIS(P_NUMANY TFC_AGR_CIU_ANY.INA_NUMANY%TYPE,
                               RSP      OUT VARCHAR2) RETURN TFC_AGR_CIU_ANY.BENNET_HIS%type IS 
    vBenNet TFC_AGR_CIU_ANY.BENNET_HIS%type;
  BEGIN
    -- El benefini net de les botigues i any est calculat en la taula d' agregat TFC_AGR_CIU_ANY 
    Select max(BENNET_HIS)
      into vBenNet 
      from TFC_AGR_CIU_ANY aca
      where aca.INA_NUMANY = P_NUMANY;

    RSP := TFC_UTL.RSP_OK;
    return nvl(vBenNet, 0);
  EXCEPTION -- Tractament d' excepcions
    WHEN OTHERS THEN -- Si es produeix un error no controlat es retorna el codi i el missatge.
      RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
  END;

  -- Procediment que seteja la variable global privada Benefici_Net_Minim 
  -- Parmetres d' entrada:
  --   P_BENNET => Benefici net a setejar.
  PROCEDURE SET_BENNET_MIN(P_BENNET TFC_AGR_CIU_ANY.BENNET%type) IS
  BEGIN
    Benefici_Net_Minim := Nvl(P_BENNET, 0);
  END;
  
  -- Procediment que seteja la variable global privada Benefici_Net_Maxim
  -- Parmetres d' entrada:
  --   P_BENNET => Benefici net a setejar.
  PROCEDURE SET_BENNET_MAX(P_BENNET TFC_AGR_CIU_ANY.BENNET%type) IS
  BEGIN
    Benefici_Net_Maxim := Nvl(P_BENNET, 0);
  END;

  -- Procediment que seteja la variable global privada Benefici_Net_Minim_HIS
  -- Parmetres d' entrada:
  --   P_BENNET => Benefici net a setejar.
  PROCEDURE SET_BENNET_MIN_HIS(P_BENNET TFC_AGR_CIU_ANY.BENNET_HIS%type) IS
  BEGIN
    Benefici_Net_Minim_HIS := Nvl(P_BENNET, 0);
  END;
  
  -- Procediment que seteja la variable global privada Benefici_Net_Maxim_HIS
  -- Parmetres d' entrada:
  --   P_BENNET => Benefici net a setejar.
  PROCEDURE SET_BENNET_MAX(P_BENNET TFC_AGR_CIU_ANY.BENNET_HIS%type) IS
  BEGIN
    Benefici_Net_Maxim_HIS := Nvl(P_BENNET, 0);
  END;

  -- Procediment que seteja les globals privades de benefici net mnim i mxim a partir 
  -- de la informaci de la taula d' agregats TFC_AGR_CIU_ANY 
  -- Parmetres d' entrada:
  --   P_NUMANY => Any per el que es vol buscar el benefici net mnim i mxim.
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE CALCULAR_BENNET_MIN_MAX(P_NUMANY TFC_AGR_CIU_ANY.INA_NUMANY%TYPE,
                                    RSP      OUT VARCHAR2) IS 
  BEGIN
    RSP := TFC_UTL.RSP_OK;
    
    -- Si no s' ha calculat prviament, s' informa el valor del benefini net mnim 
    If ((Benefici_Net_Minim is null) or (AnyTractat <> P_NUMANY)) then
      Benefici_Net_Minim := CALC_BENNET_MIN(P_NUMANY, RSP);
      AnyTractat := P_NUMANY;
    end if;

    -- Si no s' ha produt cap error i no s' ha calculat prviament el benefici net mxim, s' informa la global de benefini net mxim 
    If ((RSP=TFC_UTL.RSP_OK) and ((Benefici_Net_Maxim is null) or (AnyTractat <> P_NUMANY))) then
      Benefici_Net_Maxim := CALC_BENNET_MAX(P_NUMANY, RSP);
      AnyTractat := P_NUMANY;
    end if;
  EXCEPTION -- Tractament d' excepcions
    WHEN OTHERS THEN -- Si es produeix un error no controlat es retorna el codi i el missatge.
      RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
  END;

  -- Procediment que seteja les globals privades de benefici net mnim i mxim histrics a partir 
  -- de la informaci de la taula d' agregats TFC_AGR_CIU_ANY 
  -- Parmetres d' entrada:
  --   P_NUMANY => Any per el que es vol buscar el benefici net mnim i mxim.
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE CALCULAR_BENNET_MIN_MAX_HIS(P_NUMANY TFC_AGR_CIU_ANY.INA_NUMANY%TYPE,
                                        RSP      OUT VARCHAR2) IS 
  BEGIN
    RSP := TFC_UTL.RSP_OK;
    
    -- Si no s' ha calculat prviament, s' informa el valor del benefini net mnim histric 
    If ((Benefici_Net_Minim_HIS is null) or (AnyTractat <> P_NUMANY)) then
      Benefici_Net_Minim_HIS := CALC_BENNET_MIN_HIS(P_NUMANY, RSP);
      AnyTractat := P_NUMANY;
    end if;

    -- Si no s' ha produt cap error i no s' ha calculat prviament el benefici net mxim, s' informa la global de benefini net mxim histric
    If ((RSP=TFC_UTL.RSP_OK) and ((Benefici_Net_Maxim_HIS is null) or (AnyTractat <> P_NUMANY))) then
      Benefici_Net_Maxim_HIS := CALC_BENNET_MAX_HIS(P_NUMANY, RSP);
      AnyTractat := P_NUMANY;
    end if;
  EXCEPTION -- Tractament d' excepcions
    WHEN OTHERS THEN -- Si es produeix un error no controlat es retorna el codi i el missatge.
      RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
  END;

  -- Procediment que torna a informar les globals privades de benefici net mnim i mxim a partir 
  -- de la informaci de la taula d' agregats TFC_AGR_CIU_ANY 
  -- Parmetres d' entrada:
  --   P_NUMANY => Any per el que es vol buscar el benefici net mnim i mxim.
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE RECALCULAR_BENNET_MIN_MAX(P_NUMANY TFC_AGR_CIU_ANY.INA_NUMANY%TYPE,
                                      RSP      OUT VARCHAR2) IS 
  BEGIN
    Benefici_Net_Minim := CALC_BENNET_MIN(P_NUMANY, RSP);
    -- Si la funci anterior ha generat un error no cas continuar ... 
    If (RSP = TFC_UTL.RSP_OK) then 
      Benefici_Net_Maxim := CALC_BENNET_MAX(P_NUMANY, RSP);
    end if;
  EXCEPTION -- Tractament d' excepcions
    WHEN OTHERS THEN -- Si es produeix un error no controlat es retorna el codi i el missatge.
      RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
  END;

  -- Procediment que torna a informar les globals privades de benefici net mnim i mxim histrics a partir 
  -- de la informaci de la taula d' agregats TFC_AGR_CIU_ANY 
  -- Parmetres d' entrada:
  --   P_NUMANY => Any per el que es vol buscar el benefici net mnim i mxim.
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE RECALCULAR_BENNET_MIN_MAX_HIS(P_NUMANY TFC_AGR_CIU_ANY.INA_NUMANY%TYPE,
                                          RSP      OUT VARCHAR2) IS 
  BEGIN
    Benefici_Net_Minim_HIS := CALC_BENNET_MIN_HIS(P_NUMANY, RSP);
    -- Si la funci anterior ha generat un error no cas continuar ... 
    If (RSP = TFC_UTL.RSP_OK) then 
      Benefici_Net_Maxim_HIS := CALC_BENNET_MAX_HIS(P_NUMANY, RSP);
    end if;
  EXCEPTION -- Tractament d' excepcions
    WHEN OTHERS THEN -- Si es produeix un error no controlat es retorna el codi i el missatge.
      RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
  END;

  -- Procediment que calcula els indicador de ms i menys vendes .
  -- El resultat s' informa a la taula TFC_AGR_CIU_ANY
  -- Parmetres d' entrada:
  --   P_NUMANY     => Any per el que es vol buscar els indicadors 
  --   P_IDEBOT     => Identificador de la botiga per la que es vol calcular els indicadors de ms i menys vendes 
  --   P_BENNET_AGR => Benefici net de la botiga indicada.
  --   P_FLGMEN     => Indicador de menys vendes associat a la botiga indicada  
  --   P_FLGMES     => Indicador de ms vendes associat a la botiga indicada 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE CALC_IND_VEN(P_NUMANY     TFC_AGR_CIU_ANY.INA_NUMANY%TYPE,
                         P_BENNET_AGR TFC_AGR_CIU_ANY.BENNET%TYPE,
                         P_FLGMEN     TFC_AGR_CIU_ANY.FLGMEN%TYPE,
                         P_FLGMES     TFC_AGR_CIU_ANY.FLGMES%TYPE,
                         RSP          OUT VARCHAR2) IS
  BEGIN
    RECALCULAR_BENNET_MIN_MAX(P_NUMANY, RSP);

    If (RSP = TFC_UTL.RSP_OK) then   
      If (
          (P_BENNET_AGR >= Benefici_Net_Maxim) or 
          (P_FLGMES = 'S') or 
          (P_BENNET_AGR <= Benefici_Net_Minim) or
          (P_FLGMEN = 'S')
         ) 
      then 
        Update TFC_AGR_CIU_ANY aca
           set aca.FLGMES = Decode (aca.BENNET, Benefici_Net_Maxim, 'S', 'N'),
               aca.FLGMEN = Decode (aca.BENNET, Benefici_Net_Minim, 'S', 'N')
           where aca.INA_NUMANY = P_NUMANY and 
                 (
                  (aca.FLGMEN = 'S') or  
                  (aca.BENNET = Benefici_Net_Minim) or
                  (aca.FLGMES = 'S') or  
                  (aca.BENNET = Benefici_Net_Maxim)
                 ) and 
                 aca.FLGVEN = 'S';
      end if;
    end if;
  EXCEPTION -- Tractament d' excepcions 
    WHEN OTHERS THEN -- Si es produeix un error no controlat es retorna el codi i el missatge.
      RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
  END;

  -- Procediment que calcula els indicador de ms i menys vendes .
  -- El resultat s' informa a la taula TFC_AGR_CIU_ANY
  -- Parmetres d' entrada:
  --   P_NUMANY     => Any per el que es vol buscar els indicadors 
  --   P_IDEBOT     => Identificador de la botiga per la que es vol calcular els indicadors de ms i menys vendes 
  --   P_BENNET_AGR => Benefici net de la botiga indicada.
  --   P_FLGMEN     => Indicador de menys vendes associat a la botiga indicada  
  --   P_FLGMES     => Indicador de ms vendes associat a la botiga indicada 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE CALC_IND_VEN_HIS(P_NUMANY     TFC_AGR_CIU_ANY.INA_NUMANY%TYPE,
                             P_BENNET_AGR TFC_AGR_CIU_ANY.BENNET_HIS%TYPE,
                             P_FLGMEN_HIS     TFC_AGR_CIU_ANY.FLGMEN_HIS%TYPE,
                             P_FLGMES_HIS     TFC_AGR_CIU_ANY.FLGMES_HIS%TYPE,
                             RSP          OUT VARCHAR2) IS
  BEGIN
    RECALCULAR_BENNET_MIN_MAX_HIS(P_NUMANY, RSP);

    If (RSP = TFC_UTL.RSP_OK) then   
      If (
          (P_BENNET_AGR >= Benefici_Net_Maxim_HIS) or 
          (P_FLGMES_HIS = 'S') or 
          (P_BENNET_AGR <= Benefici_Net_Minim_HIS) or
          (P_FLGMEN_HIS = 'S')
         ) 
      then 
        Update TFC_AGR_CIU_ANY aca
           set aca.FLGMES_HIS = Decode (aca.BENNET_HIS, Benefici_Net_Maxim_HIS, 'S', 'N'),
               aca.FLGMEN_HIS = Decode (aca.BENNET_HIS, Benefici_Net_Minim_HIS, 'S', 'N')
           where aca.INA_NUMANY = P_NUMANY and 
                 (
                  (aca.FLGMEN_HIS = 'S') or  
                  (aca.BENNET_HIS = Benefici_Net_Minim_HIS) or
                  (aca.FLGMES_HIS = 'S') or  
                  (aca.BENNET_HIS = Benefici_Net_Maxim_HIS)
                 );
      end if;
    end if;
  EXCEPTION -- Tractament d' excepcions 
    WHEN OTHERS THEN -- Si es produeix un error no controlat es retorna el codi i el missatge.
      RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
  END;

  -- Procediment que dna d' alta un registre a la taula TFC_AGR_CIU_ANY 
  -- Parmetres d' entrada:
  --   P_NUMANY => Any per el que es vol buscar els indicadors 
  --   P_IDECIU => Identificador de la ciutat per la que es vol calcular els indicadors de ms i menys vendes 
  --   P_BENNET => Benefici net de la botiga indicada.
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE ALT_ACA(P_NUMANY TFC_AGR_CIU_ANY.INA_NUMANY%TYPE,
                    P_IDECIU TFC_AGR_CIU_ANY.CIU_IDECIU%TYPE,
                    P_BENNET TFC_AGR_CIU_ANY.BENNET%TYPE,
                    RSP      OUT VARCHAR2) IS
    vBenNet            TFC_AGR_BOT_ANY.BENNET%TYPE;    
    vFlgMen            TFC_AGR_BOT_ANY.FLGMEN%type;
    vFlgMes            TFC_AGR_BOT_ANY.FLGMES%type;
    vBenNet_HIS        TFC_AGR_BOT_ANY.BENNET_HIS%TYPE;    
    vBennet_HIS_Ant    TFC_AGR_BOT_ANY.BENNET_HIS%TYPE;
    vFlgMen_HIS        TFC_AGR_BOT_ANY.FLGMEN_HIS%type;
    vFlgMes_HIS        TFC_AGR_BOT_ANY.FLGMES_HIS%type;
    vNomProces         TFC_RASTRE_PROCES.PROCES%Type := 'TFC_ACA.ALT_ACA';
    vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
    vError_Controlat   exception;
    vProces_Error      varchar2(100);
  BEGIN
    If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
      TFC_RAP.INICIAR_RASTRE(vNomProces);
    end if;
    
    -- Es graven els parmetres a la taula de rastre    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_NUMANY='||P_NUMANY);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_IDECIU='||P_IDECIU);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_BENNET='||P_BENNET);

    -- S' inicialitzen, si cal, les globals de mnim i mxim 
    CALCULAR_BENNET_MIN_MAX(P_NUMANY, RSP);
    If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
      vProces_Error := 'CALCULAR_BENNET_MIN_MAX';
      raise vError_Controlat;
    end if;

    -- S' inicialitzen, si cal, les globals de mnim i mxim histrics 
    CALCULAR_BENNET_MIN_MAX_HIS(P_NUMANY, RSP);
    If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
      vProces_Error := 'CALCULAR_BENNET_MIN_MAX_HIS';
      raise vError_Controlat;
    end if;
    
    -- S' updateja el registre de la taula d' agregats de ciutats per any 
    Update TFC_AGR_CIU_ANY aca
       set aca.BENNET     = aca.BENNET + nvl(P_BENNET, 0),
           aca.BENNET_HIS = aca.BENNET_HIS + nvl(P_BENNET, 0),
           aca.FLGVEN     = 'S'
       where aca.INA_NUMANY = P_NUMANY and 
             aca.CIU_IDECIU = P_IDECIU
       returning aca.BENNET, aca.FLGMEN, aca.FLGMES, aca.BENNET_HIS, aca.FLGMEN_HIS, aca.FLGMES_HIS
       into      vBenNet,    vFlgMen,    vFlgMes,    vBenNet_HIS,    vFlgMen_HIS,    vFlgMes_HIS;
    
    If (sql%RowCount = 0) then -- No s' ha trobat cap registre i per tant s' ha de donar d' alta 
      -- S' inserta el registre 
      Insert into TFC_AGR_CIU_ANY 
        (INA_NUMANY, 
         CIU_IDECIU,
         BENNET,
         BENNET_HIS,
         FLGVEN)
      values
        (P_NUMANY,
         P_IDECIU,
         nvl(P_BENNET,0),
         nvl((Select BENNET_HIS 
                from TFC_AGR_CIU_ANY aca 
                where aca.INA_NUMANY = (P_NUMANY - 1) and 
                      aca.CIU_IDECIU = P_IDECIU), 0) + 
         nvl(P_BENNET,0),
         'S')
       returning BENNET,  FLGMEN,  FLGMES,  BENNET_HIS,  FLGMEN_HIS,  FLGMES_HIS
       into      vBenNet, vFlgMen, vFlgMes, vBenNet_HIS, vFlgMen_HIS, vFlgMes_HIS;
    end if;
    
    -- Es calculen els indicadors de mnim i mxim a partir de les dades del registre insertat o modificat 
    CALC_IND_VEN(P_NUMANY,
                 vBenNet,
                 vFlgMen,
                 vFlgMes,
                 RSP);
    
    If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
      vProces_Error := 'CALC_IND_VEN';
      raise vError_Controlat;
    end if;

    -- Es calculen els indicadors de mnim i mxim histrics a partir de les dades del registre insertat o modificat 
    CALC_IND_VEN_HIS(P_NUMANY,
                     vBenNet_HIS,
                     vFlgMen_HIS,
                     vFlgMes_HIS,
                     RSP);
    
    If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
      vProces_Error := 'CALC_IND_VEN_HIS';
      raise vError_Controlat;
    end if;
    
    -- Ens guardem el benefici net histric acumulat fins l' any tractat
    vBennet_HIS_Ant := vBennet_HIS;    
    -- En cas necessari s' hauran de recalcular els indicadors histrics futurs 
    For d in (Select INA.NUMANY 
                from TFC_INDICADORS_ANY ina
                where ina.NUMANY > P_NUMANY
                order by numany) loop

      -- S' inicialitzen, si cal, les globals de mnim i mxim histrics 
      CALCULAR_BENNET_MIN_MAX_HIS(d.numany, RSP);
      If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
        vProces_Error := 'CALCULAR_BENNET_MIN_MAX_HIS (BENNET_HIS futur)';
        raise vError_Controlat;
      end if;
      
      Update TFC_AGR_CIU_ANY aca 
         set aca.Bennet_His = aca.BENNET_HIS + nvl(P_BENNET, 0)
         where aca.Ina_Numany = d.numany and 
               aca.CIU_IDECIU = P_IDECIU
         returning aca.Bennet_His, aca.FlgMen_His, aca.FlgMes_his 
              into vBenNet_His,    vFlgMen_His,    vFlgMes_His;

      -- Si no existeix el registre...      
      If (sql%RowCount = 0) then
        Insert into TFC_AGR_CIU_ANY 
          (INA_NUMANY, 
           CIU_IDECIU,
           BENNET,
           BENNET_HIS,
           FLGVEN)
        values
          (d.NUMANY,
           P_IDECIU,
           0,
           vBennet_HIS_Ant,
           'N')
         returning BENNET_HIS,  FLGMEN_HIS,  FLGMES_HIS
         into      vBenNet_HIS, vFlgMen_HIS, vFlgMes_HIS;
      end if;

      -- Es calculen els indicadors de mnim i mxim histrics a partir de les dades del registre insertat o modificat 
      CALC_IND_VEN_HIS(d.NUMANY,
                       vBenNet_HIS,
                       vFlgMen_HIS,
                       vFlgMes_HIS,
                       RSP);
    
      If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
        vProces_Error := 'CALC_IND_VEN_HIS (BENNET_HIS futur)';
        raise vError_Controlat;
      end if;
      
      -- Es recalculen els indicadors anuals...
      TFC_INA.CALC_INA(d.NUMANY, RSP);
      If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
        vProces_Error := 'TFC_INA.CALC_INA ('||d.NUMANY||',RSP) futur';
        raise vError_Controlat;
      end if;
      
      -- Ens guardem el benefici net histric acumulat fins l' any tractat
      vBennet_HIS_Ant := vBennet_HIS;
    end loop;
    
    -- Arribats a aquest punt, tot ha anat b.
    If (RSP is null) then 
      RSP := TFC_UTL.RSP_OK;
    end if;
                 
    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);

    If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
      If (RSP = TFC_UTL.RSP_OK) then 
        TFC_RAP.FINALITZAR_RASTRE_OK;
      else
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    end if;
  EXCEPTION -- Tractament d' excepcions 
    WHEN others THEN 
      If (vProces_Error is null) then -- Si vProces_Error es null, es tracta d' un error no controlat 
        RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
      else
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Error executant el procs '||vProces_Error||':');
      end if;
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
  END;
  
  -- Procediment que modifica un registre a la taula TFC_AGR_CIU_ANY 
  -- Parmetres d' entrada:
  --   P_NUMANY => Any per el que es vol buscar els indicadors 
  --   P_IDECIU => Identificador de la ciutat per la que es vol calcular els indicadors de ms i menys vendes 
  --   P_BENNET => Benefici net de la botiga indicada.
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE MOD_ACA(P_NUMANY TFC_AGR_CIU_ANY.INA_NUMANY%TYPE,
                    P_IDECIU TFC_AGR_CIU_ANY.CIU_IDECIU%TYPE,
                    P_BENNET TFC_AGR_CIU_ANY.BENNET%TYPE,
                    RSP      OUT VARCHAR2) IS
    vBenNet            TFC_AGR_BOT_ANY.BENNET%TYPE;    
    vFlgMen            TFC_AGR_BOT_ANY.FLGMEN%type;
    vFlgMes            TFC_AGR_BOT_ANY.FLGMES%type;
    vBenNet_HIS        TFC_AGR_BOT_ANY.BENNET_HIS%TYPE;    
    vFlgMen_HIS        TFC_AGR_BOT_ANY.FLGMEN_HIS%type;
    vFlgMes_HIS        TFC_AGR_BOT_ANY.FLGMES_HIS%type;
    vNomProces         TFC_RASTRE_PROCES.PROCES%Type := 'TFC_ACA.MOD_ACA';
    vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
    vError_Controlat   exception;
    vProces_Error      varchar2(100);
  BEGIN
    If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
      TFC_RAP.INICIAR_RASTRE(vNomProces);
    end if;
    
    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_NUMANY='||P_NUMANY);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_IDECIU='||P_IDECIU);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_BENNET='||P_BENNET);
    
    -- S' inicialitzen si cal les globals de mnim i mxim 
    CALCULAR_BENNET_MIN_MAX(P_NUMANY, RSP);
    
    If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
      vProces_Error := 'CALCULAR_BENNET_MIN_MAX';
      raise vError_Controlat;
    end if;

    -- S' inicialitzen si cal les globals de mnim i mxim histrics 
    CALCULAR_BENNET_MIN_MAX_HIS(P_NUMANY, RSP);
    
    If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
      vProces_Error := 'CALCULAR_BENNET_MIN_MAX_HIS';
      raise vError_Controlat;
    end if;
    
    -- S' updateja el registre de la taula d' agregats de ciutats per any 
    Update TFC_AGR_CIU_ANY aca
       set aca.BENNET = aca.BENNET + nvl(P_BENNET, 0)
       where aca.INA_NUMANY = P_NUMANY and 
             aca.CIU_IDECIU = P_IDECIU 
       returning aca.BENNET, aca.FLGMEN, aca.FLGMES, aca.BENNET_HIS, aca.FLGMEN_HIS, aca.FLGMES_HIS
       into      vBenNet,    vFlgMen,    vFlgMes,    vBenNet_HIS,    vFlgMen_HIS,    vFlgMes_HIS;

    -- Es calculen els indicadors de mnim i mxim a partir de les dades del registre insertat o modificat 
    CALC_IND_VEN(P_NUMANY,
                 vBenNet,
                 vFlgMen,
                 vFlgMes,
                 RSP);
    
    If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
      vProces_Error := 'CALC_IND_VEN';
      raise vError_Controlat;
    end if;

    -- Es calculen els indicadors de mnim i mxim histrics a partir de les dades del registre insertat o modificat 
    CALC_IND_VEN_HIS(P_NUMANY,
                     vBenNet_HIS,
                     vFlgMen_HIS,
                     vFlgMes_HIS,
                     RSP);
    
    If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
      vProces_Error := 'CALC_IND_VEN_HIS';
      raise vError_Controlat;
    end if;

    -- Si el benefici net histric s 0 s' elimina el registre de la taula d' agregats 
    If (vBenNet_HIS = 0) then 
      SUP_ACA(P_NUMANY,
              P_BENNET,
              RSP);              
      
      If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
        vProces_Error := 'SUP_ACA';
        Raise vError_Controlat;
      end if;
    else
      RSP := TFC_UTL.RSP_OK;
    end if;
    
    -- En cas necessari s' hauran de recalcular els indicadors histrics futurs 
    For d in (Select INA.NUMANY 
                from TFC_INDICADORS_ANY ina
                where ina.NUMANY > P_NUMANY
                order by numany) loop

      -- S' inicialitzen, si cal, les globals de mnim i mxim histrics 
      CALCULAR_BENNET_MIN_MAX_HIS(d.numany, RSP);
      If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
        vProces_Error := 'CALCULAR_BENNET_MIN_MAX_HIS (BENNET_HIS futur)';
        raise vError_Controlat;
      end if;
      
      Update TFC_AGR_CIU_ANY aca 
         set aca.Bennet_His = aca.BENNET_HIS + nvl(P_BENNET, 0)
         where aca.Ina_Numany = d.numany and 
               aca.CIU_IDECIU = P_IDECIU
         returning aca.Bennet_His, aca.FlgMen_His, aca.FlgMes_his 
              into vBenNet_His,    vFlgMen_His,    vFlgMes_His;

      -- Es calculen els indicadors de mnim i mxim histrics a partir de les dades del registre insertat o modificat 
      CALC_IND_VEN_HIS(d.NUMANY,
                       vBenNet_HIS,
                       vFlgMen_HIS,
                       vFlgMes_HIS,
                       RSP);
    
      If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
        vProces_Error := 'CALC_IND_VEN_HIS (BENNET_HIS futur)';
        raise vError_Controlat;
      end if;
      
      -- Es recalculen els indicadors anuals...
      TFC_INA.CALC_INA(d.NUMANY, RSP);
      If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
        vProces_Error := 'TFC_INA.CALC_INA ('||d.NUMANY||',RSP) futur';
        raise vError_Controlat;
      end if;
    end loop;
    
    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);
    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
    If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
      If (RSP = TFC_UTL.RSP_OK) then 
        TFC_RAP.FINALITZAR_RASTRE_OK;
      else
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    end if;
    
  EXCEPTION -- Tractament d' excepcions 
    WHEN others THEN 
      If (vProces_Error is null) then -- Si vProces_Error es null, es tracta d' un error no controlat
        RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
      else
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Error executant el procs '||vProces_Error||':');
      end if;
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
  END;

  -- Procediment que decrementa un registre de la taula TFC_AGR_CIU_ANY  
  -- Parmetres d' entrada:
  --   P_NUMANY     => Any per el que es vol buscar els indicadors 
  --   P_IDECIU     => Identificador de la ciutat per la que es vol calcular els indicadors de ms i menys vendes 
  --   P_BENNET_DEC => Benefici a decrementar 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE SUP_ACA(P_NUMANY     TFC_AGR_CIU_ANY.INA_NUMANY%TYPE,
                    P_IDECIU     TFC_AGR_CIU_ANY.CIU_IDECIU%TYPE,
                    P_BENNET_DEC TFC_AGR_CIU_ANY.BENNET%TYPE,
                    RSP          OUT VARCHAR2) IS
    vNomProces TFC_RASTRE_PROCES.PROCES%Type := 'TFC_ACA.SUP_ACA';
    vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
    vError_Controlat   exception;
    vProces_Error      varchar2(100);
  BEGIN
    If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
      TFC_RAP.INICIAR_RASTRE(vNomProces);
    end if;
    
    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_NUMANY='||P_NUMANY);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_IDECIU='||P_IDECIU);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_BENNET_DEC='||P_BENNET_DEC);
    
    MOD_ACA(P_NUMANY,
            P_IDECIU,
            P_BENNET_DEC,
            RSP);
    
    If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
      vProces_Error:= 'MOD_ACA';
      raise vError_Controlat;
    end if;
            
    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);
    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
    If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
      If (RSP = TFC_UTL.RSP_OK) then 
        TFC_RAP.FINALITZAR_RASTRE_OK;
      else
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    end if;
  EXCEPTION -- Tractament d' excepcions 
    WHEN others THEN 
      If (vProces_Error is null) then -- Si vProces_Error es null, es tracta d' un error no controlat
        RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
      else
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Error executant el procs '||vProces_Error||':');
      end if;
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
  END;
  
  -- Procediment que elimina un registre de la taula TFC_AGR_CIU_ANY  
  -- Parmetres d' entrada:
  --   P_NUMANY => Any per el que es vol buscar els indicadors 
  --   P_IDECIU => Identificador de la ciutat per la que es vol calcular els indicadors de ms i menys vendes 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.                   
  PROCEDURE SUP_ACA(P_NUMANY TFC_AGR_CIU_ANY.INA_NUMANY%TYPE,
                    P_IDECIU TFC_AGR_CIU_ANY.CIU_IDECIU%TYPE,
                    RSP      OUT VARCHAR2) IS
    vNomProces TFC_RASTRE_PROCES.PROCES%Type := 'TFC_ACA.SUP_ACA';
    vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
    vBennet            TFC_AGR_BOT_ANY.BENNET%Type;
    vError_Controlat   exception;
    vProces_Error      varchar2(100);
    vBenNet_HIS        TFC_AGR_CIU_ANY.BENNET_HIS%TYPE;    
    vFlgMen_HIS        TFC_AGR_CIU_ANY.FLGMEN_HIS%type;
    vFlgMes_HIS        TFC_AGR_CIU_ANY.FLGMES_HIS%type;
  BEGIN
    If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
      TFC_RAP.INICIAR_RASTRE(vNomProces);
    end if;
    
    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_NUMANY='||P_NUMANY);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_IDECIU='||P_IDECIU);

    Select BENNET
      into vBennet 
      from TFC_AGR_CIU_ANY
      where INA_NUMANY = P_NUMANY and 
            CIU_IDECIU = P_IDECIU;

    Update TFC_AGR_CIU_ANY
       set BENNET = 0,
           FLGVEN = 'N',
           FLGMES = 'N',
           FLGMEN = 'N'
      where INA_NUMANY = P_NUMANY and 
            CIU_IDECIU = P_IDECIU;

    -- Es calculen els indicadors de mnim i mxim a partir de les dades del registre insertat o modificat 
    CALC_IND_VEN(P_NUMANY,
                 0,
                 'S',
                 'S',
                 RSP); 

    -- En cas necessari s' hauran de recalcular els indicadors histrics futurs 
    For d in (Select INA.NUMANY 
                from TFC_INDICADORS_ANY ina
                where ina.NUMANY > P_NUMANY
                order by numany) loop

      -- S' inicialitzen, si cal, les globals de mnim i mxim histrics 
      CALCULAR_BENNET_MIN_MAX_HIS(d.numany, RSP);
      If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
        vProces_Error := 'CALCULAR_BENNET_MIN_MAX_HIS (BENNET_HIS futur)';
        raise vError_Controlat;
      end if;
      
      Update TFC_AGR_CIU_ANY aca 
         set aca.Bennet_His = aca.BENNET_HIS - nvl(vBENNET, 0)
         where aca.Ina_Numany = d.numany and 
               aca.CIU_IDECIU = P_IDECIU
         returning aca.Bennet_His, aca.FlgMen_His, aca.FlgMes_his 
              into vBenNet_His,    vFlgMen_His,    vFlgMes_His;

      -- Es calculen els indicadors de mnim i mxim histrics a partir de les dades del registre insertat o modificat 
      CALC_IND_VEN_HIS(d.NUMANY,
                       vBenNet_HIS,
                       vFlgMen_HIS,
                       vFlgMes_HIS,
                       RSP);
    
      If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
        vProces_Error := 'CALC_IND_VEN_HIS (BENNET_HIS futur)';
        raise vError_Controlat;
      end if;
      
      -- Es recalculen els indicadors anuals...
      TFC_INA.CALC_INA(d.NUMANY, RSP);
      If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
        vProces_Error := 'TFC_INA.CALC_INA ('||d.NUMANY||',RSP) futur';
        raise vError_Controlat;
      end if;
    end loop;

    -- Arribats a aquest punt tot ha anat b 
    If (RSP is null) then 
      RSP := TFC_UTL.RSP_OK;
    end if;

    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);
    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
    If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
      If (RSP = TFC_UTL.RSP_OK) then 
        TFC_RAP.FINALITZAR_RASTRE_OK;
      else
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    end if;
  EXCEPTION -- Tractament d' excepcions 
    WHEN others THEN 
      RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
  END;
  
  -- Procediment que recalcula els indicadors de vendes per ciutat i any a la taula TFC_AGR_CIU_ANY 
  -- Parmetres d' entrada:
  --   P_NUMANY => Any per el que es vol buscar els indicadors                                  
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE RECALCULAR_IND_VEN(P_NUMANY TFC_AGR_CIU_ANY.INA_NUMANY%TYPE,
                               RSP      OUT VARCHAR2) IS
    vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
    vNomProces         TFC_RASTRE_PROCES.PROCES%Type := 'TFC_ACA.RECALCULAR_IND_VEN';
  BEGIN
    If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
      TFC_RAP.INICIAR_RASTRE(vNomProces);
    end if;
    
    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_NUMANY='||P_NUMANY);
    
    -- S' elimina la informaci de l' any passat com a parmetre 
    Delete 
      from TFC_AGR_CIU_ANY aba
      where aba.Ina_Numany = P_NUMANY;
    
    Insert into TFC_AGR_CIU_ANY 
        (INA_NUMANY, 
         CIU_IDECIU,
         BENNET,
         BENNET_HIS,
         FLGMES,
         FLGMEN,
         FLGMES_HIS,
         FLGMEN_HIS,
         FLGVEN)
    Select INA_NUMANY, 
           CIU_IDECIU,
           BENNET, 
           BENNET_ACU,
           'N',
           'N',
           'N',
           'N',
           Decode(COUNT_VEN, 0, 'N', 'S') as FLGVEN
      from (Select P_NUMANY as INA_NUMANY, 
                   bot.CIU_IDECIU, 
                   sum(decode(INA_NUMANY, P_NUMANY, ven.BENNET, 0)) as BENNET,
                   sum(ven.BENNET) as BENNET_ACU,
                   sum(decode(INA_NUMANY, P_NUMANY, 1, 0)) as COUNT_VEN
              from TFC_VENDES ven,
                   TFC_TEMPS  tem,
                   TFC_BOTIGUES bot 
              where ven.TEM_DATA    = tem.DATA and 
                    ven.TEM_HORA    = tem.HORA and 
                    tem.INA_NUMANY <= P_NUMANY and 
                    ven.BOT_IDEBOT  = bot.IDEBOT
              group by P_NUMANY, bot.CIU_IDECIU);

    RECALCULAR_BENNET_MIN_MAX(P_NUMANY, RSP);
    
    If (RSP = TFC_UTL.RSP_OK) then   
      Update TFC_AGR_CIU_ANY aca
         set aca.FLGMES = Decode (aca.BENNET, Benefici_Net_Maxim, 'S', 'N'),
             aca.FLGMEN = Decode (aca.BENNET, Benefici_Net_Minim, 'S', 'N')
         where aca.INA_NUMANY = P_NUMANY and 
               (
                (aca.BENNET = Benefici_Net_Minim) or
                (aca.BENNET = Benefici_Net_Maxim)
               ) and
               aca.FLGVEN = 'S';
    end if;

    RECALCULAR_BENNET_MIN_MAX_HIS(P_NUMANY, RSP);

    If (RSP = TFC_UTL.RSP_OK) then   
      Update TFC_AGR_CIU_ANY aca
         set aca.FLGMES_HIS = Decode (aca.BENNET_HIS, Benefici_Net_Maxim_HIS, 'S', 'N'),
             aca.FLGMEN_HIS = Decode (aca.BENNET_HIS, Benefici_Net_Minim_HIS, 'S', 'N')
         where aca.INA_NUMANY = P_NUMANY and 
               (
                (aca.BENNET_HIS = Benefici_Net_Minim_HIS) or
                (aca.BENNET_HIS = Benefici_Net_Maxim_HIS)
               );
    end if;

    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);
    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
    If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
      If (RSP = TFC_UTL.RSP_OK) then 
        TFC_RAP.FINALITZAR_RASTRE_OK;
      else
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    end if;
  EXCEPTION -- Tractament d' excepcions 
    WHEN others THEN 
      RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
  END;
end TFC_ACA;
/
