create or replace package TFC_AHA is
  -- Procediment que dna d' alta un registre a la taula TFC_AGR_HOR_ANY 
  -- Parmetres d' entrada:
  --   P_NUMANY => Any per el que es vol buscar els indicadors 
  --   P_HORA   => Hora del dia   
  --   P_QNTVEN => Quantitat venuda 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE ALT_AHA(P_NUMANY TFC_AGR_HOR_ANY.INA_NUMANY%TYPE,
                    P_HORA   TFC_AGR_HOR_ANY.HORA%TYPE,
                    P_QNTVEN TFC_AGR_HOR_ANY.QNTVEN%TYPE,
                    RSP      OUT VARCHAR2);

  -- Procediment que modifica un registre a la taula TFC_AGR_HOR_ANY 
  -- Parmetres d' entrada:
  --   P_NUMANY => Any per el que es vol buscar els indicadors 
  --   P_HORA   => Hora del dia   
  --   P_QNTVEN => Quantitat venuda 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE MOD_AHA(P_NUMANY TFC_AGR_HOR_ANY.INA_NUMANY%TYPE,
                    P_HORA   TFC_AGR_HOR_ANY.HORA%TYPE,
                    P_QNTVEN TFC_AGR_HOR_ANY.QNTVEN%TYPE,
                    RSP      OUT VARCHAR2);

  -- Procediment que decrementa la quantitat d' un registre a la taula TFC_AGR_HOR_ANY 
  -- Parmetres d' entrada:
  --   P_NUMANY     => Any per el que es vol buscar els indicadors 
  --   P_HORA       => Hora del dia   
  --   P_QNTVEN_DEC => Quantitat a decrementar  
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE SUP_AHA(P_NUMANY     TFC_AGR_HOR_ANY.INA_NUMANY%TYPE,
                    P_HORA       TFC_AGR_HOR_ANY.HORA%TYPE,
                    P_QNTVEN_DEC TFC_AGR_HOR_ANY.QNTVEN%TYPE,
                    RSP          OUT VARCHAR2);

  -- Procediment que elimina un registre a la taula TFC_AGR_HOR_ANY 
  -- Parmetres d' entrada:
  --   P_NUMANY => Any per el que es vol buscar els indicadors 
  --   P_HORA   => Hora del dia   
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE SUP_AHA(P_NUMANY TFC_AGR_HOR_ANY.INA_NUMANY%TYPE,
                    P_HORA   TFC_AGR_HOR_ANY.HORA%TYPE,
                    RSP      OUT VARCHAR2);

  -- Procediment que recalcula els indicadors de hora del dia i any a la taula TFC_AGR_HOR_ANY 
  -- Parmetres d' entrada:
  --   P_NUMANY => Any per el que es vol buscar els indicadors                                  
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE RECALCULAR_IND_VEN(P_NUMANY TFC_AGR_HOR_ANY.INA_NUMANY%TYPE,
                               RSP      OUT VARCHAR2);
end TFC_AHA;
/
create or replace package body TFC_AHA is
    AnyTractat TFC_AGR_BOT_ANY.INA_NUMANY%Type;

    -- Variable global privada que contindr la quantitat mnima. S' utilitza per millorar el rendiment i evitar clculs.
    Quantitat_Venuda_Minima TFC_AGR_HOR_ANY.QNTVEN%type;
    -- Variable global privada que contindr la quantitat mxima. S' utilitza per millorar el rendiment i evitar clculs.
    Quantitat_Venuda_Maxima TFC_AGR_HOR_ANY.QNTVEN%type;

    -- Variable global privada que contindr la quantitat mnima histrica. S' utilitza per millorar el rendiment i evitar clculs.
    Quantitat_Venuda_Minima_HIS TFC_AGR_HOR_ANY.QNTVEN_HIS%type;
    -- Variable global privada que contindr la quantitat mxima histrica. S' utilitza per millorar el rendiment i evitar clculs.
    Quantitat_Venuda_Maxima_HIS TFC_AGR_HOR_ANY.QNTVEN_HIS%type;

    -- Funci que retorna laquantitat mnima de l' any passat com a parmetre.
    -- Parmetres d' entrada:
    --   P_NUMANY => Any per el que es vol buscar la quantitat mnima.
    -- Parmetres de sortida:
    --   P_RSP    => Indica si l' operaci ha tingut xit o no:
    --     'OK'   => Valor indicatiu que el procs ha anat b.
    --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
    -- Retorn de la funci: Quantitat mnima de l' any passat com a parmetre
    FUNCTION CALC_QNTVEN_MIN(P_NUMANY TFC_AGR_HOR_ANY.INA_NUMANY%TYPE,
                             RSP      OUT VARCHAR2) RETURN TFC_AGR_HOR_ANY.QNTVEN%type IS
      vQntVen TFC_AGR_HOR_ANY.QNTVEN%type;
    BEGIN
      -- La quantitat de la hora est calculada en la taula d' agregat TFC_AGR_DIAMES_ANY 
      -- S' obt la quantitat mnima 
      Select min(QNTVEN)
        into vQntVen
        from TFC_AGR_HOR_ANY aha
        where aha.INA_NUMANY = P_NUMANY and 
              aha.FLGVEN     = 'S';
        
      RSP := TFC_UTL.RSP_OK;
      
      return nvl(vQntVen, 0);
    EXCEPTION -- Tractament d' excepcions 
      WHEN OTHERS THEN -- Si es produeix un error no controlat es retorna el codi i el missatge.
        RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
    END;

    -- Funci que retorna laquantitat mnima histrica de l' any passat com a parmetre.
    -- Parmetres d' entrada:
    --   P_NUMANY => Any per el que es vol buscar la quantitat mnima.
    -- Parmetres de sortida:
    --   P_RSP    => Indica si l' operaci ha tingut xit o no:
    --     'OK'   => Valor indicatiu que el procs ha anat b.
    --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
    -- Retorn de la funci: Quantitat mnima histrica de l' any passat com a parmetre
    FUNCTION CALC_QNTVEN_MIN_HIS(P_NUMANY TFC_AGR_HOR_ANY.INA_NUMANY%TYPE,
                                 RSP      OUT VARCHAR2) RETURN TFC_AGR_HOR_ANY.QNTVEN_HIS%type IS
      vQntVen TFC_AGR_HOR_ANY.QNTVEN_HIS%type;
    BEGIN
      -- La quantitat de la hora est calculada en la taula d' agregat TFC_AGR_DIAMES_ANY 
      -- S' obt la quantitat mnima 
      Select min(QNTVEN_HIS)
        into vQntVen
        from TFC_AGR_HOR_ANY aha
        where aha.INA_NUMANY = P_NUMANY;
        
      RSP := TFC_UTL.RSP_OK;
      
      return nvl(vQntVen, 0);
    EXCEPTION -- Tractament d' excepcions 
      WHEN OTHERS THEN -- Si es produeix un error no controlat es retorna el codi i el missatge.
        RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
    END;

    -- Funci que retorna laquantitat mxima de l' any passat com a parmetre.
    -- Parmetres d' entrada:
    --   P_NUMANY => Any per el que es vol buscar la quantitat mxima.
    -- Parmetres de sortida:
    --   P_RSP    => Indica si l' operaci ha tingut xit o no:
    --     'OK'   => Valor indicatiu que el procs ha anat b.
    --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
    -- Retorn de la funci: Quantitat mxima de l' any passat com a parmetre
    FUNCTION CALC_QNTVEN_MAX(P_NUMANY TFC_AGR_HOR_ANY.INA_NUMANY%TYPE,
                             RSP      OUT VARCHAR2) RETURN TFC_AGR_HOR_ANY.QNTVEN%type IS 
      vQntVen TFC_AGR_HOR_ANY.QNTVEN%type;
    BEGIN
      -- La quantitat de la hora est calculada en la taula d' agregat TFC_AGR_DIAMES_ANY 
      -- S' obt la quantitat mxima 
      Select max(QNTVEN)
        into vQntVen 
        from TFC_AGR_HOR_ANY adma
        where adma.INA_NUMANY = P_NUMANY and 
              adma.FLGVEN     = 'S';
        
      RSP := TFC_UTL.RSP_OK;
      
      return nvl(vQntVen, 0);
    EXCEPTION -- Tractament d' excepcions 
      WHEN OTHERS THEN -- Si es produeix un error no controlat es retorna el codi i el missatge.
        RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
    END;
    
    -- Funci que retorna laquantitat mxima histrica de l' any passat com a parmetre.
    -- Parmetres d' entrada:
    --   P_NUMANY => Any per el que es vol buscar la quantitat mxima.
    -- Parmetres de sortida:
    --   P_RSP    => Indica si l' operaci ha tingut xit o no:
    --     'OK'   => Valor indicatiu que el procs ha anat b.
    --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
    -- Retorn de la funci: Quantitat mxima histrica de l' any passat com a parmetre
    FUNCTION CALC_QNTVEN_MAX_HIS(P_NUMANY TFC_AGR_HOR_ANY.INA_NUMANY%TYPE,
                                 RSP      OUT VARCHAR2) RETURN TFC_AGR_HOR_ANY.QNTVEN_HIS%type IS 
      vQntVen TFC_AGR_HOR_ANY.QNTVEN_HIS%type;
    BEGIN
      -- La quantitat de la hora est calculada en la taula d' agregat TFC_AGR_DIAMES_ANY 
      -- S' obt la quantitat mxima 
      Select max(QNTVEN_HIS)
        into vQntVen 
        from TFC_AGR_HOR_ANY adma
        where adma.INA_NUMANY = P_NUMANY;
        
      RSP := TFC_UTL.RSP_OK;
      
      return nvl(vQntVen, 0);
    EXCEPTION -- Tractament d' excepcions 
      WHEN OTHERS THEN -- Si es produeix un error no controlat es retorna el codi i el missatge.
        RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
    END;
    
    -- Procediment que seteja la variable global privada Quantitat_Venuda_Minima
    -- Parmetres d' entrada:
    --   P_QNTVEN => Quantitat venuda.
    PROCEDURE SET_QNTVEN_MIN(P_QNTVEN TFC_AGR_HOR_ANY.QNTVEN%type) IS
    BEGIN
      Quantitat_Venuda_Minima := Nvl(P_QNTVEN, 0);
    END;
    
    -- Procediment que seteja la variable global privada Quantitat_Venuda_Maxima 
    -- Parmetres d' entrada:
    --   P_QNTVEN => Quantitat venuda.
    PROCEDURE SET_QNTVEN_MAX(P_QNTVEN TFC_AGR_HOR_ANY.QNTVEN%type) IS
    BEGIN
      Quantitat_Venuda_Maxima := Nvl(P_QNTVEN, 0);
    END;

    -- Procediment que seteja la variable global privada Quantitat_Venuda_Minima_HIS
    -- Parmetres d' entrada:
    --   P_QNTVEN => Quantitat venuda.
    PROCEDURE SET_QNTVEN_MIN_HIS(P_QNTVEN TFC_AGR_HOR_ANY.QNTVEN_HIS%type) IS
    BEGIN
      Quantitat_Venuda_Minima_HIS := Nvl(P_QNTVEN, 0);
    END;
    
    -- Procediment que seteja la variable global privada Quantitat_Venuda_Maxima_HIS 
    -- Parmetres d' entrada:
    --   P_QNTVEN => Quantitat venuda.
    PROCEDURE SET_QNTVEN_MAX_HIS(P_QNTVEN_HIS TFC_AGR_HOR_ANY.QNTVEN_HIS%type) IS
    BEGIN
      Quantitat_Venuda_Maxima_HIS := Nvl(P_QNTVEN_HIS, 0);
    END;
      
    -- Procediment que seteja les globals privades de quantitat mnima i mxima a partir 
    -- de la informaci de la taula d' agregats TFC_AGR_DIAMES_ANY 
    -- Parmetres d' entrada:
    --   P_NUMANY => Any per el que es vol buscar la quantitat mnima i mxima.
    -- Parmetres de sortida:
    --   P_RSP    => Indica si l' operaci ha tingut xit o no:
    --     'OK'   => Valor indicatiu que el procs ha anat b.
    --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
    PROCEDURE CALCULAR_QNTVEN_MIN_MAX(P_NUMANY TFC_AGR_HOR_ANY.INA_NUMANY%TYPE,
                                      RSP      OUT VARCHAR2) IS 
    BEGIN
      RSP := TFC_UTL.RSP_OK;
      
      -- Si no s' ha calculat prviament, s' informa el valor de la quantitat mnima  
      If ((Quantitat_Venuda_Minima is null) or (AnyTractat <> P_NUMANY)) then
        Quantitat_Venuda_Minima := CALC_QNTVEN_MIN(P_NUMANY, RSP);
        AnyTractat := P_NUMANY;
      end if;

      -- Si no s' ha produt cap error i no s' ha calculat prviament la quantitat mxima venuda, s' informa la global de quantitat mxima venuda 
      If ((RSP=TFC_UTL.RSP_OK) and ((Quantitat_Venuda_Maxima is null) or (AnyTractat <> P_NUMANY))) then
        Quantitat_Venuda_Maxima := CALC_QNTVEN_MAX(P_NUMANY, RSP);
        AnyTractat := P_NUMANY;
      end if;
    EXCEPTION -- Tractament d' excepcions
      WHEN OTHERS THEN -- Si es produeix un error no controlat es retorna el codi i el missatge.
        RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
    END;
    
    -- Procediment que seteja les globals privades de quantitat mnima i mxima histrics a partir 
    -- de la informaci de la taula d' agregats TFC_AGR_DIAMES_ANY 
    -- Parmetres d' entrada:
    --   P_NUMANY => Any per el que es vol buscar la quantitat mnima i mxima histrics.
    -- Parmetres de sortida:
    --   P_RSP    => Indica si l' operaci ha tingut xit o no:
    --     'OK'   => Valor indicatiu que el procs ha anat b.
    --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
    PROCEDURE CALCULAR_QNTVEN_MIN_MAX_HIS(P_NUMANY TFC_AGR_HOR_ANY.INA_NUMANY%TYPE,
                                          RSP      OUT VARCHAR2) IS 
    BEGIN
      RSP := TFC_UTL.RSP_OK;
      
      -- Si no s' ha calculat prviament, s' informa el valor de la quantitat mnima  
      If ((Quantitat_Venuda_Minima_HIS is null) or (AnyTractat <> P_NUMANY)) then
        Quantitat_Venuda_Minima_HIS := CALC_QNTVEN_MIN_HIS(P_NUMANY, RSP);
        AnyTractat := P_NUMANY;
      end if;

      -- Si no s' ha produt cap error i no s' ha calculat prviament la quantitat mxima venuda, s' informa la global de quantitat mxima venuda 
      If ((RSP=TFC_UTL.RSP_OK) and ((Quantitat_Venuda_Maxima_HIS is null) or (AnyTractat <> P_NUMANY))) then
        Quantitat_Venuda_Maxima_HIS := CALC_QNTVEN_MAX_HIS(P_NUMANY, RSP);
        AnyTractat := P_NUMANY;
      end if;
    EXCEPTION -- Tractament d' excepcions
      WHEN OTHERS THEN -- Si es produeix un error no controlat es retorna el codi i el missatge.
        RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
    END;

    -- Procediment que torna a informar les globals privades de quantitat mnima i mxima a partir 
    -- de la informaci de la taula d' agregats TFC_AGR_DIAMES_ANY
    -- Parmetres d' entrada:
    --   P_NUMANY => Any per el que es vol buscar la quantitat mnima i mxima venuda .
    -- Parmetres de sortida:
    --   P_RSP    => Indica si l' operaci ha tingut xit o no:
    --     'OK'   => Valor indicatiu que el procs ha anat b.
    --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
    PROCEDURE RECALCULAR_QNTVEN_MIN_MAX(P_NUMANY TFC_AGR_HOR_ANY.INA_NUMANY%TYPE,
                                        RSP      OUT VARCHAR2) IS 
    BEGIN
      Quantitat_Venuda_Minima := CALC_QNTVEN_MIN(P_NUMANY, RSP);
      -- Si la funci anterior ha generat un error no cas continuar ... 
      If (RSP = TFC_UTL.RSP_OK) then 
        Quantitat_Venuda_Maxima := CALC_QNTVEN_MAX(P_NUMANY, RSP);
      end if;
    EXCEPTION -- Tractament d' excepcions
      WHEN OTHERS THEN -- Si es produeix un error no controlat es retorna el codi i el missatge.
        RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
    END;

    -- Procediment que torna a informar les globals privades de quantitat mnima i mxima a partir histrics
    -- de la informaci de la taula d' agregats TFC_AGR_DIAMES_ANY
    -- Parmetres d' entrada:
    --   P_NUMANY => Any per el que es vol buscar la quantitat mnima i mxima venuda .
    -- Parmetres de sortida:
    --   P_RSP    => Indica si l' operaci ha tingut xit o no:
    --     'OK'   => Valor indicatiu que el procs ha anat b.
    --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
    PROCEDURE RECALCULAR_QNTVEN_MIN_MAX_HIS(P_NUMANY TFC_AGR_HOR_ANY.INA_NUMANY%TYPE,
                                            RSP      OUT VARCHAR2) IS 
    BEGIN
      Quantitat_Venuda_Minima_HIS := CALC_QNTVEN_MIN_HIS(P_NUMANY, RSP);
      -- Si la funci anterior ha generat un error no cas continuar ... 
      If (RSP = TFC_UTL.RSP_OK) then 
        Quantitat_Venuda_Maxima_HIS := CALC_QNTVEN_MAX_HIS(P_NUMANY, RSP);
      end if;
    EXCEPTION -- Tractament d' excepcions
      WHEN OTHERS THEN -- Si es produeix un error no controlat es retorna el codi i el missatge.
        RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
    END;

    -- Procediment que calcula els indicador de ms i menys vendes .
    -- El resultat s' informa a la taula TFC_AGR_DIAMES_ANY
    -- Parmetres d' entrada:
    --   P_NUMANY     => Any per el que es vol buscar els indicadors 
    --   P_HORA       => Hora del dia   
    --   P_QNTVEN_AGR => Quantitat de la hora del dia  
    --   P_FLGMEN     => Indicador de menys vendes associat a la hora del dia     
    --   P_FLGMES     => Indicador de ms vendes associat a la hora del dia
    -- Parmetres de sortida:
    --   P_RSP    => Indica si l' operaci ha tingut xit o no:
    --     'OK'   => Valor indicatiu que el procs ha anat b.
    --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
    PROCEDURE CALC_IND_VEN(P_NUMANY     TFC_AGR_HOR_ANY.INA_NUMANY%TYPE,
                           P_QNTVEN_AGR TFC_AGR_HOR_ANY.QNTVEN%TYPE,
                           P_FLGMEN     TFC_AGR_HOR_ANY.FLGMEN%TYPE,
                           P_FLGMES     TFC_AGR_HOR_ANY.FLGMES%TYPE,
                           RSP          OUT VARCHAR2) IS
    BEGIN
      RECALCULAR_QNTVEN_MIN_MAX(P_NUMANY, RSP);

      If (RSP = TFC_UTL.RSP_OK) then   
        If (
            (P_QNTVEN_AGR >= Quantitat_Venuda_Maxima) or 
            (P_FLGMES = 'S') or 
            (P_QNTVEN_AGR <= Quantitat_Venuda_Minima) or
            (P_FLGMEN = 'S')
           ) 
        then 
          -- S' actualitzen els indicadors de ms i menys vendes a la taula TFC_AGR_DIAMES_ANY 
          Update TFC_AGR_HOR_ANY aha
             set aha.FLGMES = Decode (aha.QNTVEN, Quantitat_Venuda_Maxima, 'S', 'N'),
                 aha.FLGMEN = Decode (aha.QNTVEN, Quantitat_Venuda_Minima, 'S', 'N')
             where aha.INA_NUMANY = P_NUMANY and 
                   (
                    (aha.FLGMEN = 'S') or  
                    (aha.QNTVEN = Quantitat_Venuda_Minima) or
                    (aha.FLGMES = 'S') or  
                    (aha.QNTVEN = Quantitat_Venuda_Maxima)
                   ) and 
                   aha.FLGVEN = 'S';
        end if;
      end if;
    EXCEPTION -- Tractament d' excepcions 
      WHEN OTHERS THEN -- Si es produeix un error no controlat es retorna el codi i el missatge.
        RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
    END;

    -- Procediment que calcula els indicador de ms i menys vendes .
    -- El resultat s' informa a la taula TFC_AGR_DIAMES_ANY
    -- Parmetres d' entrada:
    --   P_NUMANY     => Any per el que es vol buscar els indicadors 
    --   P_HORA       => Hora del dia   
    --   P_QNTVEN_AGR => Quantitat de la hora del dia  
    --   P_FLGMEN     => Indicador de menys vendes associat a la hora del dia     
    --   P_FLGMES     => Indicador de ms vendes associat a la hora del dia
    -- Parmetres de sortida:
    --   P_RSP    => Indica si l' operaci ha tingut xit o no:
    --     'OK'   => Valor indicatiu que el procs ha anat b.
    --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
    PROCEDURE CALC_IND_VEN_HIS(P_NUMANY     TFC_AGR_HOR_ANY.INA_NUMANY%TYPE,
                               P_QNTVEN_AGR TFC_AGR_HOR_ANY.QNTVEN_HIS%TYPE,
                               P_FLGMEN     TFC_AGR_HOR_ANY.FLGMEN_HIS%TYPE,
                               P_FLGMES     TFC_AGR_HOR_ANY.FLGMES_HIS%TYPE,
                               RSP          OUT VARCHAR2) IS
    BEGIN
      RECALCULAR_QNTVEN_MIN_MAX_HIS(P_NUMANY, RSP);

      If (RSP = TFC_UTL.RSP_OK) then   
        If (
            (P_QNTVEN_AGR >= Quantitat_Venuda_Maxima_HIS) or 
            (P_FLGMES = 'S') or 
            (P_QNTVEN_AGR <= Quantitat_Venuda_Minima_HIS) or
            (P_FLGMEN = 'S')
           ) 
        then 
          -- S' actualitzen els indicadors de ms i menys vendes a la taula TFC_AGR_DIAMES_ANY 
          Update TFC_AGR_HOR_ANY aha
             set aha.FLGMES_HIS = Decode (aha.QNTVEN_HIS, Quantitat_Venuda_Maxima_HIS, 'S', 'N'),
                 aha.FLGMEN_HIS = Decode (aha.QNTVEN_HIS, Quantitat_Venuda_Minima_HIS, 'S', 'N')
             where aha.INA_NUMANY = P_NUMANY and 
                   (
                    (aha.FLGMEN_HIS = 'S') or  
                    (aha.QNTVEN_HIS = Quantitat_Venuda_Minima_HIS) or
                    (aha.FLGMES_HIS = 'S') or  
                    (aha.QNTVEN_HIS = Quantitat_Venuda_Maxima_HIS)
                   );
        end if;
      end if;
    EXCEPTION -- Tractament d' excepcions 
      WHEN OTHERS THEN -- Si es produeix un error no controlat es retorna el codi i el missatge.
        RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
    END;

    -- Procediment que dna d' alta un registre a la taula TFC_AGR_HOR_ANY 
    -- Parmetres d' entrada:
    --   P_NUMANY => Any per el que es vol buscar els indicadors 
    --   P_HORA   => Hora del dia   
    --   P_QNTVEN => Quantitat venuda 
    -- Parmetres de sortida:
    --   P_RSP    => Indica si l' operaci ha tingut xit o no:
    --     'OK'   => Valor indicatiu que el procs ha anat b.
    --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
    PROCEDURE ALT_AHA(P_NUMANY TFC_AGR_HOR_ANY.INA_NUMANY%TYPE,
                      P_HORA   TFC_AGR_HOR_ANY.HORA%TYPE,
                      P_QNTVEN TFC_AGR_HOR_ANY.QNTVEN%TYPE,
                      RSP      OUT VARCHAR2) IS
      vQntVen TFC_AGR_HOR_ANY.QNTVEN%TYPE;    
      vFlgMen TFC_AGR_HOR_ANY.FLGMEN%type;
      vFlgMes TFC_AGR_HOR_ANY.FLGMES%type;
      vQntVen_HIS TFC_AGR_HOR_ANY.QNTVEN_HIS%TYPE;    
      vQntVen_HIS_Ant    TFC_AGR_DIAMES_ANY.QNTVEN_HIS%TYPE; 
      vFlgMen_HIS TFC_AGR_HOR_ANY.FLGMEN_HIS%type;
      vFlgMes_HIS TFC_AGR_HOR_ANY.FLGMES_HIS%type;
      vNomProces TFC_RASTRE_PROCES.PROCES%Type := 'TFC_AHA.ALT_AHA';
      vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
      vError_Controlat   exception;
      vProces_Error      varchar2(100);
    BEGIN
      If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
        TFC_RAP.INICIAR_RASTRE(vNomProces);
      end if;
      
      -- Es graven els parmetres a la taula de rastre    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('P_NUMANY='||P_NUMANY);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('P_HORA='||P_HORA);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('P_QNTVEN='||P_QNTVEN);
      
      -- S' inicialitzen si cal les globals de mnim i mxim 
      CALCULAR_QNTVEN_MIN_MAX(P_NUMANY, RSP);
      If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
        vProces_Error := 'CALCULAR_QNTVEN_MIN_MAX';
        raise vError_Controlat;
      end if;

      -- S' inicialitzen si cal les globals de mnim i mxim 
      CALCULAR_QNTVEN_MIN_MAX_HIS(P_NUMANY, RSP);
      If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
        vProces_Error := 'CALCULAR_QNTVEN_MIN_MAX_HIS';
        raise vError_Controlat;
      end if;


      -- S' updateja el registre de la taula d' agregats de ciutats per any 
      Update TFC_AGR_HOR_ANY aha
         set aha.QNTVEN = aha.QNTVEN + nvl(P_QNTVEN, 0),
             aha.QNTVEN_HIS = aha.QNTVEN_HIS + nvl(P_QNTVEN, 0),
             aha.FLGVEN = 'S'
         where aha.INA_NUMANY = P_NUMANY and 
               aha.HORA       = P_HORA
         returning aha.QNTVEN, aha.FLGMEN, aha.FLGMES, aha.QNTVEN_HIS, aha.FLGMEN_HIS, aha.FLGMES_HIS
         into      vQntVen,    vFlgMen,    vFlgMes,    vQntVen_HIS,    vFlgMen_HIS,    vFlgMes_HIS;
      
      If (sql%RowCount = 0) then -- No s' ha trobat cap registre i per tant s' ha de donar d' alta 
        -- S' inserta el registre 
        Insert into TFC_AGR_HOR_ANY 
          (INA_NUMANY, 
           HORA,
           QNTVEN,
           QNTVEN_HIS,
           FLGVEN)
        values
          (P_NUMANY,
           P_HORA,
           nvl(P_QNTVEN,0),
           nvl((Select QNTVEN_HIS 
                  from TFC_AGR_HOR_ANY aha
                  where aha.INA_NUMANY = (P_NUMANY - 1) and 
                        aha.HORA = P_HORA), 0) + 
           nvl(P_QNTVEN,0),
           'S')
        returning QNTVEN,  FLGMEN,  FLGMES,  QNTVEN_HIS,  FLGMEN_HIS,  FLGMES_HIS
        into      vQntVen, vFlgMen, vFlgMes, vQntVen_HIS, vFlgMen_HIS, vFlgMes_HIS;
      end if;
      
      -- Es calculen els indicadors de mnim i mxim a partir de les dades del registre insertat o modificat 
      CALC_IND_VEN(P_NUMANY,
                   vQntVen,
                   vFlgMen,
                   vFlgMes,
                   RSP); 
                   
      If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
        vProces_Error := 'CALC_IND_VEN';
        raise vError_Controlat;
      end if;
      
      -- Es calculen els indicadors de mnim i mxim histrics a partir de les dades del registre insertat o modificat 
      CALC_IND_VEN_HIS(P_NUMANY,
                       vQntVen_HIS,
                       vFlgMen_HIS,
                       vFlgMes_HIS,
                       RSP); 
                   
      If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
        vProces_Error := 'CALC_IND_VEN_HIS';
        raise vError_Controlat;
      end if;
      
    -- Ens guardem la quantitat histrica acumulada fins l' any tractat
    vQntVen_HIS_Ant := vQntVen_HIS;    
    -- En cas necessari s' hauran de recalcular els indicadors histrics futurs 
    For d in (Select INA.NUMANY 
                from TFC_INDICADORS_ANY ina
                where ina.NUMANY > P_NUMANY
                order by numany) loop

        -- S' inicialitzen, si cal, les globals de mnim i mxim histrics 
        CALCULAR_QNTVEN_MIN_MAX_HIS(d.numany, RSP);
        If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
          vProces_Error := 'CALCULAR_QNTVEN_MIN_MAX_HIS (QNTVEN_HIS futur)';
          raise vError_Controlat;
        end if;
        
        Update TFC_AGR_HOR_ANY aha 
           set aha.QNTVEN_His = aha.QNTVEN_HIS + nvl(P_QNTVEN, 0)
           where aha.Ina_Numany = d.numany and 
                 aha.HORA = P_HORA
           returning aha.QntVen_His, aha.FlgMen_His, aha.FlgMes_his 
                into vQntVen_His,    vFlgMen_His,    vFlgMes_His;

        -- Si no existeix el registre...      
        If (sql%RowCount = 0) then
          Insert into TFC_AGR_HOR_ANY 
            (INA_NUMANY, 
             HORA,
             QNTVEN,
             QNTVEN_HIS,
             FLGVEN)
          values
            (d.NUMANY,
             P_HORA,
             0,
             vQntVen_HIS_Ant,
             'N')
          returning QNTVEN_HIS,  FLGMEN_HIS,  FLGMES_HIS
          into      vQntVen_HIS, vFlgMen_HIS, vFlgMes_HIS;
        end if;
 
        -- Es calculen els indicadors de mnim i mxim histrics a partir de les dades del registre insertat o modificat 
        CALC_IND_VEN_HIS(d.NUMANY,
                         vQntVen_HIS,
                         vFlgMen_HIS,
                         vFlgMes_HIS,
                         RSP);
      
        If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
          vProces_Error := 'CALC_IND_VEN_HIS (BENNET_HIS futur)';
          raise vError_Controlat;
        end if;
        
        -- Es recalculen els indicadors anuals...
        TFC_INA.CALC_INA(d.NUMANY, RSP);
        If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
          vProces_Error := 'TFC_INA.CALC_INA ('||d.NUMANY||',RSP) futur';
          raise vError_Controlat;
        end if;
        
        -- Ens guardem la quantitat histrica acumulada fins l' any tractat
        vQntVen_HIS_Ant := vQntVen_HIS;
      end loop;
      
      -- Arribats a aquest punt, tot ha anat b.
      If (RSP is null) then 
        RSP := TFC_UTL.RSP_OK;
      end if;
                   
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);

      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
        If (RSP = TFC_UTL.RSP_OK) then 
          TFC_RAP.FINALITZAR_RASTRE_OK;
        else
          TFC_RAP.FINALITZAR_RASTRE_ERR;
        end if;
      end if;
    EXCEPTION -- Tractament d' excepcions 
      WHEN others THEN 
        If (vProces_Error is null) then -- Si vProces_Error es null, es tracta d' un error no controlat 
          RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
          TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
        else
          TFC_RAP.REGISTRAR_RASTRE_LINIA('Error executant el procs '||vProces_Error||':');
        end if;
        
        -- Es registre l' error a la taula de rastre 
        TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
        
        -- Es graven els parmetres a la taula de rastre 
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
        TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
        
        If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
          TFC_RAP.FINALITZAR_RASTRE_ERR;
        end if;
    END;

    -- Procediment que modifica un registre a la taula TFC_AGR_HOR_ANY 
    -- Parmetres d' entrada:
    --   P_NUMANY => Any per el que es vol buscar els indicadors 
    --   P_HORA   => Hora del dia   
    --   P_QNTVEN => Quantitat venuda 
    -- Parmetres de sortida:
    --   P_RSP    => Indica si l' operaci ha tingut xit o no:
    --     'OK'   => Valor indicatiu que el procs ha anat b.
    --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
    PROCEDURE MOD_AHA(P_NUMANY TFC_AGR_HOR_ANY.INA_NUMANY%TYPE,
                      P_HORA   TFC_AGR_HOR_ANY.HORA%TYPE,
                      P_QNTVEN TFC_AGR_HOR_ANY.QNTVEN%TYPE,
                      RSP      OUT VARCHAR2) IS
      vQntVen TFC_AGR_HOR_ANY.QNTVEN%TYPE;    
      vFlgMen TFC_AGR_HOR_ANY.FLGMEN%type;
      vFlgMes TFC_AGR_HOR_ANY.FLGMES%type;
      vQntVen_HIS TFC_AGR_HOR_ANY.QNTVEN_HIS%TYPE;    
      vFlgMen_HIS TFC_AGR_HOR_ANY.FLGMEN_HIS%type;
      vFlgMes_HIS TFC_AGR_HOR_ANY.FLGMES_HIS%type;
      vNomProces TFC_RASTRE_PROCES.PROCES%Type := 'TFC_AHA.MOD_AHA';
      vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
      vError_Controlat   exception;
      vProces_Error      varchar2(100);
    BEGIN
      If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
        TFC_RAP.INICIAR_RASTRE(vNomProces);
      end if;
      
      -- Es graven els parmetres a la taula de rastre    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('P_NUMANY='||P_NUMANY);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('P_HORA='||P_HORA);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('P_QNTVEN='||P_QNTVEN);
      
      -- S' inicialitzen si cal les globals de mnim i mxim 
      CALCULAR_QNTVEN_MIN_MAX(P_NUMANY, RSP);

      If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
        vProces_Error := 'CALCULAR_QNTVEN_MIN_MAX';
        raise vError_Controlat;
      end if;

      -- S' inicialitzen si cal les globals de mnim i mxim histrics 
      CALCULAR_QNTVEN_MIN_MAX_HIS(P_NUMANY, RSP);
      
      If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
        vProces_Error := 'CALCULAR_QNTVEN_MIN_MAX_HIS';
        raise vError_Controlat;
      end if;
      
      -- S' updateja el registre de la taula d' agregats de ciutats per any
      Update TFC_AGR_HOR_ANY aha
         set aha.QNTVEN = aha.QNTVEN + nvl(P_QNTVEN, 0),
             aha.QNTVEN_HIS = aha.QNTVEN_HIS + nvl(P_QNTVEN, 0),
             aha.FLGVEN = 'S'
         where aha.INA_NUMANY = P_NUMANY and 
               aha.HORA       = P_HORA
         returning aha.QNTVEN, aha.FLGMEN, aha.FLGMES, aha.QNTVEN_HIS, aha.FLGMEN_HIS, aha.FLGMES_HIS
         into      vQntVen,    vFlgMen,    vFlgMes,    vQntVen_HIS,    vFlgMen_HIS,    vFlgMes_HIS;
      
      -- Es calculen els indicadors de mnim i mxim a partir de les dades del registre insertat o modificat 
      CALC_IND_VEN(P_NUMANY,
                   vQntVen,
                   vFlgMen,
                   vFlgMes,
                   RSP); 
    
      If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
        vProces_Error := 'CALC_IND_VEN';
        raise vError_Controlat;
      end if;
    
      -- Es calculen els indicadors de mnim i mxim histrics a partir de les dades del registre insertat o modificat 
      CALC_IND_VEN_HIS(P_NUMANY,
                       vQntVen_HIS,
                       vFlgMen_HIS,
                       vFlgMes_HIS,
                       RSP); 
    
      If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
        vProces_Error := 'CALC_IND_VEN';
        raise vError_Controlat;
      end if;
    
      If (vQntVen_His = 0) then 
        SUP_AHA(P_NUMANY,
                P_HORA,
                RSP);
                
        If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
          vProces_Error := 'SUP_ADMA';
          Raise vError_Controlat;
        end if;
      else
        RSP := TFC_UTL.RSP_OK;
      end if;
      
      -- En cas necessari s' hauran de recalcular els indicadors histrics futurs 
      For d in (Select INA.NUMANY 
                  from TFC_INDICADORS_ANY ina
                  where ina.NUMANY > P_NUMANY
                  order by numany) loop

        -- S' inicialitzen, si cal, les globals de mnim i mxim histrics 
        CALCULAR_QNTVEN_MIN_MAX_HIS(d.numany, RSP);
        If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
          vProces_Error := 'CALCULAR_QNTVEN_MIN_MAX_HIS (QNTVEN_HIS futur)';
          raise vError_Controlat;
        end if;
        
        Update TFC_AGR_HOR_ANY aha 
           set aha.QNTVEN_His = aha.QNTVEN_HIS + nvl(P_QNTVEN, 0)
           where aha.Ina_Numany = d.numany and 
                 aha.HORA = P_HORA
           returning aha.QntVen_His, aha.FlgMen_His, aha.FlgMes_his 
                into vQntVen_His,    vFlgMen_His,    vFlgMes_His;

        -- Es calculen els indicadors de mnim i mxim histrics a partir de les dades del registre insertat o modificat 
        CALC_IND_VEN_HIS(d.NUMANY,
                         vQntVen_HIS,
                         vFlgMen_HIS,
                         vFlgMes_HIS,
                         RSP);
      
        If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
          vProces_Error := 'CALC_IND_VEN_HIS (BENNET_HIS futur)';
          raise vError_Controlat;
        end if;
        
        -- Es recalculen els indicadors anuals...
        TFC_INA.CALC_INA(d.NUMANY, RSP);
        If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
          vProces_Error := 'TFC_INA.CALC_INA ('||d.NUMANY||',RSP) futur';
          raise vError_Controlat;
        end if;
      end loop;
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);
      
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
        If (RSP = TFC_UTL.RSP_OK) then 
          TFC_RAP.FINALITZAR_RASTRE_OK;
        else
          TFC_RAP.FINALITZAR_RASTRE_ERR;
        end if;
      end if;
      
    EXCEPTION -- Tractament d' excepcions 
      WHEN others THEN 
        If (vProces_Error is null) then -- Si vProces_Error es null, es tracta d' un error no controlat
          RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
          TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
        else
          TFC_RAP.REGISTRAR_RASTRE_LINIA('Error executant el procs '||vProces_Error||':');
        end if;
        
        -- Es registre l' error a la taula de rastre 
        TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
        
        -- Es graven els parmetres a la taula de rastre 
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
        TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
        
        If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
          TFC_RAP.FINALITZAR_RASTRE_ERR;
        end if;
    END;

    -- Procediment que decrementa la quantitat d' un registre a la taula TFC_AGR_HOR_ANY 
    -- Parmetres d' entrada:
    --   P_NUMANY     => Any per el que es vol buscar els indicadors 
    --   P_HORA       => Hora del dia   
    --   P_QNTVEN_DEC => Quantitat a decrementar  
    -- Parmetres de sortida:
    --   P_RSP    => Indica si l' operaci ha tingut xit o no:
    --     'OK'   => Valor indicatiu que el procs ha anat b.
    --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
    PROCEDURE SUP_AHA(P_NUMANY     TFC_AGR_HOR_ANY.INA_NUMANY%TYPE,
                      P_HORA       TFC_AGR_HOR_ANY.HORA%TYPE,
                      P_QNTVEN_DEC TFC_AGR_HOR_ANY.QNTVEN%TYPE,
                      RSP          OUT VARCHAR2) IS
      vNomProces TFC_RASTRE_PROCES.PROCES%Type := 'TFC_AHA.SUP_AHA';
      vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
      vError_Controlat   exception;
      vProces_Error      varchar2(100);
    BEGIN
      If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
        TFC_RAP.INICIAR_RASTRE(vNomProces);
      end if;
      
      -- Es graven els parmetres a la taula de rastre    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('P_NUMANY='||P_NUMANY);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('P_HORA='||P_HORA);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('P_QNTVEN_DEC='||P_QNTVEN_DEC);
      
      MOD_AHA(P_NUMANY,
              P_HORA,
              P_QNTVEN_DEC,
              RSP);
              
      If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
        vProces_Error:= 'MOD_ACA';
        raise vError_Controlat;
      end if;
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);
      
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
        If (RSP = TFC_UTL.RSP_OK) then 
          TFC_RAP.FINALITZAR_RASTRE_OK;
        else
          TFC_RAP.FINALITZAR_RASTRE_ERR;
        end if;
      end if;
    EXCEPTION -- Tractament d' excepcions 
      WHEN others THEN 
        If (vProces_Error is null) then -- Si vProces_Error es null, es tracta d' un error no controlat
          RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
          TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
        else
          TFC_RAP.REGISTRAR_RASTRE_LINIA('Error executant el procs '||vProces_Error||':');
        end if;
        
        -- Es registre l' error a la taula de rastre 
        TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
        
        -- Es graven els parmetres a la taula de rastre 
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
        TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
        
        If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
          TFC_RAP.FINALITZAR_RASTRE_ERR;
        end if;
    END;

    -- Procediment que elimina un registre a la taula TFC_AGR_HOR_ANY 
    -- Parmetres d' entrada:
    --   P_NUMANY => Any per el que es vol buscar els indicadors 
    --   P_HORA   => Hora del dia   
    -- Parmetres de sortida:
    --   P_RSP    => Indica si l' operaci ha tingut xit o no:
    --     'OK'   => Valor indicatiu que el procs ha anat b.
    --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.              
    PROCEDURE SUP_AHA(P_NUMANY TFC_AGR_HOR_ANY.INA_NUMANY%TYPE,
                      P_HORA   TFC_AGR_HOR_ANY.HORA%TYPE,
                      RSP      OUT VARCHAR2) IS
      vNomProces TFC_RASTRE_PROCES.PROCES%Type := 'TFC_AHA.SUP_AHA';
      vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
      vQntVen            TFC_AGR_HOR_ANY.QNTVEN%TYPE;
      vError_Controlat   exception;
      vProces_Error      varchar2(100);
      vQntVen_HIS        TFC_AGR_HOR_ANY.QNTVEN_HIS%TYPE;    
      vFlgMen_HIS        TFC_AGR_HOR_ANY.FLGMEN_HIS%type;
      vFlgMes_HIS        TFC_AGR_HOR_ANY.FLGMES_HIS%type;
    BEGIN
      If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
        TFC_RAP.INICIAR_RASTRE(vNomProces);
      end if;
      
      -- Es graven els parmetres a la taula de rastre    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('P_NUMANY='||P_NUMANY);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('P_HORA='||P_HORA);

      Select QNTVEN
        into vQntVen
        from TFC_AGR_HOR_ANY
        where INA_NUMANY = P_NUMANY and 
              HORA       = P_HORA;

      Update TFC_AGR_HOR_ANY
         set QNTVEN = 0,
             FLGVEN = 'N',
             FLGMES = 'N',
             FLGMEN = 'N'
         where INA_NUMANY = P_NUMANY and 
               HORA       = P_HORA;
      
      -- Es calculen els indicadors de mnim i mxim a partir de les dades del registre insertat o modificat 
      CALC_IND_VEN(P_NUMANY,
                   0,
                   'S',
                   'S',
                   RSP); 
      
      -- En cas necessari s' hauran de recalcular els indicadors histrics futurs 
      For d in (Select INA.NUMANY 
                  from TFC_INDICADORS_ANY ina
                  where ina.NUMANY > P_NUMANY
                  order by numany) loop

        -- S' inicialitzen, si cal, les globals de mnim i mxim histrics 
        CALCULAR_QNTVEN_MIN_MAX_HIS(d.numany, RSP);
        If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
          vProces_Error := 'CALCULAR_QNTVEN_MIN_MAX_HIS (QNTVEN_HIS futur)';
          raise vError_Controlat;
        end if;
        
        Update TFC_AGR_HOR_ANY aha 
           set aha.QNTVEN_His = aha.QNTVEN_HIS - nvl(vQNTVEN, 0)
           where aha.Ina_Numany = d.numany and 
                 aha.HORA = P_HORA
           returning aha.QntVen_His, aha.FlgMen_His, aha.FlgMes_his 
                into vQntVen_His,    vFlgMen_His,    vFlgMes_His;

        -- Es calculen els indicadors de mnim i mxim histrics a partir de les dades del registre insertat o modificat 
        CALC_IND_VEN_HIS(d.NUMANY,
                         vQntVen_HIS,
                         vFlgMen_HIS,
                         vFlgMes_HIS,
                         RSP);
      
        If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
          vProces_Error := 'CALC_IND_VEN_HIS (BENNET_HIS futur)';
          raise vError_Controlat;
        end if;
        
        -- Es recalculen els indicadors anuals...
        TFC_INA.CALC_INA(d.NUMANY, RSP);
        If (RSP <> TFC_UTL.RSP_OK) then -- S' ha produt un error. S' interromp l' execuci.
          vProces_Error := 'TFC_INA.CALC_INA ('||d.NUMANY||',RSP) futur';
          raise vError_Controlat;
        end if;
      end loop;
      
      -- Arribats a aquest punt tot ha anat b 
      If (RSP is null) then 
        RSP := TFC_UTL.RSP_OK;
      end if;

      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);
      
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
        If (RSP = TFC_UTL.RSP_OK) then 
          TFC_RAP.FINALITZAR_RASTRE_OK;
        else
          TFC_RAP.FINALITZAR_RASTRE_ERR;
        end if;
      end if;
    EXCEPTION -- Tractament d' excepcions 
      WHEN others THEN 
        RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
        
        -- Es registre l' error a la taula de rastre 
        TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
        
        -- Es graven els parmetres a la taula de rastre 
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
        TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
        
        If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
          TFC_RAP.FINALITZAR_RASTRE_ERR;
        end if;
    END;

    -- Procediment que recalcula els indicadors de hora del dia i any a la taula TFC_AGR_HOR_ANY 
    -- Parmetres d' entrada:
    --   P_NUMANY => Any per el que es vol buscar els indicadors                                  
    -- Parmetres de sortida:
    --   P_RSP    => Indica si l' operaci ha tingut xit o no:
    --     'OK'   => Valor indicatiu que el procs ha anat b.
    --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
    PROCEDURE RECALCULAR_IND_VEN(P_NUMANY TFC_AGR_HOR_ANY.INA_NUMANY%TYPE,
                                 RSP      OUT VARCHAR2) IS 
      vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
      vNomProces         TFC_RASTRE_PROCES.PROCES%Type := 'TFC_AHA.RECALCULAR_IND_VEN';
    BEGIN
      If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
        TFC_RAP.INICIAR_RASTRE(vNomProces);
      end if;
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('P_NUMANY='||P_NUMANY);
      
      -- S' elimina la informaci de l' any passat com a parmetre 
      Delete 
        from TFC_AGR_HOR_ANY adma
        where adma.Ina_Numany = P_NUMANY;
      
      Insert into TFC_AGR_HOR_ANY
          (INA_NUMANY, 
           HORA,
           QNTVEN,
           FLGMES,
           FLGMEN,
           QNTVEN_HIS,
           FLGMES_HIS,
           FLGMEN_HIS,
           FLGVEN)
      Select INA_NUMANY,
             HORA,
             QNTVEN,
             'N',
             'N',
             QNTVEN_HIS,
             'N',
             'N',
             Decode(COUNT_VEN, 0, 'N', 'S') as FLGVEN
        from (Select P_NUMANY as INA_NUMANY, 
                     tem.HORA, 
                     sum(decode(INA_NUMANY, P_NUMANY, ven.QNTVEN, 0)) as QNTVEN,
                     SUM(ven.QNTVEN) as QNTVEN_HIS,
                     sum(decode(INA_NUMANY, P_NUMANY, 1, 0)) as count_ven
                from TFC_VENDES ven,
                     TFC_TEMPS  tem 
                where ven.TEM_DATA    = tem.DATA and 
                      ven.TEM_HORA    = tem.HORA and 
                      tem.INA_NUMANY <= P_NUMANY 
                group by P_NUMANY, tem.HORA);

      RECALCULAR_QNTVEN_MIN_MAX(P_NUMANY, RSP);
      
      If (RSP = TFC_UTL.RSP_OK) then   
        Update TFC_AGR_HOR_ANY aha
           set aha.FLGMES = Decode (aha.QNTVEN, Quantitat_Venuda_Maxima, 'S', 'N'),
               aha.FLGMEN = Decode (aha.QNTVEN, Quantitat_Venuda_Minima, 'S', 'N')
           where aha.INA_NUMANY = P_NUMANY and 
                 (
                  (aha.QNTVEN = Quantitat_Venuda_Minima) or
                  (aha.QNTVEN = Quantitat_Venuda_Maxima)
                 ) and 
                 aha.FLGVEN = 'S';
      end if;

      RECALCULAR_QNTVEN_MIN_MAX_HIS(P_NUMANY, RSP);
      
      If (RSP = TFC_UTL.RSP_OK) then   
        Update TFC_AGR_HOR_ANY aha
           set aha.FLGMES_HIS = Decode (aha.QNTVEN_HIS, Quantitat_Venuda_Maxima_HIS, 'S', 'N'),
               aha.FLGMEN_HIS = Decode (aha.QNTVEN_HIS, Quantitat_Venuda_Minima_HIS, 'S', 'N')
           where aha.INA_NUMANY = P_NUMANY and 
                 (
                  (aha.QNTVEN_HIS = Quantitat_Venuda_Minima_HIS) or
                  (aha.QNTVEN_HIS = Quantitat_Venuda_Maxima_HIS)
                 );
      end if;

      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);
      
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
        If (RSP = TFC_UTL.RSP_OK) then 
          TFC_RAP.FINALITZAR_RASTRE_OK;
        else
          TFC_RAP.FINALITZAR_RASTRE_ERR;
        end if;
      end if;
    EXCEPTION -- Tractament d' excepcions 
      WHEN others THEN 
        RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
        
        -- Es registre l' error a la taula de rastre 
        TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
        
        -- Es graven els parmetres a la taula de rastre 
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
        TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
        
        If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
          TFC_RAP.FINALITZAR_RASTRE_ERR;
        end if;
    END;
  end TFC_AHA;
/
