create or replace package TFC_BOT is
  -- Procediment que dna d' alta un registre a la taula TFC_BOTIGUES 
  -- Parmetres d' entrada:
  --   P_IDEBOT     => Identificador de la botiga 
  --   P_NOMBOT     => Nom de la botiga 
  --   P_CIU_IDECIU => Identificador de la ciutat 
  --   P_NUMTRBACT  => Nmero de treballadors de la botiga 
  --   P_EMAIL      => Email de contacte 
  --   P_FLGFRA     => s franqucia? (S/N)
  --     'S'        => s franqucia
  --     'N'        => No s franqucia 
  --   P_FLGVIR     => s virtual? (S/N)
  --     'S'        => s virtual
  --     'N'        => No s virtual
  --   P_FLGACT     => Est activa? (S/N) 
  --     'S'        => Est activa 
  --     'N'        => No est activa
  --   P_DATALT     => Data de alta de la botiga 
  --   P_DATBAI     => Data de baixa de la botiga 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE ALT_BOTIGA(P_IDEBOT     TFC_BOTIGUES.IDEBOT%type,
                       P_NOMBOT     TFC_BOTIGUES.NOMBOT%type,
                       P_CIU_IDECIU TFC_BOTIGUES.CIU_IDECIU%type,
                       P_NUMTRBACT  TFC_BOTIGUES.NUMTRBACT%type,
                       P_EMAIL      TFC_BOTIGUES.EMAIL%type,
                       P_FLGFRA     TFC_BOTIGUES.FLGFRA%type,
                       P_FLGVIR     TFC_BOTIGUES.FLGVIR%type,
                       P_FLGACT     TFC_BOTIGUES.FLGACT%type,
                       P_DATALT     TFC_BOTIGUES.DATALT%type,
                       P_DATBAI     TFC_BOTIGUES.DATBAI%type,
                       RSP          OUT VARCHAR2);

  -- Procediment que modifica un registre a la taula TFC_BOTIGUES 
  -- Parmetres d' entrada:
  --   P_IDEBOT     => Identificador de la botiga 
  --   P_NOMBOT     => Nom de la botiga 
  --   P_CIU_IDECIU => Identificador de la ciutat 
  --   P_NUMTRBACT  => Nmero de treballadors de la botiga 
  --   P_EMAIL      => Email de contacte 
  --   P_FLGFRA     => s franqucia? (S/N)
  --     'S'        => s franqucia
  --     'N'        => No s franqucia 
  --   P_FLGVIR     => s virtual? (S/N)
  --     'S'        => s virtual
  --     'N'        => No s virtual
  --   P_FLGACT     => Est activa? (S/N) 
  --     'S'        => Est activa 
  --     'N'        => No est activa
  --   P_DATALT     => Data de alta de la botiga 
  --   P_DATBAI     => Data de baixa de la botiga 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE MOD_BOTIGA(P_IDEBOT     TFC_BOTIGUES.IDEBOT%type,
                       P_NOMBOT     TFC_BOTIGUES.NOMBOT%type,
                       P_CIU_IDECIU TFC_BOTIGUES.CIU_IDECIU%type,
                       P_NUMTRBACT  TFC_BOTIGUES.NUMTRBACT%type,
                       P_EMAIL      TFC_BOTIGUES.EMAIL%type,
                       P_FLGFRA     TFC_BOTIGUES.FLGFRA%type,
                       P_FLGVIR     TFC_BOTIGUES.FLGVIR%type,
                       P_FLGACT     TFC_BOTIGUES.FLGACT%type,
                       P_DATALT     TFC_BOTIGUES.DATALT%type,
                       P_DATBAI     TFC_BOTIGUES.DATBAI%type,
                       RSP          OUT VARCHAR2);
                         
  -- Procediment que marca un resgitre de la taula TFC_BOTIGUES per informar que la botiga ha causat baixa 
  -- Parmetres d' entrada:
  --   P_IDEBOT     => Identificador de la botiga 
  --   P_DATBAI     => Data de baixa de la botiga 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE BAI_BOTIGA(P_IDEBOT TFC_BOTIGUES.IDEBOT%type,
                       P_DATBAI TFC_BOTIGUES.DATBAI%TYPE,
                       RSP      OUT VARCHAR2);

  -- Procediment que elimina registre de la taula TFC_BOTIGUES  
  -- Parmetres d' entrada:
  --   P_IDEBOT     => Identificador de la botiga 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE SUP_BOTIGA(P_IDEBOT TFC_BOTIGUES.IDEBOT%type,
                       RSP      OUT VARCHAR2);

  -- Funci que donat un identificador de botiga recupera l' identificador de ciutats associat   
  -- Parmetres d' entrada:
  --   P_IDEBOT     => Identificador de la botiga 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.                       
  -- Retorn de la funci: Identificador de la ciutat associada 
  FUNCTION OBT_IDECIU(P_IDEBOT TFC_BOTIGUES.IDEBOT%type,
                      RSP      OUT VARCHAR2) RETURN TFC_CIUTATS.IDECIU%TYPE;
end TFC_BOT;
/
create or replace package body TFC_BOT is
  -- Procediment que dna d' alta un registre a la taula TFC_BOTIGUES 
  -- Parmetres d' entrada:
  --   P_IDEBOT     => Identificador de la botiga 
  --   P_NOMBOT     => Nom de la botiga 
  --   P_CIU_IDECIU => Identificador de la ciutat 
  --   P_NUMTRBACT  => Nmero de treballadors de la botiga 
  --   P_EMAIL      => Email de contacte 
  --   P_FLGFRA     => s franqucia? (S/N)
  --     'S'        => s franqucia
  --     'N'        => No s franqucia 
  --   P_FLGVIR     => s virtual? (S/N)
  --     'S'        => s virtual
  --     'N'        => No s virtual
  --   P_FLGACT     => Est activa? (S/N) 
  --     'S'        => Est activa 
  --     'N'        => No est activa
  --   P_DATALT     => Data de alta de la botiga 
  --   P_DATBAI     => Data de baixa de la botiga 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE ALT_BOTIGA(P_IDEBOT     TFC_BOTIGUES.IDEBOT%type,
                       P_NOMBOT     TFC_BOTIGUES.NOMBOT%type,
                       P_CIU_IDECIU TFC_BOTIGUES.CIU_IDECIU%type,
                       P_NUMTRBACT  TFC_BOTIGUES.NUMTRBACT%type,
                       P_EMAIL      TFC_BOTIGUES.EMAIL%type,
                       P_FLGFRA     TFC_BOTIGUES.FLGFRA%type,
                       P_FLGVIR     TFC_BOTIGUES.FLGVIR%type,
                       P_FLGACT     TFC_BOTIGUES.FLGACT%type,
                       P_DATALT     TFC_BOTIGUES.DATALT%type,
                       P_DATBAI     TFC_BOTIGUES.DATBAI%type,
                       RSP          OUT VARCHAR2) IS 
    vNomProces TFC_RASTRE_PROCES.PROCES%Type := 'TFC_BOT.ALT_BOTIGA';
    vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
    FK_CIUTAT exception;
    PRAGMA EXCEPTION_INIT(FK_CIUTAT, -02291);
  BEGIN
    If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
      TFC_RAP.INICIAR_RASTRE(vNomProces);
    end if;
    
    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_IDEBOT='    ||P_IDEBOT);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_NOMBOT='    ||P_NOMBOT);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_CIU_IDECIU='||P_CIU_IDECIU);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_NUMTRBACT=' ||P_NUMTRBACT);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_EMAIL='     ||P_EMAIL);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_FLGFRA='    ||P_FLGFRA);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_FLGVIR='    ||P_FLGVIR);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_FLGACT='    ||P_FLGACT);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_DATALT='    ||TO_CHAR(P_DATALT, 'DD-MM-YYYY'));
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_DATBAI='    ||TO_CHAR(P_DATBAI, 'DD-MM-YYYY'));

    -- S' ha informat la data d' alta?
    If (P_DATALT is null) then 
      RSP := 'No s'' ha informat la data d'' alta per la botiga amb identificador '||P_IDEBOT;
    end if;
    
    If ((RSP is null) or (RSP = TFC_UTL.RSP_OK)) then 
      -- Els producte no est actiu i no s' ha informat la data de baixa 
      If ((P_FLGACT = 'N') and (P_DATBAI is null)) then 
        RSP := 'La botiga amb identificador '||P_IDEBOT||' no est activa i no s'' ha informat la data de baixa';
      end if;
    end if;

    If ((RSP is null) or (RSP = TFC_UTL.RSP_OK)) then
      begin
        -- Es crea el nou registre 
        Insert into TFC_BOTIGUES 
          (IDEBOT,
           NOMBOT,
           CIU_IDECIU,
           NUMTRBACT,
           EMAIL,
           FLGFRA,
           FLGVIR,
           FLGACT,
           DATALT,
           DATBAI)
        values
          (P_IDEBOT,
           P_NOMBOT,
           P_CIU_IDECIU,
           P_NUMTRBACT,
           P_EMAIL,
           P_FLGFRA,
           P_FLGVIR,
           P_FLGACT,
           P_DATALT,
           P_DATBAI);
      
        If (SQL%RowCount > 0) then 
          TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre insertat');
        else
          TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre no insertat');
        end if;

      exception
        WHEN DUP_VAL_ON_INDEX THEN -- Si es captura aquesta excepci vol dir que el registre ja existeix.
          -- En aquest cas s' actualitzaran els valors del registre existent.
          MOD_BOTIGA(P_IDEBOT,
                     P_NOMBOT,
                     P_CIU_IDECIU,
                     P_NUMTRBACT,
                     P_EMAIL,
                     P_FLGFRA,
                     P_FLGVIR,
                     P_FLGACT,
                     P_DATALT,
                     P_DATBAI,
                     RSP);
      end;
    end if;
    
    -- Arribats a aquest punt, tot ha anat b.
    If (RSP is null) then 
      RSP := TFC_UTL.RSP_OK;
    end if;

    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);

    If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
      If (RSP = TFC_UTL.RSP_OK) then 
        TFC_RAP.FINALITZAR_RASTRE_OK;
      else
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    end if;
  EXCEPTION -- Tractament d' excepcions 
    WHEN FK_CIUTAT THEN 
      RSP := 'No existeix cap ciutat amb l'' identificador '||P_CIU_IDECIU;
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    WHEN others THEN 
      RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
  END;

  -- Procediment que modifica un resgitre a la taula TFC_BOTIGUES 
  -- Parmetres d' entrada:
  --   P_IDEBOT     => Identificador de la botiga 
  --   P_NOMBOT     => Nom de la botiga 
  --   P_CIU_IDECIU => Identificador de la ciutat 
  --   P_NUMTRBACT  => Nmero de treballadors de la botiga 
  --   P_EMAIL      => Email de contacte 
  --   P_FLGFRA     => s franqucia? (S/N)
  --     'S'        => s franqucia
  --     'N'        => No s franqucia 
  --   P_FLGVIR     => s virtual? (S/N)
  --     'S'        => s virtual
  --     'N'        => No s virtual
  --   P_FLGACT     => Est activa? (S/N) 
  --     'S'        => Est activa 
  --     'N'        => No est activa
  --   P_DATALT     => Data de alta de la botiga 
  --   P_DATBAI     => Data de baixa de la botiga 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE MOD_BOTIGA(P_IDEBOT     TFC_BOTIGUES.IDEBOT%type,
                       P_NOMBOT     TFC_BOTIGUES.NOMBOT%type,
                       P_CIU_IDECIU TFC_BOTIGUES.CIU_IDECIU%type,
                       P_NUMTRBACT  TFC_BOTIGUES.NUMTRBACT%type,
                       P_EMAIL      TFC_BOTIGUES.EMAIL%type,
                       P_FLGFRA     TFC_BOTIGUES.FLGFRA%type,
                       P_FLGVIR     TFC_BOTIGUES.FLGVIR%type,
                       P_FLGACT     TFC_BOTIGUES.FLGACT%type,
                       P_DATALT     TFC_BOTIGUES.DATALT%type,
                       P_DATBAI     TFC_BOTIGUES.DATBAI%type,
                       RSP          OUT VARCHAR2) IS 
    vNomProces         TFC_RASTRE_PROCES.PROCES%Type := 'TFC_BOT.MOD_BOTIGA';
    vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
    FK_CIUTAT          exception;
    PRAGMA EXCEPTION_INIT(FK_CIUTAT, -02291);
  BEGIN
    If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
      TFC_RAP.INICIAR_RASTRE(vNomProces);
    end if;
    
    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_IDEBOT='||P_IDEBOT);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_NOMBOT='||P_NOMBOT);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_CIU_IDECIU='||P_CIU_IDECIU);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_NUMTRBACT='||P_NUMTRBACT);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_EMAIL='||P_EMAIL);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_FLGFRA='||P_FLGFRA);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_FLGVIR='||P_FLGVIR);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_FLGACT='||P_FLGACT);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_DATALT='||TO_CHAR(P_DATALT, 'DD-MM-YYYY'));
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_DATBAI='||TO_CHAR(P_DATBAI, 'DD-MM-YYYY'));
    
    -- S' ha informat la data d' alta?
    If (P_DATALT is null) then 
      RSP := 'No s'' ha informat la data d'' alta per la botiga amb identificador '||P_IDEBOT;
    end if;
    
    If ((RSP is null) or (RSP = TFC_UTL.RSP_OK)) then 
      -- Els producte no est actiu i no s' ha informat la data de baixa 
      If ((P_FLGACT = 'N') and (P_DATBAI is null)) then 
        RSP := 'La botiga amb identificador '||P_IDEBOT||' no est activa i no s'' ha informat la data de baixa';
      end if;
    end if;

    If ((RSP is null) or (RSP = TFC_UTL.RSP_OK)) then
      -- Es modifica el registre de la taula TFC_BOTIGUES 
      Update TFC_BOTIGUES 
         set NOMBOT     = P_NOMBOT,
             CIU_IDECIU = P_CIU_IDECIU,
             NUMTRBACT  = P_NUMTRBACT,
             EMAIL      = P_EMAIL,
             FLGFRA     = P_FLGFRA,
             FLGVIR     = P_FLGVIR,
             FLGACT     = P_FLGACT,
             DATALT     = P_DATALT,
             DATBAI     = DATBAI
         where IDEBOT   = P_IDEBOT;
       
      If (SQL%RowCount > 0) then 
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre modificat');
        RSP := TFC_UTL.RSP_OK;
      else
        TFC_RAP.REGISTRAR_RASTRE_LINIA('No s'' ha trobat cap registre amb l'' identificador = '||P_IDEBOT);
        RSP := 'No s'' ha trobat cap registre amb l'' identificador = '||P_IDEBOT;
      end if;
    end if;
          
    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);

    If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
      If (RSP = TFC_UTL.RSP_OK) then 
        TFC_RAP.FINALITZAR_RASTRE_OK;
      else
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    end if;
  EXCEPTION -- Tractament d' excepcions 
    WHEN FK_CIUTAT THEN 
      RSP := 'No existeix cap ciutat amb l'' identificador '||P_CIU_IDECIU;
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    WHEN others THEN 
      RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
  END;

  -- Procediment que marca un resgitre de la taula TFC_BOTIGUES per informar que la botiga ha causat baixa 
  -- Parmetres d' entrada:
  --   P_IDEBOT     => Identificador de la botiga 
  --   P_DATBAI     => Data de baixa de la botiga 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.        
  PROCEDURE BAI_BOTIGA(P_IDEBOT TFC_BOTIGUES.IDEBOT%type,
                       P_DATBAI TFC_BOTIGUES.DATBAI%TYPE,
                       RSP      OUT VARCHAR2) IS 
    vNomProces         TFC_RASTRE_PROCES.PROCES%Type := 'TFC_BOT.BAI_BOTIGA';
    vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
  BEGIN
    If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
      TFC_RAP.INICIAR_RASTRE(vNomProces);
    end if;
    
    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_IDEBOT='||P_IDEBOT);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_DATBAI='||TO_CHAR(P_DATBAI, 'DD-MM-YYYY'));
    
    -- Es marca el registre de la taula TFC_BOTIGUES com a baxia 
    Update TFC_BOTIGUES 
       set FLGACT   = 'N',
           DATBAI   = DATBAI
       where IDEBOT = P_IDEBOT and 
             FLGACT = 'S';
             
    If (SQL%RowCount > 0) then 
      RSP := TFC_UTL.RSP_OK;
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre modificat');
    else
      RSP := 'No existeix cap botiga amb l'' identificador '||P_IDEBOT;
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre no modificat');
    end if;
    
    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);

    If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
      If (RSP = TFC_UTL.RSP_OK) then 
        TFC_RAP.FINALITZAR_RASTRE_OK;
      else
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    end if;
  EXCEPTION -- Tractament d' excepcions 
    WHEN others THEN 
      RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
  END;

  -- Procediment que elimina resgitre de la taula TFC_BOTIGUES  
  -- Parmetres d' entrada:
  --   P_IDEBOT     => Identificador de la botiga 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE SUP_BOTIGA(P_IDEBOT TFC_BOTIGUES.IDEBOT%type,
                       RSP      OUT VARCHAR2) IS 
    vNomProces         TFC_RASTRE_PROCES.PROCES%Type := 'TFC_BOT.SUP_BOTIGA';
    vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
    FK_BOTIGA          exception;
    PRAGMA             EXCEPTION_INIT(FK_BOTIGA, -02292);
  BEGIN
    If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
      TFC_RAP.INICIAR_RASTRE(vNomProces);
    end if;
    
    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_IDEBOT='||P_IDEBOT);
    
    Delete 
      from TFC_BOTIGUES 
      where IDEBOT = P_IDEBOT;
      
    If (SQL%RowCount > 0) then 
      RSP := TFC_UTL.RSP_OK;
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre eliminat');
    else
      RSP := 'No existeix cap botiga amb l'' identificador '||P_IDEBOT;
      TFC_RAP.REGISTRAR_RASTRE_LINIA('No existeix cap botiga amb l'' identificador '||P_IDEBOT);
    end if;
    
    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);

    If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
      TFC_RAP.FINALITZAR_RASTRE_OK;
    end if;
  EXCEPTION -- Tractament d' excepcions 
    WHEN FK_BOTIGA THEN 
      RSP := 'No es pot eliminar la botiga amb identificador '||P_IDEBOT||'.';
      If (SQLERRM like '%VEN_BOT_FK%') then 
        RSP := RSP || ' Hi ha vendes associades.';
      elsIf (SQLERRM like '%ABA_BOT_FK%') then 
        RSP := RSP || ' Hi ha clculs agregats associats.';
      elsIf (SQLERRM like '%VBH_BOT_FK%') then 
        RSP := RSP || ' Hi ha clculs histrics associats.';
      end if;
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
     
    WHEN others THEN 
      RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
  END;
  
  -- Funci que donat un identificador de botiga recupera l' identificador de ciutats associat   
  -- Parmetres d' entrada:
  --   P_IDEBOT     => Identificador de la botiga 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.                       
  -- Retorn de la funci: Identificador de la ciutat associada 
  FUNCTION OBT_IDECIU(P_IDEBOT TFC_BOTIGUES.IDEBOT%type,
                      RSP      OUT VARCHAR2) RETURN TFC_CIUTATS.IDECIU%TYPE IS
    vIdeCiu            TFC_BOTIGUES.CIU_IDECIU%Type;
    vNomProces         varchar2(40) := 'TFC_BOT.OBT_IDECIU';
    vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
  BEGIN
    If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
      TFC_RAP.INICIAR_RASTRE(vNomProces);
    end if;
    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_IDEBOT='    ||P_IDEBOT);
  
    Select CIU_IDECIU
      into vIdeCiu 
      from TFC_BOTIGUES
      where IDEBOT = P_IDEBOT;
    
    RSP := TFC_UTL.RSP_OK;

    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
    
    If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.    
      If (RSP = TFC_UTL.RSP_OK) then 
        TFC_RAP.FINALITZAR_RASTRE_OK;
      else
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    end if;
    
    return vIdeCiu;
  EXCEPTION -- Tractament d' excepcions 
    WHEN NO_DATA_FOUND THEN 
      RSP := 'No existeix cap botiga amb l'' identificador '||P_IDEBOT;
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    WHEN others THEN 
      RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
  END;
end TFC_BOT;
/
