create or replace package TFC_CIU is
  -- Procediment que dna d' alta un registre a la taula TFC_CIUTATS 
  -- Parmetres d' entrada:
  --   P_IDECIU => Identificador de la ciutat 
  --   P_NOMCIU => Nom de la ciutat
  --   P_IDEREG => Regi associada a la ciutat 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE ALT_CIUTAT(P_IDECIU TFC_CIUTATS.IDECIU%TYPE,
                       P_NOMCIU TFC_CIUTATS.NOMCIU%TYPE,
                       P_IDEREG TFC_CIUTATS.REG_IDEREG%TYPE,
                       RSP      OUT VARCHAR2);

  -- Procediment que modifica un registre a la taula TFC_CIUTATS 
  -- Parmetres d' entrada:
  --   P_IDECIU => Identificador de la ciutat a modificar 
  --   P_NOMCIU => Nom de la ciutat
  --   P_IDEREG => Regi associada a la ciutat 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE MOD_CIUTAT(P_IDECIU TFC_CIUTATS.IDECIU%TYPE,
                       P_NOMCIU TFC_CIUTATS.NOMCIU%TYPE,
                       P_IDEREG TFC_CIUTATS.REG_IDEREG%TYPE,
                       RSP      OUT VARCHAR2);

  -- Procediment que elimina un registre de la taula TFC_CIUTATS 
  -- Parmetres d' entrada:
  --   P_IDECIU => Identificador de la ciutat a eliminar 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE SUP_CIUTAT(P_IDECIU TFC_CIUTATS.IDECIU%TYPE,
                       RSP      OUT VARCHAR2);

end TFC_CIU;
/
create or replace package body TFC_CIU is
  -- Procediment que dna d' alta un registre a la taula TFC_CIUTATS 
  -- Parmetres d' entrada:
  --   P_IDECIU => Identificador de la ciutat 
  --   P_NOMCIU => Nom de la ciutat
  --   P_IDEREG => Regi associada a la ciutat 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE ALT_CIUTAT(P_IDECIU TFC_CIUTATS.IDECIU%TYPE,
                       P_NOMCIU TFC_CIUTATS.NOMCIU%TYPE,
                       P_IDEREG TFC_CIUTATS.REG_IDEREG%TYPE,
                       RSP      OUT VARCHAR2) IS
    vNomProces         TFC_RASTRE_PROCES.PROCES%Type := 'TFC_CIU.ALT_CIUTAT';
    vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
    vError             Exception;
    vRegio_Null        Exception;
    vRegio_No_Existeix Exception;
    vProces_Error      varchar2(100);
  BEGIN
    If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
      TFC_RAP.INICIAR_RASTRE(vNomProces);
    end if;
    
    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_IDECIU='||P_IDECIU);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_NOMCIU='||P_NOMCIU);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_IDEREG='||P_IDEREG);
    
    If (P_IDEREG is null) then 
      raise vRegio_Null;
    end if;
    
    If (not TFC_REG.EXISTEIX_REGIO(P_IDEREG, RSP)) then 
      raise vRegio_No_Existeix;
    end if;
    
    If (RSP <> TFC_UTL.RSP_OK) then 
      vProces_Error := 'TFC_REG.EXISTEIX_REGIO';
      raise vError;
    end if;
      
    begin 
      -- S' inserta el registre 
      Insert into TFC_CIUTATS
        (IDECIU,
         NOMCIU,
         REG_IDEREG)
      values
        (P_IDECIU,
         P_NOMCIU,
         P_IDEREG);
         
      If (SQL%RowCount > 0) then 
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre insertat.');
      else
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre no insertat.');
      end if;
      
    exception
      WHEN DUP_VAL_ON_INDEX THEN -- Si es captura aquesta excepci vol dir que el registre ja existeix.
        -- En aquest cas s' actualitzaran els valors del registre existent.
        MOD_CIUTAT(P_IDECIU,
                   P_NOMCIU,
                   P_IDEREG,
                   RSP); 
    end;
    
    -- Arribats a aquest punt, tot ha anat b.
    If (RSP is null) then 
      RSP := TFC_UTL.RSP_OK;
    end if;
   
    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);

    If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
      If (RSP = TFC_UTL.RSP_OK) then 
        TFC_RAP.FINALITZAR_RASTRE_OK;
      else
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    end if;
  EXCEPTION -- Tractament d' excepcions 
    when vRegio_Null then 
      RSP := 'El parmetre P_IDEREG no pot ser null.';
      TFC_RAP.REGISTRAR_RASTRE_LINIA('La regi no existeix.');
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    when vRegio_No_Existeix then 
      RSP := 'No existeix cap regi amb l'' identificador '||P_IDEREG;
      TFC_RAP.REGISTRAR_RASTRE_LINIA('La regi no existeix.');
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    WHEN others THEN 
      If (vProces_Error is null) then -- Si vProces_Error es null, es tracta d' un error no controlat 
        RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
      else
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Error executant el procs '||vProces_Error||':');
      end if;
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
  END;

  -- Procediment que modifica un registre a la taula TFC_CIUTATS 
  -- Parmetres d' entrada:
  --   P_IDECIU => Identificador de la ciutat a modificar 
  --   P_NOMCIU => Nom de la ciutat
  --   P_IDEREG => Regi associada a la ciutat 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE MOD_CIUTAT(P_IDECIU TFC_CIUTATS.IDECIU%TYPE,
                       P_NOMCIU TFC_CIUTATS.NOMCIU%TYPE,
                       P_IDEREG TFC_CIUTATS.REG_IDEREG%TYPE,
                       RSP      OUT VARCHAR2) IS
    vNomProces         TFC_RASTRE_PROCES.PROCES%Type := 'TFC_CIU.MOD_CIUTAT';
    vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
    vError             Exception;
    vProces_Error      varchar2(100);
    vRegio_Null        Exception;
    vRegio_No_Existeix Exception;
  BEGIN
    If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
      TFC_RAP.INICIAR_RASTRE(vNomProces);
    end if;
    
    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_IDECIU='||P_IDECIU);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_NOMCIU='||P_NOMCIU);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_IDEREG='||P_IDEREG);
    
    If (P_IDEREG is null) then 
      raise vRegio_Null;
    end if;
    
    If (not TFC_REG.EXISTEIX_REGIO(P_IDEREG, RSP)) then 
      raise vRegio_No_Existeix;
    end if;
    
    -- Es modifica el registre amb els parmetres donats 
    Update TFC_CIUTATS
       set NOMCIU     = P_NOMCIU,
           REG_IDEREG = P_IDEREG
       where IDECIU   = P_IDECIU;

    If (SQL%RowCount > 0) then 
      RSP := TFC_UTL.RSP_OK;
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre modificat.');
    else
      RSP := 'No s'' ha trobat cap ciutat amb l'' identificador '||P_IDECIU;
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre no modificat.');
    end if;

    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);

    If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
      If (RSP = TFC_UTL.RSP_OK) then 
        TFC_RAP.FINALITZAR_RASTRE_OK;
      else
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    end if;
  EXCEPTION -- Tractament d' excepcions 
    when vRegio_Null then 
      RSP := 'El parmetre P_IDEREG no pot ser null.';
      TFC_RAP.REGISTRAR_RASTRE_LINIA('La regi no existeix.');
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    when vRegio_No_Existeix then 
      RSP := 'No existeix cap regi amb l'' identificador '||P_IDEREG;
      TFC_RAP.REGISTRAR_RASTRE_LINIA('La regi no existeix.');
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    WHEN others THEN 
      If (vProces_Error is null) then -- Si vProces_Error es null, es tracta d' un error no controlat 
        RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
      else
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Error executant el procs '||vProces_Error||':');
      end if;
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
  END;

  -- Procediment que elimina un registre de la taula TFC_CIUTATS 
  -- Parmetres d' entrada:
  --   P_IDECIU => Identificador de la ciutat a eliminar 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE SUP_CIUTAT(P_IDECIU TFC_CIUTATS.IDECIU%TYPE,
                       RSP      OUT VARCHAR2) IS
    vNomProces         TFC_RASTRE_PROCES.PROCES%Type := 'TFC_CIU.SUP_CIUTAT';
    vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
    FK_CIUTAT          exception;
    PRAGMA             EXCEPTION_INIT(FK_CIUTAT, -02292);
  BEGIN
    If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
      TFC_RAP.INICIAR_RASTRE(vNomProces);
    end if;
    
    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_IDECIU='||P_IDECIU);
    
    Delete 
      from TFC_CIUTATS
      where IDECIU = P_IDECIU;      
      
    If (SQL%RowCount > 0) then 
      RSP := TFC_UTL.RSP_OK;
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre eliminat');
    else
      RSP := 'No s'' ha trobat cap ciutat amb l'' identificador '||P_IDECIU;
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre no eliminat');
    end if;
    
    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);

    If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
      If (RSP = TFC_UTL.RSP_OK) then 
        TFC_RAP.FINALITZAR_RASTRE_OK;
      else
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    end if;
  EXCEPTION -- Tractament d' excepcions 
    WHEN FK_CIUTAT THEN 
      RSP := 'No es pot eliminar la ciutat amb identificador '||P_IDECIU||'.';
      If (SQLERRM like '%BOT_CIU_FK%') then 
        RSP := RSP || ' Hi ha botigues associades.';
      elsIf (SQLERRM like '%ACA_CIU_FK%') then 
        RSP := RSP || ' Hi ha clculs agregats associats.';
      elsIf (SQLERRM like '%VCH_CIU_FK%') then 
        RSP := RSP || ' Hi ha clculs histrics associats.';
      end if;
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
      
    WHEN others THEN 
      RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
  END;
end TFC_CIU;
/
