create or replace package TFC_INA is

  -- Procediment que dona d' alta un registre a la taula TFC_INDICADORS_ANY  
  -- Parmetres d' entrada:
  --   P_NUMANY => Any per el que es vol calcular els indicadors  
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE ALT_INA(P_NUMANY TFC_INDICADORS_ANY.NUMANY%TYPE,
                    RSP      OUT VARCHAR2);

  -- Procediment que elimina un calcula els indicadors de la taula TFC_INDICADORS_ANY 
  -- Parmetres d' entrada:
  --   P_NUMANY => Any per el que es vol calcular els indicadors  
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE CALC_INA(P_NUMANY TFC_INDICADORS_ANY.NUMANY%TYPE,
                     RSP      OUT VARCHAR2);
                                    
  -- Procediment que calcula els diferents indicadors del mdul estadstic  
  -- El clcul s total  
  -- Parmetres d' entrada:
  --   P_NUMANY => Any per el que es vol calcular els indicadors  
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE CALCULAR_INDICADORS_TOT(P_NUMANY TFC_INDICADORS_ANY.NUMANY%TYPE,
                                    RSP      OUT VARCHAR2);
end TFC_INA;
/
create or replace package body TFC_INA is
  -- Procediment que dona d' alta un registre a la taula TFC_INDICADORS_ANY  
  -- Parmetres d' entrada:
  --   P_NUMANY => Any per el que es vol calcular els indicadors  
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE ALT_INA(P_NUMANY TFC_INDICADORS_ANY.NUMANY%TYPE,
                    RSP      OUT VARCHAR2) IS 
    vDummy varchar2(1);
    vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
    vNomProces varchar2(100) := 'TFC_INA.ALT_INA';
    vError             Exception;
  BEGIN
    If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
      TFC_RAP.INICIAR_RASTRE(vNomProces);
    end if;
    
    -- Es graven els parmetres a la taula de rastre
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_NUMANY='||P_NUMANY);

    BEGIN
      Select '1' 
        into vDummy 
        from TFC_INDICADORS_ANY 
        where numany = P_NUMANY;
        
    exception 
      WHEN NO_DATA_FOUND THEN 
        Insert into TFC_INDICADORS_ANY 
          (NUMANY,
           BENNET,
           PER_BENNET_TENDES_VIRT,
           BENNET_HIS,
           PER_BENNET_TENDES_VIRT_HIS)
        values 
          (P_NUMANY, 
           0,
           0,
           0,
           0);
    END;
    
    -- Arribats a aquest punt, tot ha anat b.
    If (RSP is null) then 
      RSP := TFC_UTL.RSP_OK;
    end if;
    
    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);

    If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
      If (RSP = TFC_UTL.RSP_OK) then 
        TFC_RAP.FINALITZAR_RASTRE_OK;
      else
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    end if;
  EXCEPTION -- Tractament d' excepcions 
    WHEN others THEN 
      RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
  END;

  -- Procediment que elimina un calcula els indicadors de la taula TFC_INDICADORS_ANY 
  -- Parmetres d' entrada:
  --   P_NUMANY => Any per el que es vol calcular els indicadors  
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE CALC_INA(P_NUMANY TFC_INDICADORS_ANY.NUMANY%TYPE,
                     RSP      OUT VARCHAR2) IS
    vBennet_Any        TFC_INDICADORS_ANY.BENNET%Type;
    vBennet_His        TFC_INDICADORS_ANY.BENNET_HIS%Type;
    vBennet_Vir_Any    TFC_INDICADORS_ANY.BENNET%Type;
    vBennet_Vir_His    TFC_INDICADORS_ANY.BENNET_HIS%Type;
    vPer_Bot_Vir_Any   TFC_INDICADORS_ANY.PER_BENNET_TENDES_VIRT%Type     := 0;
    vPer_Bot_Vir_His   TFC_INDICADORS_ANY.PER_BENNET_TENDES_VIRT_HIS%Type := 0;
    vNomProces         TFC_RASTRE_PROCES.PROCES%Type := 'TFC_INA.CALC_INA';
    vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
    vError             Exception;
  BEGIN
    If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
      TFC_RAP.INICIAR_RASTRE(vNomProces);
    end if;
    
    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_NUMANY='||P_NUMANY);
    
    -- S' obtenen els indicaors de botigues consultat la taula d' agregats .
    -- Per tant, taula d' agregats TFC_AGR_BOT_ANY, ha d' estar calculada prviament.
    Select Nvl(Sum(aba.BENNET),0) as BENNET_ANY,
           Nvl(Sum(aba.BENNET_HIS),0) as BENNET_HIS,
           Nvl(Sum(Decode(bot.FLGVIR, 'S', aba.BENNET, 0)), 0) as BENNET_VIR_ANY,
           Nvl(Sum(Decode(bot.FLGVIR, 'S', aba.BENNET_HIS, 0)), 0) as BENNET_VIR_HIS
      into vBennet_Any,
           vBennet_His,
           vBennet_Vir_Any,
           vBennet_Vir_His
      from TFC_AGR_BOT_ANY aba, 
           TFC_BOTIGUES    bot
      where aba.INA_NUMANY  = P_NUMANY and 
            aba.BOT_IDEBOT  = bot.IDEBOT;
            
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Benefici per any = ' ||vBennet_Any);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Benefici histric = '||vBennet_His);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Benefici de botigues virutals per any = ' ||vBennet_Vir_Any);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Benefici de botigues virutals histric = '||vBennet_Vir_His);
      
    -- Es calcula el percentatge de vendes de botigues virtuals en l' any donat.
    If (vBennet_Any > 0) then 
      vPer_Bot_Vir_Any := Round((vBennet_Vir_Any * 100) / vBennet_Any, 2);
    end if;

    -- Es calcula el percentatge de vendes de botigues histriques tenint en compte tots els anys 
    If (vBennet_His > 0) then 
      vPer_Bot_Vir_His := Round((vBennet_Vir_His * 100) / vBennet_His, 2);
    end if;
    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Percentatge benefici botigues virtuals per any = '  ||vPer_Bot_Vir_Any);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Percentatge benefici botigues virtuals histric = ' ||vPer_Bot_Vir_His);
    
    -- S' actualitza la taula d' indicadors 
    Update TFC_INDICADORS_ANY 
       set BENNET                     = vBennet_Any,
           PER_BENNET_TENDES_VIRT     = vPer_Bot_Vir_Any,
           BENNET_HIS                 = vBennet_His,
           PER_BENNET_TENDES_VIRT_HIS = vPer_Bot_Vir_His
       where NUMANY = P_NUMANY;
       
    If (SQL%RowCount = 0) then -- Si no s' ha actualitzat cap fila s que no existeix l' any a la taula i s' ha de crear 
      Insert into TFC_INDICADORS_ANY 
        (NUMANY,
         BENNET,
         PER_BENNET_TENDES_VIRT,
         BENNET_HIS,
         PER_BENNET_TENDES_VIRT_HIS)
      values
        (P_NUMANY,
         vBennet_Any,
         vPer_Bot_Vir_Any,
         vBennet_His,
         vPer_Bot_Vir_His);
    end if;
    
    RSP := TFC_UTL.RSP_OK;

    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);

    If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
      If (RSP = TFC_UTL.RSP_OK) then 
        TFC_RAP.FINALITZAR_RASTRE_OK;
      else
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    end if;
  EXCEPTION -- Tractament d' excepcions 
    WHEN others THEN 
      RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
     
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
     
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
     
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
  end;
  
  -- Procediment que calcula els diferents indicadors del mdul estadstic  
  -- El clcul s total  
  -- Parmetres d' entrada:
  --   P_NUMANY => Any per el que es vol calcular els indicadors  
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE CALCULAR_INDICADORS_TOT(P_NUMANY TFC_INDICADORS_ANY.NUMANY%TYPE,
                                    RSP      OUT VARCHAR2) IS
    vNomProces         TFC_RASTRE_PROCES.PROCES%Type := 'TFC_INA.CALCULAR_INDICADORS_TOT';
    vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
    vError             Exception;
    vProces_Error      varchar2(100);
  BEGIN
    If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
      TFC_RAP.INICIAR_RASTRE(vNomProces);
    end if;
    
    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_NUMANY='||P_NUMANY);
    
    TFC_APA.RECALCULAR_IND_VEN(P_NUMANY, RSP);
    If (RSP <> TFC_UTL.RSP_OK) then 
      vProces_Error := 'TFC_APA.RECALCULAR_IND_VEN';
      raise vError;
    end if;

    TFC_AHA.RECALCULAR_IND_VEN(P_NUMANY, RSP);
    If (RSP <> TFC_UTL.RSP_OK) then 
      vProces_Error := 'TFC_AHA.RECALCULAR_IND_VEN';
      raise vError;
    end if;

    TFC_ADMA.RECALCULAR_IND_VEN(P_NUMANY, RSP);
    If (RSP <> TFC_UTL.RSP_OK) then 
      vProces_Error := 'TFC_ADMA.RECALCULAR_IND_VEN';
      raise vError;
    end if;

    TFC_ACA.RECALCULAR_IND_VEN(P_NUMANY, RSP);
    If (RSP <> TFC_UTL.RSP_OK) then 
      vProces_Error := 'TFC_ACA.REACLCULAR_IND_VEN';
      raise vError;
    end if;

    TFC_ABA.RECALCULAR_IND_VEN(P_NUMANY, RSP);
    If (RSP <> TFC_UTL.RSP_OK) then 
      vProces_Error := 'TFC_ABA.RECALCULAR_IND_VEN';
      raise vError;
    end if;
    
    CALC_INA(P_NUMANY, RSP);
    If (RSP <> TFC_UTL.RSP_OK) then 
      vProces_Error := 'TFC_INA.CALC_INA';
      raise vError;
    end if;
    
    If (RSP is null) then 
      RSP := TFC_UTL.RSP_OK;
    end if;

    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);

    If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
      If (RSP = TFC_UTL.RSP_OK) then 
        TFC_RAP.FINALITZAR_RASTRE_OK;
      else
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    end if;
  EXCEPTION -- Tractament d' excepcions 
    WHEN others THEN 
      If (vProces_Error is null) then -- Si vProces_Error es null, es tracta d' un error no controlat 
        RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
      else
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Error executant el procs '||vProces_Error||':');
      end if;
     
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
  end;
end TFC_INA;
/
