create or replace package TFC_LDR is
  -- Procediment que tracta les dades associades a les regions.
  -- Parmetres d' entrada:
  --   P_TIPUS_ACCIO => Tipus de registres a tractar. Segons el valor:
  --     'ALT'  => Altes 
  --     'MOD'  => Modificacions 
  --     'BAI'  => Baixes 
  -- Parmetres de sortida:
  --   P_NOMFIC_ERR: Si s' ha produt algun error aquest parmetre contindr el nom del fitxer amb els registres que han donat error 
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE TRACTAR_REGIONS(P_TIPUS_ACCIO varchar2,
                            P_NOM_FIC_ERR out varchar2,
                            RSP           out varchar2);

  -- Procediment que tracta les dades associades a les ciutats.
  -- Parmetres d' entrada:
  --   P_TIPUS_ACCIO => Tipus de registres a tractar. Segons el valor:
  --     'ALT'  => Altes 
  --     'MOD'  => Modificacions 
  --     'BAI'  => Baixes 
  -- Parmetres de sortida:
  --   P_NOMFIC_ERR: Si s' ha produt algun error aquest parmetre contindr el nom del fitxer amb els registres que han donat error 
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.  
  PROCEDURE TRACTAR_CIUTATS(P_TIPUS_ACCIO varchar2,
                            P_NOM_FIC_ERR out varchar2,
                            RSP           out varchar2);
  
  -- Procediment que tracta les dades associades als productes .
  -- Parmetres d' entrada:
  --   P_TIPUS_ACCIO => Tipus de registres a tractar. Segons el valor:
  --     'ALT'  => Altes 
  --     'MOD'  => Modificacions 
  --     'BAI'  => Baixes 
  -- Parmetres de sortida:
  --   P_NOMFIC_ERR: Si s' ha produt algun error aquest parmetre contindr el nom del fitxer amb els registres que han donat error 
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE TRACTAR_PRODUCTES(P_TIPUS_ACCIO varchar2,
                              P_NOM_FIC_ERR out varchar2,
                              RSP           out varchar2);
  
  -- Procediment que tracta les dades associades a les botigues .
  -- Parmetres d' entrada:
  --   P_TIPUS_ACCIO => Tipus de registres a tractar. Segons el valor:
  --     'ALT'  => Altes 
  --     'MOD'  => Modificacions 
  --     'BAI'  => Baixes 
  -- Parmetres de sortida:
  --   P_NOMFIC_ERR: Si s' ha produt algun error aquest parmetre contindr el nom del fitxer amb els registres que han donat error 
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE TRACTAR_BOTIGUES(P_TIPUS_ACCIO varchar2,
                             P_NOM_FIC_ERR out varchar2,
                             RSP           out varchar2);
  
  -- Procediment que tracta les dades associades a les vendes .
  -- Parmetres d' entrada:
  --   P_TIPUS_ACCIO => Tipus de registres a tractar. Segons el valor:
  --     'ALT'  => Altes 
  --     'MOD'  => Modificacions 
  --     'BAI'  => Baixes 
  -- Parmetres de sortida:
  --   P_NOMFIC_ERR: Si s' ha produt algun error aquest parmetre contindr el nom del fitxer amb els registres que han donat error 
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE TRACTAR_VENDES(P_TIPUS_ACCIO varchar2,
                           P_NOM_FIC_ERR out varchar2,
                           RSP           out varchar2);

  -- Procediment que tracta els fitxers de dades corresponent a les altes.
  -- Parmetres de sortida:
  --   P_IDERAP => Identificador del rastre de procs generat 
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE PROCESSAR_ALTES(P_IDERAP out TFC_RASTRE_PROCES.IDERAP%Type, 
                            RSP      out varchar2);
  
  -- Procediment que tracta els fitxers de dades corresponent a les modificacions.
  -- Parmetres de sortida:
  --   P_IDERAP => Identificador del rastre de procs generat 
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE PROCESSAR_MODIFICACIONS(P_IDERAP out TFC_RASTRE_PROCES.IDERAP%Type, 
                                    RSP      out varchar2);
  
  -- Procediment que tracta els fitxers de dades corresponent a les baixes.
  -- Parmetres de sortida: 
  --   P_IDERAP => Identificador del rastre de procs generat 
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE PROCESSAR_BAIXES(P_IDERAP out TFC_RASTRE_PROCES.IDERAP%Type, 
                             RSP      out varchar2);
end TFC_LDR;
/
create or replace package body TFC_LDR is

  -- Contant per identificar la operaci de tipus 'altes'  
  cAccio_ALT constant varchar2(3) := 'ALT';
  -- Contant per identificar la operaci de tipus 'modificacions' 
  cAccio_MOD constant varchar2(3) := 'MOD';
  -- Contant per identificar la operaci de tipus 'baixes' 
  cAccio_BAI constant varchar2(3) := 'BAI';
  
  -- Procediment que mou els fitxes de dades pendents a la ubicaci adecuade per les taules externes 
  -- Parmetres d' entrada:
  --   P_ACCIO  => Tipus de registres a tractar. Segons el valor:
  --     'ALT'  => Altes 
  --     'MOD'  => Modificacions 
  --     'BAI'  => Baixes 
  -- Parmetres de sortida:
  --   P_FLGREG => S' han de tractar regions? S/N . Segons el valor:
  --     'S'    => Hi ha regions a tractar 
  --     'N'    => No hi ha regions a tractar 
  --   P_FLGCIU => S' han de tractar ciutats? S/N
  --     'S'    => Hi ha ciutats a tractar 
  --     'N'    => No hi ha ciutats a tractar 
  --   P_FLGPRO => S' han de tractar productes? S/N
  --     'S'    => Hi ha productes a tractar 
  --     'N'    => No hi ha productes a tractar
  --   P_FLGBOT => S' han de tractar botigues? S/N
  --     'S'    => Hi ha botigues a tractar 
  --     'N'    => No hi ha botigues a tractar
  --   P_FLGVEN => S' han de tractar vendes? S/N
  --     'S'    => Hi ha vendes a tractar 
  --     'N'    => No hi ha vendes a tractar
  PROCEDURE PREPARAR_FITXERS_DADES(P_ACCIO VARCHAR2, 
                                   P_FLGREG OUT VARCHAR2, 
                                   P_FLGCIU OUT VARCHAR2, 
                                   P_FLGPRO OUT VARCHAR2, 
                                   P_FLGBOT OUT VARCHAR2, 
                                   P_FLGVEN OUT VARCHAR2) IS 
    vNomFic_Reg_Dad  varchar2(30) := TFC_UTL.OBT_VAL_PAR('NOM_FIC_REG_DAD'); -- Nom del fitxer de dades de les regions.
    vNomFic_Ciu_Dad  varchar2(30) := TFC_UTL.OBT_VAL_PAR('NOM_FIC_CIU_DAD'); -- Nom del fitxer de dades de les ciutats.
    vNomFic_Pro_Dad  varchar2(30) := TFC_UTL.OBT_VAL_PAR('NOM_FIC_PRO_DAD'); -- Nom del fitxer de dades dels productes.
    vNomFic_Bot_Dad  varchar2(30) := TFC_UTL.OBT_VAL_PAR('NOM_FIC_BOT_DAD'); -- Nom del fitxer de dades de les botigues. 
    vNomFic_Ven_Dad  varchar2(30) := TFC_UTL.OBT_VAL_PAR('NOM_FIC_VEN_DAD'); -- Nom del fitxer de dades de les vendes. 
    vDir_BD_Dades    varchar2(30) := TFC_UTL.OBT_VAL_PAR('DIR_BD_DADES');    -- Directori de base de dades on estan els fitxers de dades de les taules externes.
    vDir_BD_PDT_PROC varchar2(30) := TFC_UTL.OBT_VAL_PAR('DIR_BD_PDT_PROC_'||P_ACCIO); -- Directori de base de dades on estan els fitxers pendents de tractar en funci del tipus d' acci (ALT/MOD/BAI) 
  BEGIN
    -- Si existeix un fitxer pendent de regions en la ruta de pendents ... 
    If (TFC_UTL.EXISTEIX_FITXER(vDir_BD_PDT_PROC, vNomFic_Reg_Dad)) then 
      P_FLGREG := 'S'; -- Es marca el flag indicat que hi ha regions pendents 
      -- Es mou el fitxer a la ubicaci adecuada 
      UTL_FILE.frename(vDir_BD_PDT_PROC, 
                       vNomFic_Reg_Dad, 
                       vDir_BD_Dades, 
                       vNomFic_Reg_Dad, 
                       true);
    else
      P_FLGREG := 'N'; -- No hi ha regions pendents 
    end if;

    -- Si existeix un fitxer pendent de ciutats en la ruta de pendents ... 
    If (TFC_UTL.EXISTEIX_FITXER(vDir_BD_PDT_PROC, vNomFic_Ciu_Dad)) then 
      P_FLGCIU := 'S'; -- Es marca el flag indicat que hi ha ciutats pendents
      -- Es mou el fitxer a la ubicaci adecuada 
      UTL_FILE.frename(vDir_BD_PDT_PROC, 
                       vNomFic_Ciu_Dad, 
                       vDir_BD_Dades, 
                       vNomFic_Ciu_Dad, 
                       true);
    else
      P_FLGCIU := 'N'; -- No hi ha ciutats pendents
    end if;

    -- Si existeix un fitxer pendent de productes en la ruta de pendents ... 
    If (TFC_UTL.EXISTEIX_FITXER(vDir_BD_PDT_PROC, vNomFic_Pro_Dad)) then 
      P_FLGPRO := 'S'; -- Es marca el flag indicat que hi ha productes pendents
      -- Es mou el fitxer a la ubicaci adecuada 
      UTL_FILE.frename(vDir_BD_PDT_PROC, 
                       vNomFic_Pro_Dad, 
                       vDir_BD_Dades, 
                       vNomFic_Pro_Dad, 
                       true);
    else
      P_FLGPRO := 'N'; -- No hi ha productes pendents
    end if;
    
    -- Si existeix un fitxer pendent de botigues en la ruta de pendents ... 
    If (TFC_UTL.EXISTEIX_FITXER(vDir_BD_PDT_PROC, vNomFic_Bot_Dad)) then 
      P_FLGBOT := 'S'; -- Es marca el flag indicat que hi ha botigues pendents
      -- Es mou el fitxer a la ubicaci adecuada 
      UTL_FILE.frename(vDir_BD_PDT_PROC, 
                       vNomFic_Bot_Dad, 
                       vDir_BD_Dades, 
                       vNomFic_Bot_Dad, 
                       true);
    else
      P_FLGBOT := 'N'; -- No hi ha botigues pendents
    end if;

    -- Si existeix un fitxer pendent de vendes en la ruta de pendents ... 
    If (TFC_UTL.EXISTEIX_FITXER(vDir_BD_PDT_PROC, vNomFic_Ven_Dad)) then 
      P_FLGVEN := 'S'; -- Es marca el flag indicat que hi ha vendes pendents
      -- Es mou el fitxer a la ubicaci adecuada 
      UTL_FILE.frename(vDir_BD_PDT_PROC, 
                       vNomFic_Ven_Dad, 
                       vDir_BD_Dades, 
                       vNomFic_Ven_Dad, 
                       true);
    else
      P_FLGVEN := 'N'; -- No hi ha vendes pendents
    end if;

  END;
  
  -- Procediment que mou els fitxes de dades a la ubicaci de fitxers processats  
  -- Parmetres d' entrada:
  --   P_ACCIO  => Tipus de registres a tractar. Segons el valor:
  --     'ALT'  => Altes 
  --     'MOD'  => Modificacions 
  --     'BAI'  => Baixes 
  --   P_FLGREG => S' ha de moure el fitxer de regions? S/N . Segons el valor:
  --     'S'    => Hi ha un fitxer a moure 
  --     'N'    => No hi ha cap fitxer a moure  
  --   P_FLGCIU => S' han de moure el fitxer de ciutats? S/N
  --     'S'    => Hi ha un fitxer a moure 
  --     'N'    => No hi ha cap fitxer a moure 
  --   P_FLGPRO => S' han de moure el fitxer de productes? S/N
  --     'S'    => Hi ha un fitxer a moure 
  --     'N'    => No hi ha cap fitxer a moure 
  --   P_FLGBOT => S' han de moure el fitxer de botigues? S/N
  --     'S'    => Hi ha un fitxer a moure 
  --     'N'    => No hi ha cap fitxer a moure 
  --   P_FLGVEN => S' han de moure el fitxer de vendes? S/N
  --     'S'    => Hi ha un fitxer a moure 
  --     'N'    => No hi ha cap fitxer a moure 
  PROCEDURE MOURE_FITXERS_A_PROCESSAT(P_ACCIO VARCHAR2, 
                                      P_FLGREG VARCHAR2, 
                                      P_FLGCIU VARCHAR2, 
                                      P_FLGPRO VARCHAR2, 
                                      P_FLGBOT VARCHAR2, 
                                      P_FLGVEN VARCHAR2) IS 
    vNomFic_Reg_Dad  varchar2(30) := TFC_UTL.OBT_VAL_PAR('NOM_FIC_REG_DAD'); -- Nom del fitxer de dades de les regions.
    vNomFic_Ciu_Dad  varchar2(30) := TFC_UTL.OBT_VAL_PAR('NOM_FIC_CIU_DAD'); -- Nom del fitxer de dades de les ciutats.
    vNomFic_Pro_Dad  varchar2(30) := TFC_UTL.OBT_VAL_PAR('NOM_FIC_PRO_DAD'); -- Nom del fitxer de dades dels productes.
    vNomFic_Bot_Dad  varchar2(30) := TFC_UTL.OBT_VAL_PAR('NOM_FIC_BOT_DAD'); -- Nom del fitxer de dades de les botigues. 
    vNomFic_Ven_Dad  varchar2(30) := TFC_UTL.OBT_VAL_PAR('NOM_FIC_VEN_DAD'); -- Nom del fitxer de dades de les vendes. 
    vDir_BD_Dades    varchar2(30) := TFC_UTL.OBT_VAL_PAR('DIR_BD_DADES');    -- Directori de base de dades on estan els fitxers de dades de les taules externes.
    vDir_BD_PROC_OK  varchar2(30) := TFC_UTL.OBT_VAL_PAR('DIR_BD_PROC_OK_'||P_ACCIO); -- Directori de base de dades on estan els fitxers processats. 
  BEGIN
    -- Si s' indica, es mout el fitxer de regions a la ruta de processats.
    If (P_FLGREG = 'S') then 
      UTL_FILE.frename(vDir_BD_Dades, 
                       vNomFic_Reg_Dad, 
                       vDir_BD_PROC_OK, 
                       TFC_UTL.OBT_NOM_FITXER_UNIC(vDir_BD_PROC_OK, vNomFic_Reg_Dad||'_'||to_char(sysdate, 'YYYYMMDDHH24MISS')));
    end if;

    -- Si s' indica, es mout el fitxer de ciutats a la ruta de processats.
    If (P_FLGCIU = 'S') then 
      UTL_FILE.frename(vDir_BD_Dades, 
                       vNomFic_Ciu_Dad, 
                       vDir_BD_PROC_OK, 
                       TFC_UTL.OBT_NOM_FITXER_UNIC(vDir_BD_PROC_OK, vNomFic_Ciu_Dad||'_'||to_char(sysdate, 'YYYYMMDDHH24MISS')));
    end if;
    
    -- Si s' indica, es mout el fitxer de productes a la ruta de processats.
    If (P_FLGPRO = 'S') then 
      UTL_FILE.frename(vDir_BD_Dades, 
                       vNomFic_PRO_Dad, 
                       vDir_BD_PROC_OK, 
                       TFC_UTL.OBT_NOM_FITXER_UNIC(vDir_BD_PROC_OK, vNomFic_Pro_Dad||'_'||to_char(sysdate, 'YYYYMMDDHH24MISS')));
    end if;
    
    -- Si s' indica, es mout el fitxer de botigues a la ruta de processats.
    If (P_FLGBOT = 'S') then 
      UTL_FILE.frename(vDir_BD_Dades, 
                       vNomFic_Bot_Dad, 
                       vDir_BD_PROC_OK, 
                       TFC_UTL.OBT_NOM_FITXER_UNIC(vDir_BD_PROC_OK, vNomFic_Bot_Dad||'_'||to_char(sysdate, 'YYYYMMDDHH24MISS')));
    end if;
    
    -- Si s' indica, es mout el fitxer de vendes a la ruta de processats.
    If (P_FLGVEN = 'S') then 
      UTL_FILE.frename(vDir_BD_Dades, 
                       vNomFic_Ven_Dad, 
                       vDir_BD_PROC_OK, 
                       TFC_UTL.OBT_NOM_FITXER_UNIC(vDir_BD_PROC_OK, vNomFic_Ven_Dad||'_'||to_char(sysdate, 'YYYYMMDDHH24MISS')));
    end if;
  END;

  -- Procediment que tracta les dades associades a les regions.
  -- Parmetres d' entrada:
  --   P_TIPUS_ACCIO => Tipus de registres a tractar. Segons el valor:
  --     'ALT'  => Altes 
  --     'MOD'  => Modificacions 
  --     'BAI'  => Baixes 
  -- Parmetres de sortida:
  --   P_NOMFIC_ERR: Si s' ha produt algun error aquest parmetre contindr el nom del fitxer amb els registres que han donat error 
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE TRACTAR_REGIONS(P_TIPUS_ACCIO varchar2,
                            P_NOM_FIC_ERR out varchar2,
                            RSP           out varchar2) IS 
    cursor cRegions is 
      Select r.IDEREG,
             r.NOMREG
        from TFC_LDR_REG r;
        
    TYPE t_Regions_TAB IS TABLE OF cRegions%ROWTYPE;
    
    vRegions         t_Regions_TAB := t_Regions_TAB();
    vRsp             varchar2(1000);
    vDir_BD_PROC_ERR varchar2(30) := TFC_UTL.OBT_VAL_PAR('DIR_BD_PROC_ERR_'||P_TIPUS_ACCIO); -- Directori de BD on es guarden els errors 
    vNomFic_Reg_Dad  varchar2(30) := TFC_UTL.OBT_VAL_PAR('NOM_FIC_REG_DAD'); -- Nom del fitxer de dades de les regions.
    vId_Fitxer       UTL_FILE.file_type;
    vNum_Err         number := 0;
  begin 
    open cRegions;
    loop
      fetch cRegions bulk collect into vRegions limit 1000;
      
      For i in 1..vRegions.count loop
        SavePoint SP1;
        
        -- Es tracten els registres en funci de la tipologia d' acci a realitzar (ALT/MOD/BAI) 
        If    (P_TIPUS_ACCIO = cAccio_ALT) then -- Tractament d' altes 
          TFC_REG.ALT_REGIO(vRegions(i).IDEREG, vRegions(i).NOMREG, vRsp);
        elsIf (P_TIPUS_ACCIO = cAccio_MOD) then -- Tractament de modificacions 
          TFC_REG.MOD_REGIO(vRegions(i).IDEREG, vRegions(i).NOMREG, vRsp);
        elsIf (P_TIPUS_ACCIO = cAccio_BAI) then -- Tractament de baixes 
          TFC_REG.SUP_REGIO(vRegions(i).IDEREG, vRsp);
        end if;
        
        -- En cas que s' hagi produt un error, el registre tracta es desa en el fitxer d' error per tal que pugui ser revisat a posteriori 
        If (vRsp <> TFC_UTL.RSP_OK) then 
          rollback to SP1;
          If (vNum_Err = 0) then -- Com que es tracta del primer error, s' ha de crear el fitxer 
            P_NOM_FIC_ERR := TFC_UTL.OBT_NOM_FITXER_UNIC(vDir_BD_PROC_ERR, vNomFic_Reg_Dad||'_'||to_char(sysdate, 'YYYYMMDDHH24MISS'));
            vId_Fitxer := UTL_FILE.fopen(vDir_BD_PROC_ERR, P_NOM_FIC_ERR, 'W');
          end if;
          
          vNum_Err := vNum_Err + 1;
          
          -- Es guarden les dades en el fitxer d' errors 
          UTL_FILE.put_line(vId_Fitxer, vRegions(i).IDEREG||'|'||vRegions(i).NOMREG);          
        end if;
      end loop;
      
      commit;
    exit when cRegions%NotFound;
    end loop;
    close cRegions;
    
    If (vNum_Err = 0) then -- Si no hi ha errors el retorn RSP ser correctes 
      RSP := TFC_UTL.RSP_OK;
    else -- S' han produt error i per tant s' informa del fet. 
      RSP := 'Errors tranctant les regions.';
      If (UTL_FILE.is_open(vId_Fitxer)) then -- Es tanca el fitxer d' errors 
        UTL_FILE.fclose(vId_Fitxer);
      end if;      
    end if;
  end;
  
  -- Procediment que tracta les dades associades a les ciutats.
  -- Parmetres d' entrada:
  --   P_TIPUS_ACCIO => Tipus de registres a tractar. Segons el valor:
  --     'ALT'  => Altes 
  --     'MOD'  => Modificacions 
  --     'BAI'  => Baixes 
  -- Parmetres de sortida:
  --   P_NOMFIC_ERR: Si s' ha produt algun error aquest parmetre contindr el nom del fitxer amb els registres que han donat error 
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.  
  PROCEDURE TRACTAR_CIUTATS(P_TIPUS_ACCIO varchar2,
                            P_NOM_FIC_ERR out varchar2,
                            RSP           out varchar2) IS 
    cursor cCiutats is 
      Select c.Ideciu,
             c.nomciu,
             c.reg_idereg 
        from TFC_LDR_CIU c;
        
    TYPE t_Ciutats_TAB IS TABLE OF cCiutats%ROWTYPE;
    
    vCiutats         t_Ciutats_TAB := t_Ciutats_TAB();
    vRsp             varchar2(1000); 
    vDir_BD_PROC_ERR varchar2(30) := TFC_UTL.OBT_VAL_PAR('DIR_BD_PROC_ERR_'||P_TIPUS_ACCIO); -- Directori de BD on es guarden els errors 
    vNomFic_Ciu_Dad  varchar2(30) := TFC_UTL.OBT_VAL_PAR('NOM_FIC_CIU_DAD'); -- Nom del fitxer de dades de les ciutats.
    vId_Fitxer       UTL_FILE.file_type;
    vNum_Err         number := 0;
  begin
    open cCiutats;
    loop
      fetch cCiutats bulk collect into vCiutats limit 1000;
      
      For i in 1..vCiutats.count loop
        SavePoint SP1;
        
        -- Es tracten els registres en funci de la tipologia d' acci a realitzar (ALT/MOD/BAI) 
        If    (P_TIPUS_ACCIO = cAccio_ALT) then -- Tractament d' altes
          TFC_CIU.ALT_CIUTAT(vCiutats(i).IDECIU, vCiutats(i).NOMCIU, vCiutats(i).reg_idereg, vRsp);
        elsIf (P_TIPUS_ACCIO = cAccio_MOD) then -- Tractament de modificacions 
          TFC_CIU.MOD_CIUTAT(vCiutats(i).IDECIU, vCiutats(i).NOMCIU, vCiutats(i).reg_idereg, vRsp);
        elsIf (P_TIPUS_ACCIO = cAccio_BAI) then -- Tractament de baixes 
          TFC_CIU.SUP_CIUTAT(vCiutats(i).IDECIU, vRsp);
        end if;
        
        -- En cas que s' hagi produt un error, el registre tracta es desa en el fitxer d' error per tal que pugui ser revisat a posteriori 
        If (vRsp <> TFC_UTL.RSP_OK) then 
          rollback to SP1;
          
          If (vNum_Err = 0) then -- Com que es tracta del primer error, s' ha de crear el fitxer
            P_NOM_FIC_ERR := TFC_UTL.OBT_NOM_FITXER_UNIC(vDir_BD_PROC_ERR, vNomFic_Ciu_Dad||'_'||to_char(sysdate, 'YYYYMMDDHH24MISS'));
            vId_Fitxer := UTL_FILE.fopen(vDir_BD_PROC_ERR, P_NOM_FIC_ERR, 'W');
          end if;
          
          vNum_Err := vNum_Err + 1;
          
          -- Es guarden les dades en el fitxer d' errors 
          UTL_FILE.put_line(vId_Fitxer, vCiutats(i).IDECIU||'|'||vCiutats(i).NOMCIU||'|'||vCiutats(i).reg_idereg);          
        end if;
      end loop;
      
      commit;
    exit when cCiutats%NotFound;
    end loop;
    close cCiutats;
    
    If (vNum_Err = 0) then -- Si no hi ha errors el retorn RSP ser correctes 
      RSP := TFC_UTL.RSP_OK;
    else -- S' han produt error i per tant s' informa del fet. 
      RSP := 'Errors tranctant les ciutats.';
      If (UTL_FILE.is_open(vId_Fitxer)) then -- Es tanca el fitxer d' errors 
        UTL_FILE.fclose(vId_Fitxer);
      end if;      
    end if;
  end;
  
  -- Procediment que tracta les dades associades als productes .
  -- Parmetres d' entrada:
  --   P_TIPUS_ACCIO => Tipus de registres a tractar. Segons el valor:
  --     'ALT'  => Altes 
  --     'MOD'  => Modificacions 
  --     'BAI'  => Baixes 
  -- Parmetres de sortida:
  --   P_NOMFIC_ERR: Si s' ha produt algun error aquest parmetre contindr el nom del fitxer amb els registres que han donat error 
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE TRACTAR_PRODUCTES(P_TIPUS_ACCIO varchar2,
                              P_NOM_FIC_ERR out varchar2,
                              RSP           out varchar2) IS 
    cursor cProductes is 
      Select p.codean13,
             p.nompro,
             p.descripcio,
             p.flgact,
             p.datalt,
             p.datbai  
        from TFC_LDR_PRO p;
        
    TYPE t_Productes_TAB IS TABLE OF cProductes%ROWTYPE;
    
    vProductes       t_Productes_TAB := t_Productes_TAB();
    vRsp             varchar2(1000);
    vDir_BD_PROC_ERR varchar2(30) := TFC_UTL.OBT_VAL_PAR('DIR_BD_PROC_ERR_'||P_TIPUS_ACCIO); -- Directori de BD on es guarden els errors 
    vNomFic_Pro_Dad  varchar2(30) := TFC_UTL.OBT_VAL_PAR('NOM_FIC_PRO_DAD'); -- Nom del fitxer de dades dels productes.
    vId_Fitxer       UTL_FILE.file_type;
    vNum_Err         number := 0;
  begin
    open cProductes;
    loop
      fetch cProductes bulk collect into vProductes limit 1000;
      
      For i in 1..vProductes.count loop
        SavePoint SP1;
        
        -- Es tracten els registres en funci de la tipologia d' acci a realitzar (ALT/MOD/BAI) 
        If    (P_TIPUS_ACCIO = cAccio_ALT) then -- Tractament d' altes
          TFC_PRO.ALT_PRODUCTE(vProductes(i).CODEAN13, 
                               vProductes(i).NOMPRO, 
                               vProductes(i).DESCRIPCIO, 
                               vProductes(i).FLGACT,
                               vProductes(i).DATALT, 
                               vProductes(i).DATBAI, 
                               vRsp);
        elsIf (P_TIPUS_ACCIO = cAccio_MOD) then -- Tractament de modificacions
          TFC_PRO.MOD_PRODUCTE(vProductes(i).CODEAN13, 
                               vProductes(i).NOMPRO, 
                               vProductes(i).DESCRIPCIO, 
                               vProductes(i).FLGACT,
                               vProductes(i).DATALT, 
                               vProductes(i).DATBAI, 
                               vRsp);
        elsIf (P_TIPUS_ACCIO = cAccio_BAI) then -- Tractament de baixes 
          TFC_PRO.BAI_PRODUCTE(vProductes(i).CODEAN13,
                               vProductes(i).DATBAI, 
                               vRsp);
        end if;
        
        -- En cas que s' hagi produt un error, el registre tracta es desa en el fitxer d' error per tal que pugui ser revisat a posteriori 
        If (vRsp <> TFC_UTL.RSP_OK) then 
          rollback to SP1;
          If (vNum_Err = 0) then -- Com que es tracta del primer error, s' ha de crear el fitxer 
            P_NOM_FIC_ERR := TFC_UTL.OBT_NOM_FITXER_UNIC(vDir_BD_PROC_ERR, vNomFic_Pro_Dad||'_'||to_char(sysdate, 'YYYYMMDDHH24MISS'));
            vId_Fitxer := UTL_FILE.fopen(vDir_BD_PROC_ERR, P_NOM_FIC_ERR, 'W');
          end if;
          
          vNum_Err := vNum_Err + 1;
          
          -- Es guarden les dades en el fitxer d' errors 
          UTL_FILE.put_line(vId_Fitxer, vProductes(i).CODEAN13||'|'||
                                        vProductes(i).NOMPRO||'|'||
                                        vProductes(i).DESCRIPCIO||'|'||
                                        vProductes(i).FLGACT||'|'||
                                        to_char(vProductes(i).DATALT, 'YYYYMMDD')||'|'||
                                        to_char(vProductes(i).DATBAI, 'YYYYMMDD'));          
        end if;
      end loop;
      
      commit;
    exit when cProductes%NotFound;
    end loop;
    close cProductes;
    
    If (vNum_Err = 0) then 
      RSP := TFC_UTL.RSP_OK;
    else -- S' han produt error i per tant s' informa del fet.
      RSP := 'Errors tranctant els productes.';
      If (UTL_FILE.is_open(vId_Fitxer)) then 
        UTL_FILE.fclose(vId_Fitxer); -- Es tanca el fitxer d' errors
      end if;      
    end if;
  end;
  
  -- Parmetres d' entrada:
  --   P_TIPUS_ACCIO => Tipus de registres a tractar. Segons el valor:
  --     'ALT'  => Altes 
  --     'MOD'  => Modificacions 
  --     'BAI'  => Baixes 
  -- Parmetres de sortida:
  --   P_NOMFIC_ERR: Si s' ha produt algun error aquest parmetre contindr el nom del fitxer amb els registres que han donat error 
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE TRACTAR_BOTIGUES(P_TIPUS_ACCIO varchar2,
                             P_NOM_FIC_ERR out varchar2,
                             RSP           out varchar2) IS 
    cursor cBotigues is 
      Select b.IDEBOT,
             b.nombot,
             b.Ciu_Ideciu,
             b.numtrbact,
             b.email,
             b.flgfra,
             b.flgvir,
             b.flgact,
             b.datalt,
             b.datbai
        from TFC_LDR_BOT b;
        
    TYPE t_Botigues_TAB IS TABLE OF cBotigues%ROWTYPE;
    
    vBotigues        t_Botigues_TAB := t_Botigues_TAB();
    vRsp             varchar2(1000);
    vDir_BD_PROC_ERR varchar2(30) := TFC_UTL.OBT_VAL_PAR('DIR_BD_PROC_ERR_'||P_TIPUS_ACCIO); -- Directori de BD on es guarden els errors 
    vNomFic_Bot_Dad  varchar2(30) := TFC_UTL.OBT_VAL_PAR('NOM_FIC_BOT_DAD'); -- Nom del fitxer de dades de les botigues .
    vId_Fitxer       UTL_FILE.file_type;
    vNum_Err         number := 0;
  begin
    open cBotigues;
    loop
      fetch cBotigues bulk collect into vBotigues limit 1000;
      
      For i in 1..vBotigues.count loop
        SavePoint SP1;
        
        -- Es tracten els registres en funci de la tipologia d' acci a realitzar (ALT/MOD/BAI) 
        If    (P_TIPUS_ACCIO = cAccio_ALT) then -- Tractament d' altes 
          TFC_BOT.ALT_BOTIGA(vBotigues(i).IDEBOT,
                             vBotigues(i).NOMBOT,
                             vBotigues(i).CIU_IDECIU,
                             vBotigues(i).NUMTRBACT,
                             vBotigues(i).EMAIL,
                             vBotigues(i).FLGFRA,
                             vBotigues(i).FLGVIR,
                             vBotigues(i).FLGACT,
                             vBotigues(i).DATALT,
                             vBotigues(i).DATBAI, 
                             vRsp);
        elsIf (P_TIPUS_ACCIO = cAccio_MOD) then -- Tractament de modificacions 
          TFC_BOT.MOD_BOTIGA(vBotigues(i).IDEBOT,
                             vBotigues(i).NOMBOT,
                             vBotigues(i).CIU_IDECIU,
                             vBotigues(i).NUMTRBACT,
                             vBotigues(i).EMAIL,
                             vBotigues(i).FLGFRA,
                             vBotigues(i).FLGVIR,
                             vBotigues(i).FLGACT,
                             vBotigues(i).DATALT,
                             vBotigues(i).DATBAI, 
                             vRsp);
        elsIf (P_TIPUS_ACCIO = cAccio_BAI) then -- Tractament de baixes 
          TFC_BOT.BAI_BOTIGA(vBotigues(i).IDEBOT,
                             vBotigues(i).DATBAI, 
                             vRsp);
        end if;
        
        -- En cas que s' hagi produt un error, el registre tracta es desa en el fitxer d' error per tal que pugui ser revisat a posteriori 
        If (vRsp <> TFC_UTL.RSP_OK) then 
          rollback to SP1;
          
          If (vNum_Err = 0) then  -- Com que es tracta del primer error, s' ha de crear el fitxer
            P_NOM_FIC_ERR := TFC_UTL.OBT_NOM_FITXER_UNIC(vDir_BD_PROC_ERR, vNomFic_Bot_Dad||'_'||to_char(sysdate, 'YYYYMMDDHH24MISS'));
            vId_Fitxer := UTL_FILE.fopen(vDir_BD_PROC_ERR, P_NOM_FIC_ERR, 'W');
          end if;
          
          vNum_Err := vNum_Err + 1;
          
          -- Es guarden les dades en el fitxer d' errors 
          UTL_FILE.put_line(vId_Fitxer, vBotigues(i).IDEBOT||'|'||
                                        vBotigues(i).NOMBOT||'|'||
                                        vBotigues(i).CIU_IDECIU||'|'||
                                        vBotigues(i).NUMTRBACT||'|'||
                                        vBotigues(i).EMAIL||'|'||
                                        vBotigues(i).FLGFRA||'|'||
                                        vBotigues(i).FLGVIR||'|'||
                                        vBotigues(i).FLGACT||'|'||
                                        to_char(vBotigues(i).DATALT, 'YYYYMMDD')||'|'||
                                        to_char(vBotigues(i).DATBAI, 'YYYYMMDD'));          
        end if;
      end loop;
      
      commit;
    exit when cBotigues%NotFound;
    end loop;
    close cBotigues;
    
    If (vNum_Err = 0) then 
      RSP := TFC_UTL.RSP_OK;
    else -- S' han produt error i per tant s' informa del fet.
      RSP := 'Errors tranctant les botigues.';
      If (UTL_FILE.is_open(vId_Fitxer)) then 
        UTL_FILE.fclose(vId_Fitxer); -- Es tanca el fitxer d' errors
      end if;      
    end if;
  END;
  
  -- Procediment que tracta les dades associades a les regions.
  -- Parmetres d' entrada:
  --   P_TIPUS_ACCIO => Tipus de registres a tractar. Segons el valor:
  --     'ALT'  => Altes 
  --     'MOD'  => Modificacions 
  --     'BAI'  => Baixes 
  -- Parmetres de sortida:
  --   P_NOMFIC_ERR: Si s' ha produt algun error aquest parmetre contindr el nom del fitxer amb els registres que han donat error 
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE TRACTAR_VENDES(P_TIPUS_ACCIO varchar2,
                           P_NOM_FIC_ERR out varchar2,
                           RSP           out varchar2) IS
    cursor cVendes is 
      Select v.ven_datahora,
             v.bot_idebot,
             v.pro_codean13,
             v.qntven,
             v.bennet,
             v.impbrut
        from TFC_LDR_VEN v;
        
    TYPE t_Vendes_TAB IS TABLE OF cVendes%ROWTYPE;
    
    vVendes          t_Vendes_TAB := t_Vendes_TAB();
    vRsp             varchar2(1000);
    vDir_BD_PROC_ERR varchar2(30) := TFC_UTL.OBT_VAL_PAR('DIR_BD_PROC_ERR_'||P_TIPUS_ACCIO); -- Directori de BD on es guarden els errors 
    vNomFic_Ven_Dad  varchar2(30) := TFC_UTL.OBT_VAL_PAR('NOM_FIC_VEN_DAD'); -- Nom del fitxer de dades de les vendes .
    vId_Fitxer       UTL_FILE.file_type;
    vNum_Err         number := 0;
  begin
    open cVendes;
    loop
      fetch cVendes bulk collect into vVendes limit 1000;
      
      For i in 1..vVendes.count loop
        SavePoint SP1;
        
        -- Es tracten els registres en funci de la tipologia d' acci a realitzar (ALT/MOD/BAI)
        If    (P_TIPUS_ACCIO in (cAccio_ALT, cAccio_BAI)) then -- Tractament d' altes
          TFC_VEN.ALT_VENDA(trunc(vVendes(i).ven_datahora),
                            to_number(to_char(trunc(vVendes(i).ven_datahora, 'HH24'),'HH24')),
                            vVendes(i).bot_IDEBOT,
                            vVendes(i).pro_CODEAN13,
                            vVendes(i).QNTVEN,
                            vVendes(i).BENNET,
                            vVendes(i).IMPBRUT,
                            vRSP);
        elsIf (P_TIPUS_ACCIO = cAccio_MOD) then -- Tractament de modificacions 
          TFC_VEN.MOD_VENDA(trunc(vVendes(i).ven_datahora),
                            to_number(to_char(trunc(vVendes(i).ven_datahora, 'HH24'),'HH24')),
                            vVendes(i).bot_IDEBOT,
                            vVendes(i).pro_CODEAN13,
                            vVendes(i).QNTVEN,
                            vVendes(i).BENNET,
                            vVendes(i).IMPBRUT,
                            vRSP);
        end if;
                          
        -- En cas que s' hagi produt un error, el registre tracta es desa en el fitxer d' error per tal que pugui ser revisat a posteriori 
        If (vRsp <> TFC_UTL.RSP_OK) then 
          rollback to SP1;
          
          If (vNum_Err = 0) then  -- Com que es tracta del primer error, s' ha de crear el fitxer
            P_NOM_FIC_ERR := TFC_UTL.OBT_NOM_FITXER_UNIC(vDir_BD_PROC_ERR, vNomFic_Ven_Dad||'_'||to_char(sysdate, 'YYYYMMDDHH24MISS'));
            vId_Fitxer := UTL_FILE.fopen(vDir_BD_PROC_ERR, P_NOM_FIC_ERR, 'W');
          end if;
          
          vNum_Err := vNum_Err + 1;
          
          -- Es guarden les dades en el fitxer d' errors 
          UTL_FILE.put_line(vId_Fitxer, to_char(vVendes(i).ven_datahora, 'YYYYMMDDHH24MISS')||'|'||
                                        vVendes(i).bot_IDEBOT||'|'||
                                        vVendes(i).pro_CODEAN13||'|'||
                                        vVendes(i).QNTVEN||'|'||
                                        vVendes(i).BENNET||'|'||
                                        vVendes(i).IMPBRUT);          
        end if;
      end loop;
      
      commit;
    exit when cVendes%NotFound;
    end loop;
    close cVendes;
    
    If (vNum_Err = 0) then 
      RSP := TFC_UTL.RSP_OK;
    else -- S' han produt error i per tant s' informa del fet.
      RSP := 'Errors tranctant les vendes.';
      If (UTL_FILE.is_open(vId_Fitxer)) then 
        UTL_FILE.fclose(vId_Fitxer); -- Es tanca el fitxer d' errors
      end if;      
    end if;
  END;
  
  -- Procediment que mou els fitxes de dades a la ubicaci de fitxers processats  
  -- Parmetres d' entrada:
  --   P_TIPUS   => Tipus de registres a tractar. Segons el valor:
  --     'ALT'  => Altes 
  --     'MOD'  => Modificacions 
  --     'BAI'  => Baixes
  -- Parmetres de sortida: 
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE TRACTAR_PDT_PROC(P_TIPUS varchar2,
                             RSP     out varchar2) IS 
    vFlgReg varchar2(1);
    vFlgCiu varchar2(1);
    vFlgPro varchar2(1);
    vFlgBot varchar2(1);
    vFlgVen varchar2(1);
    vRSP_REG varchar2(500);
    vRSP_CIU varchar2(500);
    vRSP_PRO varchar2(500);
    vRSP_BOT varchar2(500);
    vRSP_VEN varchar2(500);
    vNomFic_ERR_REG varchar2(100);
    vNomFic_ERR_CIU varchar2(100);
    vNomFic_ERR_PRO varchar2(100);
    vNomFic_ERR_BOT varchar2(100);
    vNomFic_ERR_VEN varchar2(100);
  BEGIN
    -- Es revisa quins fitxers hi ha pendents de processar ... 
    PREPARAR_FITXERS_DADES(P_TIPUS, 
                           vFlgReg, 
                           vFlgCiu, 
                           vFlgPro, 
                           vFlgBot, 
                           vFlgVen);

    If (vFlgReg = 'S') then -- Si hi ha regions a processar ... 
      -- Tractament de les regions pendents 
      TRACTAR_REGIONS(P_TIPUS, 
                      vNomFic_ERR_REG,
                      vRSP_REG);
    end if;

    If (vFlgCiu = 'S') then -- Si hi ha ciutats a processar ... 
      -- Tractament de les ciutats pendents 
      TRACTAR_CIUTATS(P_TIPUS, 
                      vNomFic_ERR_CIU,
                      vRSP_CIU);
    end if;

    If (vFlgPro = 'S') then -- Si hi ha productes a processar ... 
      -- Tractament dels productes pendents 
      TRACTAR_PRODUCTES(P_TIPUS, 
                        vNomFic_ERR_PRO,
                        vRSP_PRO);
    end if;
    
    If (vFlgBot = 'S') then -- Si hi ha botigues a processar ... 
      -- Tractament de les botigues pendents 
      TRACTAR_BOTIGUES(P_TIPUS, 
                       vNomFic_ERR_BOT,
                       vRSP_BOT);
    end if;
    
    If (vFlgVen = 'S') then -- Si hi ha vendes a processar ... 
      -- Tractament de les vendes pendents ... 
      TRACTAR_VENDES(P_TIPUS, 
                     vNomFic_ERR_VEN,
                     vRSP_VEN);
    end if;
    
    -- Es mouen els fitxers tractas a la ruta de fitxers processats.
    MOURE_FITXERS_A_PROCESSAT(P_TIPUS, 
                              vFlgReg, 
                              vFlgCiu, 
                              vFlgPro, 
                              vFlgBot, 
                              vFlgVen);
                              
    If (vRSP_REG <> TFC_UTL.RSP_OK) then 
      RSP := RSP||' '||vRSP_REG;
    end if;
    
    If (vRSP_CIU <> TFC_UTL.RSP_OK) then 
      RSP := RSP||' '||vRSP_CIU;
    end if;
    
    If (vRSP_PRO <> TFC_UTL.RSP_OK) then 
      RSP := RSP||' '||vRSP_PRO;
    end if;
    
    If (vRSP_BOT <> TFC_UTL.RSP_OK) then 
      RSP := RSP||' '||vRSP_BOT;
    end if;
    
    If (vRSP_VEN <> TFC_UTL.RSP_OK) then 
      RSP := RSP||' '||vRSP_VEN;
    end if;
    
    If (RSP is null) then 
      RSP := TFC_UTL.RSP_OK;
    end if;
  END;
  
  -- Procediment que tracta els fitxers de dades corresponent a les altes.
  -- Parmetres de sortida:
  --   P_IDERAP => Identificador del rastre de procs generat 
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE PROCESSAR_ALTES(P_IDERAP out TFC_RASTRE_PROCES.IDERAP%Type, 
                            RSP      out varchar2) IS
  BEGIN
    TFC_RAP.INICIAR_RASTRE('TFC_LDR.PROCESSAR_ALTES');
    P_IDERAP := TFC_RAP.OBT_IDERAP;

    TRACTAR_PDT_PROC(cAccio_ALT, RSP);
    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP = '||RSP);
    
    If (RSP = TFC_UTL.RSP_OK) then 
      TFC_RAP.FINALITZAR_RASTRE_OK;
    else
      TFC_RAP.FINALITZAR_RASTRE_ERR;
    end if;
  END;
  
  -- Procediment que tracta els fitxers de dades corresponent a les modificacions.
  -- Parmetres de sortida:
  --   P_IDERAP => Identificador del rastre de procs generat 
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE PROCESSAR_MODIFICACIONS(P_IDERAP out TFC_RASTRE_PROCES.IDERAP%Type, 
                                    RSP      out varchar2) IS
  BEGIN
    TFC_RAP.INICIAR_RASTRE('TFC_LDR.PROCESSAR_MODIFICACIONS');
    P_IDERAP := TFC_RAP.OBT_IDERAP;
  
    TRACTAR_PDT_PROC(cAccio_MOD, RSP);
    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP = '||RSP);

    If (RSP = TFC_UTL.RSP_OK) then 
      TFC_RAP.FINALITZAR_RASTRE_OK;
    else
      TFC_RAP.FINALITZAR_RASTRE_ERR;
    end if;
  END;
  
  -- Procediment que tracta els fitxers de dades corresponent a les baixes.
  -- Parmetres de sortida: 
  --   P_IDERAP => Identificador del rastre de procs generat 
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE PROCESSAR_BAIXES(P_IDERAP out TFC_RASTRE_PROCES.IDERAP%Type, 
                             RSP      out varchar2) IS
  BEGIN
    TFC_RAP.INICIAR_RASTRE('TFC_LDR.PROCESSAR_BAIXES');
    P_IDERAP := TFC_RAP.OBT_IDERAP;

    TRACTAR_PDT_PROC(cAccio_BAI, RSP);
    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP = '||RSP);
    
    If (RSP = TFC_UTL.RSP_OK) then 
      TFC_RAP.FINALITZAR_RASTRE_OK;
    else
      TFC_RAP.FINALITZAR_RASTRE_ERR;
    end if;
  END;
end TFC_LDR;
/
