create or replace package TFC_PRO is
  -- Procediment que inserta un registre a la taula TFC_PRODUCTES  
  -- Parmetres d' entrada:
  --   P_CODEAN13   => Codi de barres del producte 
  --   P_NOMPRO     => Nom del producte 
  --   P_DESCRIPCIO => Descripci del producte 
  --   P_FLGACT     => Producte actiu? En funci del valor:
  --     'S'        => El producte est actiu
  --     'N'        => El producte no est actiu 
  --   P_DATALT     => Data de alta del producte 
  --   P_DATBAI     => Data de baixa del producte 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE ALT_PRODUCTE(P_CODEAN13   TFC_PRODUCTES.CODEAN13%TYPE,
                         P_NOMPRO     TFC_PRODUCTES.NOMPRO%TYPE,
                         P_DESCRIPCIO TFC_PRODUCTES.DESCRIPCIO%TYPE,
                         P_FLGACT     TFC_PRODUCTES.FLGACT%TYPE,
                         P_DATALT     TFC_PRODUCTES.DATALT%TYPE,
                         P_DATBAI     TFC_PRODUCTES.DATBAI%TYPE,
                         RSP          OUT VARCHAR2);

  -- Procediment que modifica un registre de la taula TFC_PRODUCTES  
  -- Parmetres d' entrada:
  --   P_CODEAN13   => Codi de barres del producte 
  --   P_NOMPRO     => Nom del producte 
  --   P_DESCRIPCIO => Descripci del producte 
  --   P_FLGACT     => Producte actiu? En funci del valor:
  --     'S'        => El producte est actiu
  --     'N'        => El producte no est actiu 
  --   P_DATALT     => Data de alta del producte 
  --   P_DATBAI     => Data de baixa del producte 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE MOD_PRODUCTE(P_CODEAN13   TFC_PRODUCTES.CODEAN13%TYPE,
                         P_NOMPRO     TFC_PRODUCTES.NOMPRO%TYPE,
                         P_DESCRIPCIO TFC_PRODUCTES.DESCRIPCIO%TYPE,
                         P_FLGACT     TFC_PRODUCTES.FLGACT%TYPE,
                         P_DATALT     TFC_PRODUCTES.DATALT%TYPE,
                         P_DATBAI     TFC_PRODUCTES.DATBAI%TYPE,
                         RSP          OUT VARCHAR2);

  -- Procediment que marca un registre de la taula TFC_PRODUCTES com no actiu (FLGACT = 'N')
  -- Parmetres d' entrada:
  --   P_CODEAN13   => Codi de barres del producte 
  --   P_DATBAI     => Data de baixa del producte 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE BAI_PRODUCTE(P_CODEAN13 TFC_PRODUCTES.CODEAN13%TYPE,
                         P_DATBAI   TFC_PRODUCTES.DATBAI%TYPE,
                         RSP        OUT VARCHAR2);

  -- Procediment que elimina un registre de la taula TFC_PRODUCTES 
  -- Parmetres d' entrada:
  --   P_CODEAN13   => Codi de barres del producte 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE SUP_PRODUCTE(P_CODEAN13 TFC_PRODUCTES.CODEAN13%TYPE,
                         RSP        OUT VARCHAR2);
end TFC_PRO;
/
create or replace package body TFC_PRO is
  -- Procediment que inserta un registre a la taula TFC_PRODUCTES  
  -- Parmetres d' entrada:
  --   P_CODEAN13   => Codi de barres del producte 
  --   P_NOMPRO     => Nom del producte 
  --   P_DESCRIPCIO => Descripci del producte 
  --   P_FLGACT     => Producte actiu? En funci del valor:
  --     'S'        => El producte est actiu
  --     'N'        => El producte no est actiu 
  --   P_DATALT     => Data de alta del producte 
  --   P_DATBAI     => Data de baixa del producte 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE ALT_PRODUCTE(P_CODEAN13   TFC_PRODUCTES.CODEAN13%TYPE,
                         P_NOMPRO     TFC_PRODUCTES.NOMPRO%TYPE,
                         P_DESCRIPCIO TFC_PRODUCTES.DESCRIPCIO%TYPE,
                         P_FLGACT     TFC_PRODUCTES.FLGACT%TYPE,
                         P_DATALT     TFC_PRODUCTES.DATALT%TYPE,
                         P_DATBAI     TFC_PRODUCTES.DATBAI%TYPE,
                         RSP          OUT VARCHAR2) IS
    vNomProces         TFC_RASTRE_PROCES.PROCES%Type := 'TFC_PRO.ALT_PRODUCTE';
    vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
    vError             Exception;
  BEGIN
    If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
      TFC_RAP.INICIAR_RASTRE(vNomProces);
    end if;
    
    -- Es graven els parmetres a la taula de rastre    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_CODEAN13='  ||P_CODEAN13);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_NOMPRO='    ||P_NOMPRO);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_DESCRIPCIO='||P_DESCRIPCIO);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_FLGACT='    ||P_FLGACT);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_DATALT='    ||TO_CHAR(P_DATALT,'DD-MM-YYYY'));
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_DATBAI='    ||TO_CHAR(P_DATBAI,'DD-MM-YYYY'));
    
    -- S' ha informat la data d' alta?
    If (P_DATALT is null) then 
      RSP := 'No s'' ha informat la data d'' alta per el producte amb codi '||P_CODEAN13;
    end if;
    
    If ((RSP is null) or (RSP = TFC_UTL.RSP_OK)) then 
      -- Els producte no est actiu i no s' ha informat la data de baixa 
      If ((P_FLGACT = 'N') and (P_DATBAI is null)) then 
        RSP := 'El producte amb codi '||P_CODEAN13||' no est actiu i no s'' ha informat la data de baixa';
      end if;
    end if;
    
    If ((RSP is null) or (RSP = TFC_UTL.RSP_OK)) then 
      begin
        -- S' inserta el registre 
        Insert into TFC_PRODUCTES 
          (CODEAN13,
           NOMPRO,
           DESCRIPCIO,
           FLGACT,
           DATALT,
           DATBAI)
        values
          (P_CODEAN13,
           P_NOMPRO,
           P_DESCRIPCIO,
           P_FLGACT,
           P_DATALT,
           P_DATBAI);  
    
        If (SQL%RowCount > 0) then 
          TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre insertat.');
        else
          TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre no insertat.');
        end if;    
      exception
        WHEN DUP_VAL_ON_INDEX THEN -- Si es captura aquesta excepci vol dir que el registre ja existeix.
          -- En aquest cas s' actualitzaran els valors del registre existent.
          MOD_PRODUCTE(P_CODEAN13,
                       P_NOMPRO,
                       P_DESCRIPCIO,
                       P_FLGACT,
                       P_DATALT,
                       P_DATBAI,
                       RSP);
      end;  
    end if;
  
    -- Arribats a aquest punt, tot ha anat b.
    If (RSP is null) then 
      RSP := TFC_UTL.RSP_OK;
    end if;
  
    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);

    If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
      If (RSP = TFC_UTL.RSP_OK) then 
        TFC_RAP.FINALITZAR_RASTRE_OK;
      else
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    end if;
  EXCEPTION -- Tractament d' excepcions 
    WHEN others THEN 
      RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
  END;

  -- Procediment que modifica un registre de la taula TFC_PRODUCTES  
  -- Parmetres d' entrada:
  --   P_CODEAN13   => Codi de barres del producte 
  --   P_NOMPRO     => Nom del producte 
  --   P_DESCRIPCIO => Descripci del producte 
  --   P_FLGACT     => Producte actiu? En funci del valor:
  --     'S'        => El producte est actiu
  --     'N'        => El producte no est actiu 
  --   P_DATALT     => Data de alta del producte 
  --   P_DATBAI     => Data de baixa del producte 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE MOD_PRODUCTE(P_CODEAN13   TFC_PRODUCTES.CODEAN13%TYPE,
                         P_NOMPRO     TFC_PRODUCTES.NOMPRO%TYPE,
                         P_DESCRIPCIO TFC_PRODUCTES.DESCRIPCIO%TYPE,
                         P_FLGACT     TFC_PRODUCTES.FLGACT%TYPE,
                         P_DATALT     TFC_PRODUCTES.DATALT%TYPE,
                         P_DATBAI     TFC_PRODUCTES.DATBAI%TYPE,
                         RSP          OUT VARCHAR2) IS
    vNomProces         TFC_RASTRE_PROCES.PROCES%Type := 'TFC_PRO.MOD_PRODUCTE';
    vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
    vError             Exception;
  BEGIN
    If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
      TFC_RAP.INICIAR_RASTRE(vNomProces);
    end if;
    
    -- Es graven els parmetres a la taula de rastre   
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_CODEAN13='  ||P_CODEAN13);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_NOMPRO='    ||P_NOMPRO);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_DESCRIPCIO='||P_DESCRIPCIO);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_FLGACT='    ||P_FLGACT);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_DATALT='    ||TO_CHAR(P_DATALT,'DD-MM-YYYY'));
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_DATBAI='    ||TO_CHAR(P_DATBAI,'DD-MM-YYYY'));

    -- S' ha informat la data d' alta?
    If (P_DATALT is null) then 
      RSP := 'No s'' ha informat la data d'' alta per el producte amb codi '||P_CODEAN13;
    end if;
    
    If ((RSP is null) or (RSP = TFC_UTL.RSP_OK)) then 
      -- Els producte no est actiu i no s' ha informat la data de baixa 
      If ((P_FLGACT = 'N') and (P_DATBAI is null)) then 
        RSP := 'El producte amb codi '||P_CODEAN13||' no est actiu i no s'' ha informat la data de baixa';
      end if;
    end if;
    
    If ((RSP is null) or (RSP = TFC_UTL.RSP_OK)) then 
      -- Es modifica el producte 
      Update TFC_PRODUCTES pro
         set pro.NOMPRO     = P_NOMPRO,
             pro.DESCRIPCIO = P_DESCRIPCIO,
             pro.FLGACT     = P_FLGACT,
             pro.DATALT     = P_DATALT,
             pro.DATBAI     = P_DATBAI
         where pro.CODEAN13 = P_CODEAN13;
       
      If (SQL%RowCount > 0) then 
        RSP := TFC_UTL.RSP_OK;
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre modificat.');
      else
        RSP := 'No existeix cap producte amb el codi '||P_CODEAN13;
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre no modificat.');
      end if;
    end if;
         
    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);

    If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
      If (RSP = TFC_UTL.RSP_OK) then 
        TFC_RAP.FINALITZAR_RASTRE_OK;
      else
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    end if;
  EXCEPTION -- Tractament d' excepcions 
    WHEN others THEN 
      RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
  END;

  -- Procediment que marca un registre de la taula TFC_PRODUCTES com no actiu (FLGACT = 'N')
  -- Parmetres d' entrada:
  --   P_CODEAN13   => Codi de barres del producte 
  --   P_DATBAI     => Data de baixa del producte 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.    
  PROCEDURE BAI_PRODUCTE(P_CODEAN13 TFC_PRODUCTES.CODEAN13%TYPE,
                         P_DATBAI   TFC_PRODUCTES.DATBAI%TYPE,
                         RSP        OUT VARCHAR2) IS
    vNomProces TFC_RASTRE_PROCES.PROCES%Type := 'TFC_PRO.BAI_PRODUCTE';
    vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
    vError             Exception;
    vProces_Error      varchar2(100);
  BEGIN
    If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
      TFC_RAP.INICIAR_RASTRE(vNomProces);
    end if;
    
    -- Es graven els parmetres a la taula de rastre   
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_CODEAN13='||P_CODEAN13);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_DATBAI='  ||TO_CHAR(P_DATBAI,'DD-MM-YYYY'));
    
    If (P_DATBAI is null) then 
      vProces_Error := 'Verificaci de la data de baixa';
      raise vError;
    end if;
    
    -- Es dna de baixa el producte 
    Update TFC_PRODUCTES pro
       set pro.FLGACT     = 'N',
           pro.DATBAI     = P_DATBAI
       where pro.CODEAN13 = P_CODEAN13;
             
    If (SQL%RowCount > 0) then 
      RSP := TFC_UTL.RSP_OK;
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre modificat.');
    else
      RSP := 'No existeix cap producte amb el codi '||P_CODEAN13;
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre no modificat.');
    end if;
   
    -- Arribats a aquest punt, tot ha anat b.
    If (RSP is null) then 
      RSP := TFC_UTL.RSP_OK;
    end if;
  
    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);

    If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
      If (RSP = TFC_UTL.RSP_OK) then 
        TFC_RAP.FINALITZAR_RASTRE_OK;
      else
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    end if;
  EXCEPTION -- Tractament d' excepcions 
    WHEN others THEN 
      If (vProces_Error is null) then 
        RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
      else
        RSP := 'Data de baixa no informada';
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Error en el procs: '||vProces_Error);
        TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      end if;
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
  END;

  -- Procediment que elimina un registre de la taula TFC_PRODUCTES 
  -- Parmetres d' entrada:
  --   P_CODEAN13   => Codi de barres del producte 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE SUP_PRODUCTE(P_CODEAN13 TFC_PRODUCTES.CODEAN13%TYPE,
                         RSP        OUT VARCHAR2) IS
    vNomProces         TFC_RASTRE_PROCES.PROCES%Type := 'TFC_PRO.SUP_PRODUCTE';
    vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
    vError             Exception;
    FK_PRODUCTE        exception;
    PRAGMA             EXCEPTION_INIT(FK_PRODUCTE, -02292);
  BEGIN
    If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
      TFC_RAP.INICIAR_RASTRE(vNomProces);
    end if;
    
    -- Es graven els parmetres a la taula de rastre   
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_CODEAN13='||P_CODEAN13);
    
    Delete 
      from TFC_PRODUCTES
      where CODEAN13 = P_CODEAN13;

    If (SQL%RowCount > 0) then 
      RSP := TFC_UTL.RSP_OK;
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre eliminat.');
    else
      RSP := 'No existeix cap producte amb el codi '||P_CODEAN13;
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre no eliminat.');
    end if;
  
    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);

    If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
      If (RSP = TFC_UTL.RSP_OK) then 
        TFC_RAP.FINALITZAR_RASTRE_OK;
      else
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    end if;
  EXCEPTION -- Tractament d' excepcions 
    WHEN FK_PRODUCTE THEN 
      RSP := 'No es pot eliminar el producte amb codi '||P_CODEAN13||'.';
      If (SQLERRM like '%VEN_PRO_FK%') then 
        RSP := RSP || ' Hi ha vendes associades.';
      elsIf (SQLERRM like '%APA_PRO_FK%') then 
        RSP := RSP || ' Hi ha clculs agregats associats.';
      elsIf (SQLERRM like '%VPH_PRO_FK%') then 
        RSP := RSP || ' Hi ha clculs histrics associats.';
      end if;
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
      
    WHEN others THEN 
      RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
  END;
end TFC_PRO;
/
