create or replace package TFC_REG is
  -- Procediment que inserta un registre a la taula TFC_REGIONS 
  -- Parmetres d' entrada:
  --   P_IDEREG => Identificador de regi 
  --   P_NOMREG => Nom de la regi 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE ALT_REGIO(P_IDEREG TFC_REGIONS.IDEREG%TYPE,
                      P_NOMREG TFC_REGIONS.NOMREG%TYPE,
                      RSP      OUT VARCHAR2);

  -- Procediment que modifica un registre a la taula TFC_REGIONS 
  -- Parmetres d' entrada:
  --   P_IDEREG => Identificador de regi 
  --   P_NOMREG => Nom de la regi 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE MOD_REGIO(P_IDEREG TFC_REGIONS.IDEREG%TYPE,
                      P_NOMREG TFC_REGIONS.NOMREG%TYPE,
                      RSP          OUT VARCHAR2);

  -- Procediment que elimina un registre a la taula TFC_REGIONS 
  -- Parmetres d' entrada:
  --   P_IDEREG => Identificador de regi 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE SUP_REGIO(P_IDEREG TFC_REGIONS.IDEREG%TYPE,
                      RSP      OUT VARCHAR2);

  -- Funci que verifica si existeix una regi amb l' identificador donat  
  -- Parmetres d' entrada:
  --   P_IDEREG => Identificador de regi 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  -- Retorn de la funci: En funci del valor:
  --     TRUE   => Existeix una regi amb l' identificador donat 
  --     FALSE  => No existeix cap regi amb l' identificador donat  
  FUNCTION EXISTEIX_REGIO(P_IDEREG TFC_REGIONS.IDEREG%TYPE,
                          RSP      OUT VARCHAR2) RETURN BOOLEAN;
end TFC_REG;
/
create or replace package body TFC_REG is
  -- Procediment que inserta un registre a la taula TFC_REGIONS 
  -- Parmetres d' entrada:
  --   P_IDEREG => Identificador de regi 
  --   P_NOMREG => Nom de la regi 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE ALT_REGIO(P_IDEREG TFC_REGIONS.IDEREG%TYPE,
                      P_NOMREG TFC_REGIONS.NOMREG%TYPE,
                      RSP      OUT VARCHAR2) IS
    vNomProces TFC_RASTRE_PROCES.PROCES%Type := 'TFC_REG.ALT_REGIO';
    vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
    vError             Exception;
  BEGIN
    If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
      TFC_RAP.INICIAR_RASTRE(vNomProces);
    end if;
    
    -- Es graven els parmetres a la taula de rastre    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_IDEREG='||P_IDEREG);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_NOMREG='||P_NOMREG);
    
    BEGIN
      Insert into TFC_REGIONS
        (IDEREG,
         NOMREG)
      values
        (P_IDEREG,
         P_NOMREG);
        
      If (SQL%RowCount > 0) then 
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre insertat.');
      else
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre no insertat.');
      end if;    
    exception
      WHEN DUP_VAL_ON_INDEX THEN -- Si es captura aquesta excepci vol dir que el registre ja existeix.
        -- En aquest cas s' actualitzaran els valors del registre existent.
        MOD_REGIO(P_IDEREG,
                  P_NOMREG,
                  RSP);
    end;  
    
    -- Arribats a aquest punt, tot ha anat b.
    If (RSP is null) then 
      RSP := TFC_UTL.RSP_OK;
    end if;
  
    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);

    If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
      If (RSP = TFC_UTL.RSP_OK) then 
        TFC_RAP.FINALITZAR_RASTRE_OK;
      else
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    end if;
  EXCEPTION -- Tractament d' excepcions 
    WHEN others THEN 
      RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);

      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);

      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
  END;

  -- Procediment que modifica un registre a la taula TFC_REGIONS 
  -- Parmetres d' entrada:
  --   P_IDEREG => Identificador de regi 
  --   P_NOMREG => Nom de la regi 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE MOD_REGIO(P_IDEREG TFC_REGIONS.IDEREG%TYPE,
                      P_NOMREG TFC_REGIONS.NOMREG%TYPE,
                      RSP          OUT VARCHAR2) IS
    vNomProces TFC_RASTRE_PROCES.PROCES%Type := 'TFC_REG.MOD_REGIO';
    vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
    vError             Exception;
  BEGIN
    If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
      TFC_RAP.INICIAR_RASTRE(vNomProces);
    end if;
    
    -- Es graven els parmetres a la taula de rastre        
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_IDEREG='||P_IDEREG);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_NOMREG='||P_NOMREG);
    
    -- S' actualitza la regi 
    Update TFC_REGIONS
       set NOMREG   = P_NOMREG
       where IDEREG = P_IDEREG;
       
    If (SQL%RowCount > 0) then 
      RSP := TFC_UTL.RSP_OK;
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre modificat.');
    else
      RSP := 'No existeix cap regi amb l'' identificador '||P_IDEREG;
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre no modificat.');
    end if;
       
    -- Arribats a aquest punt, tot ha anat b.
    If (RSP is null) then 
      RSP := TFC_UTL.RSP_OK;
    end if;
  
    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);

    If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
      If (RSP = TFC_UTL.RSP_OK) then 
        TFC_RAP.FINALITZAR_RASTRE_OK;
      else
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    end if;
  EXCEPTION -- Tractament d' excepcions 
    WHEN others THEN 
      RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
  END;

  -- Procediment que elimina un registre a la taula TFC_REGIONS 
  -- Parmetres d' entrada:
  --   P_IDEREG => Identificador de regi 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE SUP_REGIO(P_IDEREG TFC_REGIONS.IDEREG%TYPE,
                      RSP      OUT VARCHAR2) IS
    vNomProces TFC_RASTRE_PROCES.PROCES%Type := 'TFC_REG.SUP_REGIO';
    vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
    vError             Exception;
    FK_REGIO           exception;
    PRAGMA             EXCEPTION_INIT(FK_REGIO, -02292);
  BEGIN
    If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
      TFC_RAP.INICIAR_RASTRE(vNomProces);
    end if;
    
    -- Es graven els parmetres a la taula de rastre    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_IDEREG='||P_IDEREG);
    
    -- S' elimina la regi 
    Delete 
      from TFC_REGIONS
      where IDEREG = P_IDEREG;
      
    If (SQL%RowCount > 0) then 
      RSP := TFC_UTL.RSP_OK;
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre eliminat.');
    else
      RSP := 'No existeix cap regi amb l'' identificador '||P_IDEREG;
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre no eliminat.');
    end if;
      
    -- Arribats a aquest punt, tot ha anat b.
    If (RSP is null) then 
      RSP := TFC_UTL.RSP_OK;
    end if;
  
    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);

    If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
      If (RSP = TFC_UTL.RSP_OK) then 
        TFC_RAP.FINALITZAR_RASTRE_OK;
      else
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    end if;
  EXCEPTION -- Tractament d' excepcions 
    WHEN FK_REGIO THEN 
      RSP := 'No es pot eliminar la regi amb identificador '||P_IDEREG||'. Hi ha ciutats associades.';
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
      
    WHEN others THEN 
      RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
  END;
  
  -- Funci que verifica si existeix una regi amb l' identificador donat  
  -- Parmetres d' entrada:
  --   P_IDEREG => Identificador de regi 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  -- Retorn de la funci: En funci del valor:
  --     TRUE   => Existeix una regi amb l' identificador donat 
  --     FALSE  => No existeix cap regi amb l' identificador donat 
  FUNCTION EXISTEIX_REGIO(P_IDEREG TFC_REGIONS.IDEREG%TYPE,
                          RSP      OUT VARCHAR2) RETURN BOOLEAN IS
    vDummy varchar2(1);
    vNomProces TFC_RASTRE_PROCES.PROCES%Type := 'TFC_REG.EXISTEIX_REGIO';
    vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
    vError             Exception;
  BEGIN
    If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
      TFC_RAP.INICIAR_RASTRE(vNomProces);
    end if;
    
    -- Es graven els parmetres a la taula de rastre        
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_IDEREG='||P_IDEREG);

    begin
      Select '1' 
        into vDummy 
        from TFC_REGIONS reg
        where reg.IDEREG = P_IDEREG;
        
      TFC_RAP.REGISTRAR_RASTRE_LINIA('La regi existeix.');
    EXCEPTION
      WHEN NO_DATA_FOUND THEN 
        vDummy := '0';
        TFC_RAP.REGISTRAR_RASTRE_LINIA('La regi no existeix.');
    end;
            
    -- Arribats a aquest punt, tot ha anat b.
    If (RSP is null) then 
      RSP := TFC_UTL.RSP_OK;
    end if;
  
    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);

    If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
      If (RSP = TFC_UTL.RSP_OK) then 
        TFC_RAP.FINALITZAR_RASTRE_OK;
      else
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    end if;

    return (vDummy = '1');
  EXCEPTION -- Tractament d' excepcions 
    WHEN others THEN 
      RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
      
      return false;
  END;
end TFC_REG;
/
