create or replace package TFC_TEM is
  -- Procediment que inserta un registre a la taula TFC_TEMPS 
  -- Parmetres d' entrada:
  --   P_DATA => Data 
  --   P_HORA => Hora . Els valors possibles van del 0 al 23
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE ALT_TEMPS(P_DATA TFC_TEMPS.DATA%TYPE,
                      P_HORA TFC_TEMPS.HORA%TYPE,
                      RSP    OUT VARCHAR2);

  -- Procediment que elimina un registre de la taula TFC_TEMPS 
  -- Parmetres d' entrada:
  --   P_DATA => Data 
  --   P_HORA => Hora . Els valors possibles van del 0 al 23
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE SUP_TEMPS(P_DATA TFC_TEMPS.DATA%TYPE,
                      P_HORA TFC_TEMPS.HORA%TYPE,
                      RSP    OUT VARCHAR2);
                      
  -- Funci que verifica si existeix un registre a la taula TFC_TEMPS 
  -- Parmetres d' entrada:
  --   P_DATA => Data 
  --   P_HORA => Hora . Els valors possibles van del 0 al 23
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  -- Retorn de la funci: En funci del valor:
  --     TRUE   => Existeix un registre a la taula TFC_TEMPS amb els parmetres donats  
  --     FALSE  => No existeix un registre a la taula TFC_TEMPS amb els parmetres donats  
  FUNCTION EXISTEIX_TEMPS(P_DATA TFC_TEMPS.DATA%TYPE,
                          P_HORA TFC_TEMPS.HORA%TYPE,
                          RSP    OUT VARCHAR2) RETURN BOOLEAN;
end TFC_TEM;
/
create or replace package body TFC_TEM is
  -- Procediment que inserta un registre a la taula TFC_TEMPS 
  -- Parmetres d' entrada:
  --   P_DATA => Data 
  --   P_HORA => Hora . Els valors possibles van del 0 al 23
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE ALT_TEMPS(P_DATA TFC_TEMPS.DATA%TYPE,
                      P_HORA TFC_TEMPS.HORA%TYPE,
                      RSP    OUT VARCHAR2) IS
    vNomProces TFC_RASTRE_PROCES.PROCES%Type := 'TFC_TEM.ALT_TEMPS';
    vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
    vError             Exception;
  BEGIN
    If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
      TFC_RAP.INICIAR_RASTRE(vNomProces);
    end if;
    
    -- Es graven els parmetres a la taula de rastre        
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_DATA='||TO_CHAR(P_DATA,'DD-MM-YYYY'));
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_HORA='||P_HORA);
    
    TFC_INA.ALT_INA(to_number(to_char(P_DATA, 'YYYY')), RSP);
    
    If (RSP = TFC_UTL.RSP_OK) then 
    
      BEGIN
        Insert into TFC_TEMPS
          (DATA,
           HORA,
           INA_NUMANY,
           TIMESTRE,
           MES,
           DIAMES,
           DIASETMANA)
        values
          (trunc(P_DATA),
           P_HORA,
           to_number(to_char(P_DATA, 'YYYY')),
           to_number(to_char(P_DATA, 'Q')),
           to_number(to_char(P_DATA, 'MM')),
           to_number(to_char(P_DATA, 'DD')),
           (1 + TRUNC (P_DATA) - TRUNC (P_DATA, 'IW')));
       
        If (SQL%RowCount > 0) then 
          TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre insertat.');
        else
          TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre no insertat.');
        end if; 
      exception
        WHEN DUP_VAL_ON_INDEX THEN -- Si es captura aquesta excepci vol dir que el registre ja existeix.
          -- En aquest cas s' actualitzaran els valors del registre existent.
          RSP := 'Ja existeix un registre de temps amb la data '||TO_CHAR(P_DATA, 'DD-MM-YYYY')||' i hora '||P_HORA;
      end;
    end if;
   
    -- Arribats a aquest punt, tot ha anat b.
    If (RSP is null) then 
      RSP := TFC_UTL.RSP_OK;
    end if;
  
    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);

    If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
      If (RSP = TFC_UTL.RSP_OK) then 
        TFC_RAP.FINALITZAR_RASTRE_OK;
      else
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    end if;
  EXCEPTION -- Tractament d' excepcions 
    WHEN others THEN 
      RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
  END;

  -- Procediment que elimina un registre de la taula TFC_TEMPS 
  -- Parmetres d' entrada:
  --   P_DATA => Data 
  --   P_HORA => Hora . Els valors possibles van del 0 al 23
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE SUP_TEMPS(P_DATA TFC_TEMPS.DATA%TYPE,
                      P_HORA TFC_TEMPS.HORA%TYPE,
                      RSP    OUT VARCHAR2) IS
    vNomProces TFC_RASTRE_PROCES.PROCES%Type := 'TFC_TEM.MOD_TEMPS';
    vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
    vError             Exception;
    FK_TEMPS           exception;
    PRAGMA             EXCEPTION_INIT(FK_TEMPS, -02292);
  BEGIN
    If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
      TFC_RAP.INICIAR_RASTRE(vNomProces);
    end if;
    
    -- Es graven els parmetres a la taula de rastre        
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_DATA='||TO_CHAR(P_DATA,'DD-MM-YYYY'));
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_HORA='||P_HORA);
    
    Delete 
      from TFC_TEMPS
      where DATA = P_DATA and 
            HORA = P_HORA;
            
    If (SQL%RowCount > 0) then 
      RSP := TFC_UTL.RSP_OK;
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre eliminat.');
    else
      RSP := 'No exsiteix cap registre de temps amb la data '||TO_CHAR(P_DATA, 'DD-MM-YYYY')||' i hora '||P_HORA;
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre no eliminat.');
    end if;
            
    
    -- Arribats a aquest punt, tot ha anat b.
    If (RSP is null) then 
      RSP := TFC_UTL.RSP_OK;
    end if;
  
    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);

    If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
      If (RSP = TFC_UTL.RSP_OK) then 
        TFC_RAP.FINALITZAR_RASTRE_OK;
      else
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    end if;
  EXCEPTION -- Tractament d' excepcions 
    WHEN FK_TEMPS THEN 
      RSP := 'No es pot eliminar la venda per la data '||TO_CHAR(P_DATA, 'DD-MM-YYYY')||' i hora '||P_HORA||':00:00 .';
     
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;

    WHEN others THEN 
      RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
  END;
  
  -- Funci que verifica si existeix un registre a la taula TFC_TEMPS 
  -- Parmetres d' entrada:
  --   P_DATA => Data 
  --   P_HORA => Hora . Els valors possibles van del 0 al 23
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  -- Retorn de la funci: En funci del valor:
  --     TRUE   => Existeix un registre a la taula TFC_TEMPS amb els parmetres donats  
  --     FALSE  => No existeix un registre a la taula TFC_TEMPS amb els parmetres donats  
  FUNCTION EXISTEIX_TEMPS(P_DATA TFC_TEMPS.DATA%TYPE,
                          P_HORA TFC_TEMPS.HORA%TYPE,
                          RSP    OUT VARCHAR2) RETURN BOOLEAN IS 
    vDummy varchar2(1) := 0;
    vNomProces TFC_RASTRE_PROCES.PROCES%Type := 'TFC_TEM.EXISTEIX_TEMPS';
    vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
    vError             Exception;
  BEGIN
    If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
      TFC_RAP.INICIAR_RASTRE(vNomProces);
    end if;
    
    -- Es graven els parmetres a la taula de rastre        
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_DATA='||TO_CHAR(P_DATA,'DD-MM-YYYY'));
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_HORA='||P_HORA);
    
    begin
      Select '1' 
        into vDummy 
        from TFC_TEMPS tem
        where tem.DATA = P_DATA and 
              tem.HORA = P_HORA;
              
      TFC_RAP.REGISTRAR_RASTRE_LINIA('La data i hora existeixen.');
    exception
      when no_data_found then 
        vDummy := '0';
        TFC_RAP.REGISTRAR_RASTRE_LINIA('La data i hora no existeixen.');
    end; 

    -- Arribats a aquest punt, tot ha anat b.
    If (RSP is null) then 
      RSP := TFC_UTL.RSP_OK;
    end if;
  
    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);

    If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
      If (RSP = TFC_UTL.RSP_OK) then 
        TFC_RAP.FINALITZAR_RASTRE_OK;
      else
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    end if;

    return (vDummy = '1');
  EXCEPTION -- Tractament d' excepcions 
    WHEN others THEN 
      RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
      
      return false;
  END;
end TFC_TEM;
/
