create or replace package TFC_UTL is
  TYPE mail_attachment IS RECORD (
    attachment_name varchar2(2000),
    attachment_mime varchar2(200),
    attachment_blob blob);

  TYPE table_attachments IS TABLE OF mail_attachment index by binary_integer;

  -- Funci que enva un mail amb adjunts.
  -- Parmetres d' entrada:
  --   p_to          => Adrea de mail del destinatari. Si sn vries s' han de separar per ','
  --   p_cc          => Adrea de mail en cpia . Si sn vries s' han de separar per ','
  --   p_bcc         => Adrea de mail en cpia oculta. Si sn vries s' han de separar per ','
  --   p_from        => Adrea de mail del remitent.
  --   p_subject     => Asumpte del mail 
  --   p_text_msg    => Cos del mail en format pla.
  --   p_html_msg    => Cos del mail en format HTML 
  --   p_attachments => Llista de fitxers adjunts 
  --   p_smtp_host   => Servidor de mail 
  --   p_smtp_port   => Port del servidor de mail . Per defecte port 25. 
  --   p_username    => Usuari en cas de requerir autenticaci. Es pot deixar en blanc.
  --   p_password    => Contrasenya en cas de requerir atutenticaci. Es pot deixar en blanc.
  -- Parmetres de sortida: No t parmetres de sortida 
  -- Retorn de la funci: 
  --   NULL          => No s' ha produt cap error 
  --   ERROR         => En cas d' error retorna l' error.
  FUNCTION ENVIAR_MAIL_ADJUNTS (p_to       IN VARCHAR2,
                                p_cc       IN VARCHAR2 DEFAULT NULL,
                                p_bcc      IN VARCHAR2 DEFAULT NULL,
                                p_from     IN VARCHAR2,
                                p_subject  IN VARCHAR2,
                                p_text_msg IN VARCHAR2 DEFAULT NULL,
                                p_html_msg IN CLOB DEFAULT NULL,
                                p_attachments IN table_attachments,
                                p_smtp_host   IN VARCHAR2,
                                p_smtp_port   IN NUMBER DEFAULT 25,
                                p_username    IN VARCHAR2 DEFAULT NULL,
                                p_password    IN VARCHAR2 DEFAULT NULL) RETURN VARCHAR2;

  -- Funci que enva sense adjunts. 
  -- Parmetres d' entrada:
  --   P_FROM        => Adrea de mail del remitent.
  --   P_TO          => Adrea de mail del destinatari. Si sn vries s' han de separar per ','
  --   P_CC          => Adrea de mail en cpia . Si sn vries s' han de separar per ','
  --   P_SUBJECT     => Asumpte del mail
  --   P_BODY        => Cos del mail en format pla.
  --   p_smtp_host   => Servidor de mail 
  --   p_smtp_port   => Port del servidor de mail . Per defecte port 25. 
  --   p_username    => Usuari en cas de requerir autenticaci. Es pot deixar en blanc.
  --   p_password    => Contrasenya en cas de requerir atutenticaci. Es pot deixar en blanc.
  -- Parmetres de sortida: No t parmetres de sortida 
  -- Retorn de la funci: 
  --   NULL          => No s' ha produt cap error 
  --   ERROR         => En cas d' error retorna l' error.
  FUNCTION ENVIAR_MAIL (P_FROM    VARCHAR2,
                        P_TO      VARCHAR2,
                        P_CC      VARCHAR2,
                        P_SUBJECT VARCHAR2,
                        P_BODY    CLOB,
                        p_smtp_host   IN VARCHAR2,
                        p_smtp_port   IN NUMBER DEFAULT 25,
                        p_username    IN VARCHAR2 DEFAULT NULL,
                        p_password    IN VARCHAR2 DEFAULT NULL) RETURN VARCHAR2;

  -- Procediment que afegeix una partici a la taula indicada.
  -- Parmetres d' entrada:
  --   P_TABLE_OWNER     => Propietari de la taula 
  --   P_TABLE_NAME      => Nom de la taula 
  --   P_PARTITION_NAME  => Nom de la partici 
  --   P_TABLESPACE_PART => Tablespace de la partici 
  --   P_MAX_DATE        => Data mxima de la partici 
  -- Parmetres de sortida: No t parmetres de sortida
  PROCEDURE ADD_PARTITION(P_TABLE_OWNER     VARCHAR2,
                          P_TABLE_NAME      VARCHAR2,
                          P_PARTITION_NAME  VARCHAR2,
                          P_TABLESPACE_PART VARCHAR2,
                          P_MAX_DATE        DATE);

  -- Procediment que elimina la partici indicada de la taula donada.
  -- Parmetres d' entrada:
  --   P_TABLE_OWNER     => Propietari de la taula 
  --   P_TABLE_NAME      => Nom de la taula 
  --   P_PARTITION_NAME  => Nom de la partici 
  -- Parmetres de sortida: No t parmetres de sortida
  PROCEDURE DROP_PARTITION(P_TABLE_OWNER    VARCHAR2,
                           P_TABLE_NAME     VARCHAR2,
                           P_PARTITION_NAME VARCHAR2);

  -- Funci que mira si la taula i partici existeixen .
  -- Parmetres d' entrada:
  --   P_TABLE_OWNER     => Propietari de la taula 
  --   P_TABLE_NAME      => Nom de la taula 
  --   P_PARTITION_NAME  => Nom de la partici 
  -- Parmetres de sortida: No t parmetres de sortida
  -- Retorn de la funci: Retorna true o false en funci de si existeix la partici o no.
  --     TRUE  => Existeix la partici 
  --     FALSE => No existeix la partici 
  FUNCTION EXISTE_TABLA(P_TABLE_OWNER VARCHAR2,
                        P_TABLE_NAME VARCHAR2,
                        P_PARTITION_NAME VARCHAR2 DEFAULT NULL) RETURN BOOLEAN ;

  -- Funci que mira si la taula i partici donats contenen dades.
  -- Parmetres d' entrada:
  --   P_TABLE_OWNER     => Propietari de la taula 
  --   P_TABLE_NAME      => Nom de la taula 
  --   P_PARTITION_NAME  => Nom de la partici 
  -- Parmetres de sortida: No t parmetres de sortida
  -- Retorn de la funci: Retorna true o false en funci de si la partici cont dades o no.
  --     TRUE  => Cont dades 
  --     FALSE => No cont dades  
  FUNCTION TABLA_CONTIENE_DATOS(P_TABLE_OWNER VARCHAR2,
                                P_TABLE_NAME VARCHAR2,
                                P_PARTITION_NAME VARCHAR2 DEFAULT NULL) RETURN BOOLEAN;

  -- Funci mira si un fitxer existeix.
  -- Parmetres d' entrada:
  --   P_DIRECTORI_BD => Directori de base de dades  
  --   P_NOM_FITXER   => Nom del fitxer 
  -- Parmetres de sortida: No t parmetres de sortida
  -- Retorn de la funci: Retorna true o false en funci de si existeix o no el fitxer donat en el directori de base de dades indicat 
  --     TRUE  => Existeix el fitxer
  --     FALSE => No existeix el fitxer 
  FUNCTION EXISTEIX_FITXER(P_DIRECTORI_BD varchar2,
                           P_NOM_FITXER   varchar2) RETURN BOOLEAN;

  -- Funci que carrega el contingut d' un fitxer pla i el retorna com a CLOB
  -- Parmetres d' entrada:
  --   P_DIRECTORI_BD => Directori de base de dades  
  --   P_NOM_FITXER   => Nom del fitxer 
  -- Parmetres de sortida: No t parmetres de sortida
  -- Retorn de la funci: Contingut del fitxer en format CLOB 
  FUNCTION OBT_CONTINGUT_FITXER_PLA(P_DIRECTORI_BD VARCHAR2,
                                    P_NOM_FITXER   VARCHAR2) RETURN CLOB;

  -- Funci que afegeix un adjunt a la variable P_ADJUTNS
  -- Parmetres d' entrada i sortida:
  --   P_ADJUNTS      => Llista de fitxers a adjuntar 
  -- Parmetre noms d' entrada: 
  --   P_DIRECTORI_BD => Directori de base de dades  
  --   P_NOM_FITXER   => Nom del fitxer 
  --   P_MIME_ADJUNT  => Tipus mime del fitxer adjunt 
  -- Parmetres noms de sortida: No t parmetres de sortida
  -- Retorn de la funci: 
  --   NULL          => No s' ha produt cap error 
  --   ERROR         => En cas d' error retorna l' error.
  FUNCTION AFEGIR_MAIL_ADJUNT(P_ADJUNTS      IN OUT table_attachments,
                              P_DIRECTORI_BD VARCHAR2,
                              P_NOM_FITXER   VARCHAR2,
                              P_MIME_ADJUNT  VARCHAR2 default 'text/plain') RETURN VARCHAR2;

  -- Procediment que buida la taula passada per parmetre
  -- Parmetres d' entrada:
  --   P_PROPIETARI => Propietari de la taula   
  --   P_TAULA      => Nom de la taula 
  -- Parmetres de sortida: No t parmetres de sortida
  PROCEDURE BUIDAR_TAULA(P_PROPIETARI VARCHAR2,
                         P_TAULA      VARCHAR2);
  
  -- Funci que retorna el valor d' una resposta OK 
  -- Parmetres d' entrada: No t parmetres d' entrada 
  -- Parmetres de sortida: No t parmetres de sortida .
  -- Retorn de la funci:
  --     OK => Valor d' una resposta correcte. 
  FUNCTION RSP_OK RETURN VARCHAR2;

  -- Funci que retorna el valor INC 
  -- Parmetres d' entrada: No t parmetres d' entrada 
  -- Parmetres de sortida: No t parmetres de sortida .
  -- Retorn de la funci:
  --     INC => Valor per el clcul incremental.   
  FUNCTION CALCUL_INC RETURN VARCHAR2;

  -- Funci que retorna el valor TOT 
  -- Parmetres d' entrada: No t parmetres d' entrada 
  -- Parmetres de sortida: No t parmetres de sortida .
  -- Retorn de la funci:
  --     TOT  => Valor per el clcul incremental.   
  FUNCTION CALCUL_TOT RETURN VARCHAR2;

  -- Funci que retorna el valor del parmetre TIPUS_CALCUL_INDICADORS  
  -- Parmetres d' entrada: No t parmetres d' entrada 
  -- Parmetres de sortida: No t parmetres de sortida .
  -- Retorn de la funci:
  --     Valor del parmetre TIPUS_CALCUL_INDICADORS => Est definit a la taula TFC_PARAMETRES 
  FUNCTION TIPUS_CALCUL_INDICADORS RETURN VARCHAR2;
  
  -- Funci que verifica si el clcul d' indicadors s' ha de fer cada venda   
  -- Parmetres d' entrada: No t parmetres d' entrada 
  -- Parmetres de sortida: No t parmetres de sortida .
  -- Retorn de la funci:
  --     TRUE  => El clcul d' indicadors s' ha de fer per cada venda  
  --     FLASE => El clcul d' indicadors NO s' ha de fer per cada venda. S' haur d' invocar manualment desprs de la crrega de vendes 
  FUNCTION CALCULAR_IND_PER_CADA_VENDA RETURN BOOLEAN;
  
  -- Funci que retorna el valor del parmetre P_NOMPAR segons s' hagi definit a la taula TFC_PARAMETRES    
  -- Parmetres d' entrada: 
  --   P_NOMPAR => Nom del parmetre a buscar 
  -- Parmetres de sortida: No t parmetres de sortida .
  -- Retorn de la funci: Valor del parmetre P_NOMPAR segons s' hagi definit a la taula TFC_PARAMETRES 
  FUNCTION OBT_VAL_PAR(P_NOMPAR TFC_PARAMETRES.Nompar%Type) return tfc_parametres.valpar%Type;
  
  -- Funci que retorna un nom de fitxer nic en el directori indiciat partint del nom passat com a parmetre 
  -- Parmetres d' entrada: 
  --   P_DIR_BD  => Directori de base de dades
  --   P_NOM_FIC => Nom del fitxer a verificar 
  -- Parmetres de sortida: No t parmetres de sortida .
  -- Retorn de la funci: Nom nic dintre del directori de base de dades i partint del nom P_NOM_FIC 
  FUNCTION OBT_NOM_FITXER_UNIC(P_DIR_BD  varchar2, 
                               P_NOM_FIC varchar2) return varchar2;
end TFC_UTL;
/
create or replace package body TFC_UTL is
  -- Contant de resposta OK 
  cRsp_OK constant varchar2(2) := 'OK';
  -- Clcula incremental 
  cCalc_Ind_INC constant varchar2(3) := 'INC';
  -- Clcul total 
  cCalc_Ind_TOT constant varchar2(3) := 'TOT';
  -- Tipus de clcul informat en la taula de parmetres 
  vTipus_Calcul_Indicadors varchar2(3);
  -- Calcular els indicadors per cada venda? 
  vCalcular_Indicador_Per_Venda boolean;

  -- Funci que enva un mail amb adjunts.
  -- Parmetres d' entrada:
  --   p_to          => Adrea de mail del destinatari. Si sn vries s' han de separar per ','
  --   p_cc          => Adrea de mail en cpia . Si sn vries s' han de separar per ','
  --   p_bcc         => Adrea de mail en cpia oculta. Si sn vries s' han de separar per ','
  --   p_from        => Adrea de mail del remitent.
  --   p_subject     => Asumpte del mail 
  --   p_text_msg    => Cos del mail en format pla.
  --   p_html_msg    => Cos del mail en format HTML 
  --   p_attachments => Llista de fitxers adjunts 
  --   p_smtp_host   => Servidor de mail 
  --   p_smtp_port   => Port del servidor de mail . Per defecte port 25. 
  --   p_username    => Usuari en cas de requerir autenticaci. Es pot deixar en blanc.
  --   p_password    => Contrasenya en cas de requerir atutenticaci. Es pot deixar en blanc.
  -- Parmetres de sortida: No t parmetres de sortida 
  -- Retorn de la funci: 
  --   NULL          => No s' ha produt cap error 
  --   ERROR         => En cas d' error retorna l' error.
  FUNCTION ENVIAR_MAIL_ADJUNTS (p_to       IN VARCHAR2,
                                p_cc       IN VARCHAR2 DEFAULT NULL,
                                p_bcc      IN VARCHAR2 DEFAULT NULL,
                                p_from     IN VARCHAR2,
                                p_subject  IN VARCHAR2,
                                p_text_msg IN VARCHAR2 DEFAULT NULL,
                                p_html_msg IN CLOB DEFAULT NULL,
                                p_attachments IN table_attachments,
                                p_smtp_host   IN VARCHAR2,
                                p_smtp_port   IN NUMBER DEFAULT 25,
                                p_username    IN VARCHAR2 DEFAULT NULL,
                                p_password    IN VARCHAR2 DEFAULT NULL) RETURN VARCHAR2 IS
    l_mail_conn   UTL_SMTP.connection;
    l_boundary    VARCHAR2(50) := '----=*#abc1234321cba#*=';
    l_step        PLS_INTEGER  := 24573;
    l_length      PLS_INTEGER;
    l_begin       PLS_INTEGER  := 1;
    l_buffer_size INTEGER      := 75;
    l_raw         RAW(32767);
    vError        varchar2(500);
  BEGIN
    l_mail_conn := UTL_SMTP.open_connection(p_smtp_host, p_smtp_port);

    -- S' obtre la connexi 
    IF (p_username is not null) then
      utl_smtp.command( l_mail_conn, 'AUTH LOGIN');
      utl_smtp.command( l_mail_conn, utl_raw.cast_to_varchar2( utl_encode.base64_encode( utl_raw.cast_to_raw( p_username ))) );
      utl_smtp.command( l_mail_conn, utl_raw.cast_to_varchar2( utl_encode.base64_encode( utl_raw.cast_to_raw( p_password ))) );
    end if;

    UTL_SMTP.helo(l_mail_conn, p_smtp_host);
    UTL_SMTP.mail(l_mail_conn, p_from);
    UTL_SMTP.rcpt(l_mail_conn, p_to);

    UTL_SMTP.open_data(l_mail_conn);

    UTL_SMTP.write_data(l_mail_conn, 'Date: ' || TO_CHAR(SYSDATE, 'DD-MON-YYYY HH24:MI:SS') || UTL_TCP.crlf);
    UTL_SMTP.write_data(l_mail_conn, 'To: ' || p_to || UTL_TCP.crlf);
    UTL_SMTP.write_data(l_mail_conn, 'From: ' || p_from || UTL_TCP.crlf);

    -- Mails en cpia 
    if (p_cc is not null) then
      UTL_SMTP.write_data(l_mail_conn, 'CC: ' || p_cc || UTL_TCP.crlf);
    end if;

    -- Mails en cpia oculta 
    if (p_bcc is not null) then
      UTL_SMTP.write_data(l_mail_conn, 'BCC: ' || p_bcc || UTL_TCP.crlf);
    end if;

    UTL_SMTP.write_data(l_mail_conn, 'Content-Transfer-Encoding: 8bit' || UTL_TCP.crlf);
    UTL_smtp.write_raw_data(l_mail_conn, utl_raw.cast_to_raw('Subject: ' || p_subject || UTL_TCP.crlf));
    UTL_SMTP.write_data(l_mail_conn, 'Reply-To: ' || p_from || UTL_TCP.crlf);
    UTL_SMTP.write_data(l_mail_conn, 'MIME-Version: 1.0' || UTL_TCP.crlf);
    UTL_SMTP.write_data(l_mail_conn, 'Content-Type: multipart/mixed; boundary="' || l_boundary || '"' || UTL_TCP.crlf || UTL_TCP.crlf);

    -- Cos del missatge en texte pla 
    IF p_text_msg IS NOT NULL THEN -- Plain Text Body
      UTL_SMTP.write_data(l_mail_conn, '--' || l_boundary || UTL_TCP.crlf);
      UTL_SMTP.write_data(l_mail_conn, 'Content-Type: text/plain; charset="iso-8859-1"' || UTL_TCP.crlf || UTL_TCP.crlf);
      UTL_smtp.write_raw_data(l_mail_conn, utl_raw.cast_to_raw(p_text_msg));
      UTL_SMTP.write_data(l_mail_conn, UTL_TCP.crlf || UTL_TCP.crlf);
    -- Cos del missatge en format HTML 
    ELSIF p_html_msg IS NOT NULL THEN 
      UTL_SMTP.write_data(l_mail_conn, '--' || l_boundary || UTL_TCP.crlf);
      UTL_SMTP.write_data(l_mail_conn, 'Content-Type: text/html; charset="iso-8859-1"' || UTL_TCP.crlf || UTL_TCP.crlf);
      FOR i IN 0 .. TRUNC((DBMS_LOB.getlength(p_html_msg) - 1 )/l_step) LOOP
        UTL_SMTP.write_data(l_mail_conn,  DBMS_LOB.substr(p_html_msg, l_step, i * l_step + 1));
      END LOOP;

      UTL_SMTP.write_data(l_mail_conn, UTL_TCP.crlf || UTL_TCP.crlf);
    END IF;

    -- Si hi ha annexes ... 
    IF p_attachments IS NOT NULL THEN

      FOR i IN 1..p_attachments.count LOOP
        UTL_SMTP.write_data(l_mail_conn, '--' || l_boundary || UTL_TCP.crlf);
        UTL_SMTP.write_data(l_mail_conn, 'Content-Type: ' ||p_attachments(i).attachment_mime || UTL_TCP.crlf);
        UTL_SMTP.write_data(l_mail_conn, 'Content-Transfer-Encoding: base64' || UTL_TCP.crlf);
        UTL_SMTP.write_data(l_mail_conn, 'Content-Disposition: attachment; filename="' || p_attachments(i).attachment_name || '"' || UTL_TCP.crlf || UTL_TCP.crlf);
        l_length := dbms_lob.getlength(p_attachments(i).attachment_blob);
        l_begin := 1;
        l_raw := null;
        l_buffer_size := 75;
        WHILE l_begin < l_length LOOP
           dbms_lob.read( p_attachments(i).attachment_blob, l_buffer_size, l_begin, l_raw );
           utl_smtp.write_raw_data( l_mail_conn, utl_encode.base64_encode(l_raw) );
           utl_smtp.write_data( l_mail_conn, utl_tcp.crlf );
           l_begin := l_begin + l_buffer_size;
        END LOOP ;
        UTL_SMTP.write_data(l_mail_conn, UTL_TCP.crlf || UTL_TCP.crlf);
      END LOOP;
    END IF;

    UTL_SMTP.write_data(l_mail_conn, '--' || l_boundary || '--' || UTL_TCP.crlf);
    UTL_SMTP.close_data(l_mail_conn);

    UTL_SMTP.quit(l_mail_conn);

    return null;

  EXCEPTION -- Control d' errors 
    WHEN utl_smtp.transient_error OR utl_smtp.permanent_error THEN
      BEGIN
        utl_smtp.quit(l_mail_conn);
      EXCEPTION
        WHEN utl_smtp.transient_error OR utl_smtp.permanent_error THEN
          null;
      END;
      vError := SubStr(SQLERRM, 1, 500);
      return vError;
      RAISE_APPLICATION_ERROR(-20000,'Failed to send mail due to the following error: ' || sqlerrm);
    WHEN OTHERS THEN
      utl_smtp.quit(l_mail_conn);
      vError := SubStr(SQLERRM, 1, 500);
      return vError;
  END;

  -- Funci que enva sense adjunts. 
  -- Parmetres d' entrada:
  --   P_FROM        => Adrea de mail del remitent.
  --   P_TO          => Adrea de mail del destinatari. Si sn vries s' han de separar per ','
  --   P_CC          => Adrea de mail en cpia . Si sn vries s' han de separar per ','
  --   P_SUBJECT     => Asumpte del mail
  --   P_BODY        => Cos del mail en format pla.
  --   p_smtp_host   => Servidor de mail 
  --   p_smtp_port   => Port del servidor de mail . Per defecte port 25. 
  --   p_username    => Usuari en cas de requerir autenticaci. Es pot deixar en blanc.
  --   p_password    => Contrasenya en cas de requerir atutenticaci. Es pot deixar en blanc.
  -- Parmetres de sortida: No t parmetres de sortida 
  -- Retorn de la funci: 
  --   NULL          => No s' ha produt cap error 
  --   ERROR         => En cas d' error retorna l' error.
  FUNCTION ENVIAR_MAIL (P_FROM    VARCHAR2,
                        P_TO      VARCHAR2,
                        P_CC      VARCHAR2,
                        P_SUBJECT VARCHAR2,
                        P_BODY    CLOB,
                        p_smtp_host   IN VARCHAR2,
                        p_smtp_port   IN NUMBER DEFAULT 25,
                        p_username    IN VARCHAR2 DEFAULT NULL,
                        p_password    IN VARCHAR2 DEFAULT NULL) RETURN VARCHAR2 IS
    vError varchar2(500);

    vIdx PLS_INTEGER := 1;
    vMaxLong PLS_INTEGER := 32767;
    vLongBody PLS_INTEGER := DBMS_LOB.GetLength(P_BODY);
    vServidorCorreo VARCHAR2(100) := p_smtp_host;
    vConnect utl_smtp.connection;
  BEGIN
    vConnect := utl_smtp.open_connection(vServidorCorreo, p_smtp_port);

    -- Autenticaci 
    IF (p_username is not null) then
      utl_smtp.command( vConnect, 'AUTH LOGIN');
      utl_smtp.command( vConnect, utl_raw.cast_to_varchar2( utl_encode.base64_encode( utl_raw.cast_to_raw( p_username ))) );
      utl_smtp.command( vConnect, utl_raw.cast_to_varchar2( utl_encode.base64_encode( utl_raw.cast_to_raw( p_password ))) );
    end if;

    utl_smtp.helo(vConnect, vServidorCorreo);
    utl_smtp.mail(vConnect, P_FROM);
    utl_smtp.rcpt(vConnect, P_TO);
    utl_smtp.open_data (vConnect);
    utl_smtp.write_data(vConnect, 'From: ' || P_FROM || utl_tcp.crlf);
    utl_smtp.write_data(vConnect, 'To: ' || P_TO || utl_tcp.crlf);
    If (P_CC is not null) then
      utl_smtp.write_data(vConnect, 'Cc: ' || P_CC || utl_tcp.crlf);
    end if;
    utl_smtp.write_data(vConnect, 'Subject: ' || P_SUBJECT || utl_tcp.crlf);
    utl_smtp.write_data(vConnect, 'Content-Type: text/plain; charset=WE8ISO8859P1'||utl_tcp.CRLF);

    While vIdx <= vLongBody loop
      utl_smtp.write_raw_data(vConnect, utl_raw.cast_to_raw(DBMS_LOB.SubStr(P_BODY, vMaxLong, vIdx)));
      vIdx := vIdx + vMaxLong;
    end loop;

    utl_smtp.write_data(vConnect, utl_tcp.crlf);

    utl_smtp.close_data (vConnect);
    utl_smtp.quit(vConnect);

    return null;

  EXCEPTION -- Gesti d' errors 
    WHEN utl_smtp.transient_error OR utl_smtp.permanent_error THEN
      BEGIN
        utl_smtp.quit(vConnect);
      EXCEPTION
        WHEN utl_smtp.transient_error OR utl_smtp.permanent_error THEN
          null;
      END;
      vError := SubStr(SQLERRM, 1, 500);
      return vError;
      RAISE_APPLICATION_ERROR(-20000,'Failed to send mail due to the following error: ' || sqlerrm);
    WHEN OTHERS THEN
      utl_smtp.quit(vConnect);
      vError := SubStr(SQLERRM, 1, 500);
      return vError;
  END;

  -- Procediment que afegeix una partici a la taula indicada.
  -- Parmetres d' entrada:
  --   P_TABLE_OWNER     => Propietari de la taula 
  --   P_TABLE_NAME      => Nom de la taula 
  --   P_PARTITION_NAME  => Nom de la partici 
  --   P_TABLESPACE_PART => Tablespace de la partici 
  --   P_MAX_DATE        => Data mxima de la partici 
  -- Parmetres de sortida: No t parmetres de sortida
  PROCEDURE ADD_PARTITION(P_TABLE_OWNER     VARCHAR2,
                          P_TABLE_NAME      VARCHAR2,
                          P_PARTITION_NAME  VARCHAR2,
                          P_TABLESPACE_PART VARCHAR2,
                          P_MAX_DATE        DATE) is
  BEGIN
    EXECUTE IMMEDIATE 'ALTER TABLE '||P_TABLE_OWNER||'.'||P_TABLE_NAME||' ADD PARTITION '||P_PARTITION_NAME||' VALUES LESS THAN (TO_DATE('''||TO_CHAR(P_MAX_DATE, 'DD-MM-YYYY HH24:MI:SS')||''', ''DD-MM-YYYY HH24:MI:SS'')) TABLESPACE '||P_TABLESPACE_PART;
  END;

  -- Procediment que elimina la partici indicada de la taula donada.
  -- Parmetres d' entrada:
  --   P_TABLE_OWNER     => Propietari de la taula 
  --   P_TABLE_NAME      => Nom de la taula 
  --   P_PARTITION_NAME  => Nom de la partici 
  -- Parmetres de sortida: No t parmetres de sortida
  PROCEDURE DROP_PARTITION(P_TABLE_OWNER    VARCHAR2,
                           P_TABLE_NAME     VARCHAR2,
                           P_PARTITION_NAME VARCHAR2) IS
  BEGIN
    EXECUTE IMMEDIATE 'ALTER TABLE '||P_TABLE_OWNER||'.'||P_TABLE_NAME||' DROP PARTITION '||P_PARTITION_NAME;
  END;
  
  -- Funci que mira si la taula i partici existeixen .
  -- Parmetres d' entrada:
  --   P_TABLE_OWNER     => Propietari de la taula 
  --   P_TABLE_NAME      => Nom de la taula 
  --   P_PARTITION_NAME  => Nom de la partici 
  -- Parmetres de sortida: No t parmetres de sortida
  -- Retorn de la funci: Retorna true o false en funci de si existeix la partici o no.
  --     TRUE  => Existeix la partici 
  --     FALSE => No existeix la partici 
  FUNCTION EXISTE_TABLA(P_TABLE_OWNER VARCHAR2,
                        P_TABLE_NAME VARCHAR2,
                        P_PARTITION_NAME VARCHAR2 DEFAULT NULL) RETURN BOOLEAN IS
    vDummy varchar2(1) := '0';
    vSelect varchar2(300);
  BEGIN
    vSelect := 'Select ''1'' from '||P_TABLE_OWNER||'.'||P_TABLE_NAME;
    If (P_PARTITION_NAME is not null) then
      vSelect := vSelect || ' PARTITION ('||P_PARTITION_NAME||')';
    end if;

    vSelect := vSelect ||' WHERE ROWNUM = 1';

    Execute immediate vSelect into vDummy;

    return vDummy = '1';
  EXCEPTION
    WHEN NO_DATA_FOUND THEN
      return true;
    WHEN OTHERS THEN
      return false;
  END;

  -- Funci que mira si la taula i partici donats contenen dades.
  -- Parmetres d' entrada:
  --   P_TABLE_OWNER     => Propietari de la taula 
  --   P_TABLE_NAME      => Nom de la taula 
  --   P_PARTITION_NAME  => Nom de la partici 
  -- Parmetres de sortida: No t parmetres de sortida
  -- Retorn de la funci: Retorna true o false en funci de si la partici cont dades o no.
  --     TRUE  => Cont dades 
  --     FALSE => No cont dades 
  FUNCTION TABLA_CONTIENE_DATOS(P_TABLE_OWNER VARCHAR2,
                                P_TABLE_NAME VARCHAR2,
                                P_PARTITION_NAME VARCHAR2 DEFAULT NULL) RETURN BOOLEAN IS
    vDummy varchar2(1) := '0';
    vSelect varchar2(300);
  BEGIN
    vSelect := 'Select ''1'' from '||P_TABLE_OWNER||'.'||P_TABLE_NAME;
    If (P_PARTITION_NAME is not null) then
      vSelect := vSelect || ' PARTITION ('||P_PARTITION_NAME||')';
    end if;

    vSelect := vSelect ||' WHERE ROWNUM = 1';

    Execute immediate vSelect into vDummy;

    return vDummy = '1';
  EXCEPTION
    WHEN NO_DATA_FOUND THEN
      return false;
  END;

  -- Funci mira si un fitxer existeix.
  -- Parmetres d' entrada:
  --   P_DIRECTORI_BD => Directori de base de dades  
  --   P_NOM_FITXER   => Nom del fitxer 
  -- Parmetres de sortida: No t parmetres de sortida
  -- Retorn de la funci: Retorna true o false en funci de si existeix o no el fitxer donat en el directori de base de dades indicat 
  --     TRUE  => Existeix el fitxer
  --     FALSE => No existeix el fitxer 
  FUNCTION EXISTEIX_FITXER(P_DIRECTORI_BD varchar2,
                           P_NOM_FITXER   varchar2) RETURN BOOLEAN IS
    v_exists BOOLEAN;
    v_length NUMBER;
    v_blocksize NUMBER;
  BEGIN
    UTL_FILE.fgetattr (P_DIRECTORI_BD, P_NOM_FITXER, v_exists, v_length, v_blocksize);

    return v_exists;
  EXCEPTION
    when others then
      return false;
  END;

  -- Funci que carrega el contingut d' un fitxer pla i el retorna com a CLOB
  -- Parmetres d' entrada:
  --   P_DIRECTORI_BD => Directori de base de dades  
  --   P_NOM_FITXER   => Nom del fitxer 
  -- Parmetres de sortida: No t parmetres de sortida
  -- Retorn de la funci: Contingut del fitxer en format CLOB 
  FUNCTION OBT_CONTINGUT_FITXER_PLA(P_DIRECTORI_BD VARCHAR2,
                                    P_NOM_FITXER   VARCHAR2) RETURN CLOB IS
    vFitxer_CLOB clob;
    vFitxer_BFILE BFILE;
    vIdx          smallint;
  BEGIN
    dbms_lob.createtemporary(vFitxer_CLOB, false);
    DBMS_LOB.OPEN(vFitxer_CLOB, DBMS_LOB.LOB_READWRITE);
    vFitxer_BFILE := BFILENAME(P_DIRECTORI_BD, P_NOM_FITXER);
    DBMS_LOB.fileopen (vFitxer_BFILE, DBMS_LOB.file_readonly);
    DBMS_LOB.LOADFROMFILE(vFitxer_CLOB, vFitxer_BFILE, DBMS_LOB.LOBMAXSIZE, 1, 1);
    dbms_lob.close(vFitxer_CLOB);
    dbms_lob.close(vFitxer_BFILE);

    return vFitxer_CLOB;
  END;

  -- Funci que afegeix un adjunt a la variable P_ADJUTNS
  -- Parmetres d' entrada i sortida:
  --   P_ADJUNTS      => Llista de fitxers a adjuntar 
  -- Parmetre noms d' entrada: 
  --   P_DIRECTORI_BD => Directori de base de dades  
  --   P_NOM_FITXER   => Nom del fitxer 
  --   P_MIME_ADJUNT  => Tipus mime del fitxer adjunt 
  -- Parmetres noms de sortida: No t parmetres de sortida
  -- Retorn de la funci: 
  --   NULL          => No s' ha produt cap error 
  --   ERROR         => En cas d' error retorna l' error.
  FUNCTION AFEGIR_MAIL_ADJUNT(P_ADJUNTS      IN OUT table_attachments,
                              P_DIRECTORI_BD VARCHAR2,
                              P_NOM_FITXER   VARCHAR2,
                              P_MIME_ADJUNT  VARCHAR2 default 'text/plain') RETURN VARCHAR2 IS
    vFitxer_BLOB  BLOB;
    vFitxer_BFILE BFILE;
    vIdx          smallint;
    vError        varchar2(1000);
  BEGIN
    If (
        (P_DIRECTORI_BD is null) or
        (P_NOM_FITXER   is null)
       )
    then
      vError := 'No se ha informado el directorio o el nombre de fichero';
    else
      If (EXISTEIX_FITXER(P_DIRECTORI_BD, P_NOM_FITXER)) then
        dbms_lob.createtemporary(vFitxer_BLOB, false);
        DBMS_LOB.OPEN(vFitxer_BLOB, DBMS_LOB.LOB_READWRITE);
        vFitxer_BFILE := BFILENAME(P_DIRECTORI_BD, P_NOM_FITXER);
        DBMS_LOB.fileopen (vFitxer_BFILE, DBMS_LOB.file_readonly);
        DBMS_LOB.LOADFROMFILE(vFitxer_BLOB, vFitxer_BFILE, DBMS_LOB.LOBMAXSIZE, 1, 1);
        vIdx := P_ADJUNTS.count + 1;
        P_ADJUNTS(vIdx).attachment_name := P_NOM_FITXER;
        P_ADJUNTS(vIdx).attachment_mime := P_MIME_ADJUNT;
        P_ADJUNTS(vIdx).attachment_blob := vFitxer_BLOB;
        dbms_lob.close(vFitxer_BLOB);
        dbms_lob.close(vFitxer_BFILE);
      else
        vError := 'No existe el fichero "'||P_NOM_FITXER||'" en el directorio de base de datos "'||P_DIRECTORI_BD||'". ';
      end if;
    end if;

    return vError;
  EXCEPTION
    WHEN OTHERS THEN
      vError := SubStr(SQLERRM, 1, 1000);
      return vError;
  END;

  -- Procediment que buida la taula passada per parmetre
  -- Parmetres d' entrada:
  --   P_PROPIETARI => Propietari de la taula   
  --   P_TAULA      => Nom de la taula 
  -- Parmetres de sortida: No t parmetres de sortida
  PROCEDURE BUIDAR_TAULA(P_PROPIETARI VARCHAR2,
                         P_TAULA      VARCHAR2) IS
  BEGIN
    Execute immediate 'Truncate table '||P_PROPIETARI||'.'||P_TAULA;
  END;
  
  -- Funci que retorna el valor d' una resposta OK 
  -- Parmetres d' entrada: No t parmetres d' entrada 
  -- Parmetres de sortida: No t parmetres de sortida .
  -- Retorn de la funci:
  --     OK => Valor d' una resposta correcte.   
  FUNCTION RSP_OK RETURN VARCHAR2 IS
  BEGIN
    return cRsp_OK;
  END;

  -- Funci que retorna el valor INC 
  -- Parmetres d' entrada: No t parmetres d' entrada 
  -- Parmetres de sortida: No t parmetres de sortida .
  -- Retorn de la funci:
  --     INC => Valor per el clcul incremental.  
  FUNCTION CALCUL_INC RETURN VARCHAR2 IS
  BEGIN
    return cCalc_Ind_INC;
  END;

  -- Funci que retorna el valor TOT 
  -- Parmetres d' entrada: No t parmetres d' entrada 
  -- Parmetres de sortida: No t parmetres de sortida .
  -- Retorn de la funci:
  --     TOT  => Valor per el clcul incremental.   
  FUNCTION CALCUL_TOT RETURN VARCHAR2 IS
  BEGIN
    return cCalc_Ind_TOT;
  END;
  
  -- Funci que retorna el valor del parmetre TIPUS_CALCUL_INDICADORS  
  -- Parmetres d' entrada: No t parmetres d' entrada 
  -- Parmetres de sortida: No t parmetres de sortida .
  -- Retorn de la funci:
  --     Valor del parmetre TIPUS_CALCUL_INDICADORS => Est definit a la taula TFC_PARAMETRES 
  FUNCTION TIPUS_CALCUL_INDICADORS RETURN VARCHAR2 IS 
  BEGIN
    If (vTipus_Calcul_Indicadors is null) then 
      Select p.valpar 
        into vTipus_Calcul_Indicadors
        from TFC_PARAMETRES p
        where p.Nompar = 'TIPUS_CALCUL_INDICADORS';
        
      vTipus_Calcul_Indicadors := nvl(vTipus_Calcul_Indicadors, 'TOT');
      
      If (vTipus_Calcul_Indicadors not in ('INC','TOT')) then 
        vTipus_Calcul_Indicadors := 'TOT';
      end if;
    end if;
    
    return vTipus_Calcul_Indicadors;
  EXCEPTION
    when others then 
      vTipus_Calcul_Indicadors := 'TOT';
      return vTipus_Calcul_Indicadors;
  END;
  
  -- Funci que verifica si el clcul d' indicadors s' ha de fer cada venda   
  -- Parmetres d' entrada: No t parmetres d' entrada 
  -- Parmetres de sortida: No t parmetres de sortida .
  -- Retorn de la funci:
  --     TRUE  => El clcul d' indicadors s' ha de fer per cada venda  
  --     FLASE => El clcul d' indicadors NO s' ha de fer per cada venda. S' haur d' invocar manualment desprs de la crrega de vendes 
  FUNCTION CALCULAR_IND_PER_CADA_VENDA RETURN BOOLEAN IS 
    vDummy varchar2(1);
  BEGIN
    Select nvl(p.valpar, 'N')
      into vDummy 
      from TFC_PARAMETRES p
      where p.nompar = 'CALC_IND_PER_CADA_VENDA';
  
    vCalcular_Indicador_Per_Venda := (vDummy = 'S');
    
    return vCalcular_Indicador_Per_Venda;
  EXCEPTION
    WHEN OTHERS THEN 
      vCalcular_Indicador_Per_Venda := FALSE;
      RETURN vCalcular_Indicador_Per_Venda;
  END;
  
  -- Funci que retorna el valor del parmetre P_NOMPAR segons s' hagi definit a la taula TFC_PARAMETRES    
  -- Parmetres d' entrada: 
  --   P_NOMPAR => Nom del parmetre a buscar 
  -- Parmetres de sortida: No t parmetres de sortida .
  -- Retorn de la funci: Valor del parmetre P_NOMPAR segons s' hagi definit a la taula TFC_PARAMETRES 
  FUNCTION OBT_VAL_PAR(P_NOMPAR TFC_PARAMETRES.Nompar%Type) return tfc_parametres.valpar%Type IS
    vValPar tfc_parametres.valpar%Type;
  BEGIN
    IF (P_NOMPAR IS NOT NULL) THEN 
      Select VALPAR 
        into vValPar 
        from TFC_PARAMETRES par
        where par.Nompar = P_NOMPAR;
    END IF;
    
    return vValPar;
  EXCEPTION
    WHEN NO_DATA_FOUND THEN 
      RETURN NULL;
  END;
  
  -- Funci que retorna un nom de fitxer nic en el directori indiciat partint del nom passat com a parmetre 
  -- Parmetres d' entrada: 
  --   P_DIR_BD  => Directori de base de dades
  --   P_NOM_FIC => Nom del fitxer a verificar 
  -- Parmetres de sortida: No t parmetres de sortida .
  -- Retorn de la funci: Nom nic dintre del directori de base de dades i partint del nom P_NOM_FIC 
  FUNCTION OBT_NOM_FITXER_UNIC(P_DIR_BD  varchar2, 
                               P_NOM_FIC varchar2) return varchar2 IS
    vSeq            smallint := 0;
    vMax_Iteracions smallint := 1000;
    vNom_Fitxer     varchar2(100);
    vError          exception;
  BEGIN
    If (P_DIR_BD is not null) and (P_NOM_FIC is not null) then 
      vNom_Fitxer := P_NOM_FIC;

      While EXISTEIX_FITXER(P_DIR_BD, vNom_Fitxer) loop
        vSeq := vSeq + 1;
      
        vNom_Fitxer := P_NOM_FIC||'_'||vSeq;
      
        If (vSeq > vMax_Iteracions) then 
          raise vError;
        end if;
      end loop;
    end if;
    
    return vNom_Fitxer;
  END;
end TFC_UTL;
/
