create or replace package TFC_VEN is
  -- Procediment que inserta un registre a la taula TFC_VENDES 
  -- Parmetres d' entrada:
  --   P_DATA     => Data de la venda 
  --   P_HORA     => Hora de la venda 
  --   P_IDEBOT   => Identificador de la botiga que ha fet la venda 
  --   P_CODEAN13 => Codi del producte 
  --   P_QNTVEN   => Quantitat venuda 
  --   P_BENNET   => Benefici net total 
  --   P_IMPBRUT  => Import brut 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE ALT_VENDA(P_DATA     TFC_VENDES.TEM_DATA%type,
                      P_HORA     TFC_VENDES.TEM_HORA%type,
                      P_IDEBOT   TFC_VENDES.BOT_IDEBOT%type,
                      P_CODEAN13 TFC_VENDES.PRO_CODEAN13%type,
                      P_QNTVEN   TFC_VENDES.QNTVEN%type,
                      P_BENNET   TFC_VENDES.BENNET%type,
                      P_IMPBRUT  TFC_VENDES.IMPBRUT%type,
                      RSP        OUT VARCHAR2);

  -- Procediment que modifica un registre de la taula TFC_VENDES 
  -- Parmetres d' entrada:
  --   P_DATA     => Data de la venda 
  --   P_HORA     => Hora de la venda 
  --   P_IDEBOT   => Identificador de la botiga que ha fet la venda 
  --   P_CODEAN13 => Codi del producte 
  --   P_QNTVEN   => Quantitat venuda 
  --   P_BENNET   => Benefici net total 
  --   P_IMPBRUT  => Import brut 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE MOD_VENDA(P_DATA     TFC_VENDES.TEM_DATA%type,
                      P_HORA     TFC_VENDES.TEM_HORA%type,
                      P_IDEBOT   TFC_VENDES.BOT_IDEBOT%type,
                      P_CODEAN13 TFC_VENDES.PRO_CODEAN13%type,
                      P_QNTVEN   TFC_VENDES.QNTVEN%type,
                      P_BENNET   TFC_VENDES.BENNET%type,
                      P_IMPBRUT  TFC_VENDES.IMPBRUT%type,
                      RSP        OUT VARCHAR2);

  -- Procediment que elimina un registre de la taula TFC_VENDES 
  -- Parmetres d' entrada:
  --   P_DATA     => Data de la venda 
  --   P_HORA     => Hora de la venda 
  --   P_IDEBOT   => Identificador de la botiga que ha fet la venda 
  --   P_CODEAN13 => Codi del producte 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE SUP_VENDA(P_DATA     TFC_VENDES.TEM_DATA%type,
                      P_HORA     TFC_VENDES.TEM_HORA%type,
                      P_IDEBOT   TFC_VENDES.BOT_IDEBOT%type,
                      P_CODEAN13 TFC_VENDES.PRO_CODEAN13%type,
                      RSP        OUT VARCHAR2);
end TFC_VEN;
/
create or replace package body TFC_VEN is
  Type T_ANYS_PART is table of varchar2(1) index by varchar2(4);

  vTFC_VENDES_Particionada boolean;
  vAnys_Particionats       T_ANYS_PART;
  
  -- Creaci de particions si cal 
  FUNCTION REVISAR_I_CREAR_PART(P_NUMANY number) RETURN BOOLEAN IS 
    vNom_Taula       varchar2(20) := 'TFC_VENDES';
    vNom_Particio    varchar2(20);
    vPropietari      varchar2(20) := user;
    vMax_Date        date;
    vTablespace_Part varchar2(50) := TFC_UTL.OBT_VAL_PAR('TABLESPACE_PART');
  BEGIN
    For mes in 1..12 loop
      vNom_Particio := 'PAR_'||P_NUMANY||lpad(mes,2,'0');
      If (not TFC_UTL.EXISTE_TABLA(vPropietari, vNom_Taula, vNom_Particio)) then
        If (mes < 12) then 
          vMax_Date := TO_DATE('01/'||lpad((mes  + 1),2,'0')||'/'||P_NUMANY, 'DD/MM/YYYY');
        else
          vMax_Date := TO_DATE('01/01/'||(P_NUMANY + 1), 'DD/MM/YYYY');
        end if;
        
        TFC_UTL.ADD_PARTITION(vPropietari, vNom_Taula, vNom_Particio, vTablespace_Part, vMax_Date);
      end if;
    end loop;
    RETURN TRUE;
  EXCEPTION
    WHEN OTHERS THEN 
      RETURN FALSE;
  END;
  
  -- Procediment que verifica si la taula est particionada. 
  -- En cas que estigui particiona, crea les particions necessaries  
  PROCEDURE VERIFICAR_PARTICIONAMENT(P_NUMANY number) IS 
    vDummy varchar2(1) := 'N';
  BEGIN
    -- S' ha verificat si la base de dades est particionada?
    If (vTFC_VENDES_Particionada is null) then 
      begin
        Select 'S'
          into vDummy
          from USER_TABLES ut
          where upper(ut.partitioned) = 'YES' and 
                ut.table_name = 'TFC_VENDES';
      exception
        when no_data_found then 
          vDummy := 'N';
      end;   
      vTFC_VENDES_Particionada := (vDummy = 'S');
    end if;
    
    -- Si la taula de vendes s' ha particiona es verifica si existeixen totes les particions per l' any donat 
    If (vTFC_VENDES_Particionada) then 
      IF (vAnys_Particionats.count = 0) or (not vAnys_Particionats.Exists(to_char(P_NUMANY))) then 
        If (REVISAR_I_CREAR_PART(P_NUMANY)) then 
          vAnys_Particionats(to_char(P_NUMANY)) := 'S';
        end if;
      end if;
    end if;
  END;

  PROCEDURE INDICADORS_AGR_ALT(P_DATA     TFC_VENDES.TEM_DATA%type,
                               P_HORA     TFC_VENDES.TEM_HORA%type,
                               P_IDEBOT   TFC_VENDES.BOT_IDEBOT%type,
                               P_CODEAN13 TFC_VENDES.PRO_CODEAN13%type,
                               P_QNTVEN   TFC_VENDES.QNTVEN%type,
                               P_BENNET   TFC_VENDES.BENNET%type,
                               RSP        out varchar2) IS
    vNumAny       number := to_number(TO_CHAR(P_DATA, 'YYYY'));
    vError        Exception;
    vProces_Error varchar2(100);
    vIdeCiu       TFC_CIUTATS.IDECIU%Type;
  BEGIN
    TFC_APA.ALT_APA  (vNumAny, P_CODEAN13, P_QNTVEN, RSP);
    If (RSP <> TFC_UTL.RSP_OK) then 
      vProces_Error := 'TFC_APA.ALT_APA';
      raise vError;
    end if;
    
    TFC_ABA.ALT_ABA  (vNumAny, P_IDEBOT, P_BENNET, RSP);
    If (RSP <> TFC_UTL.RSP_OK) then 
      vProces_Error := 'TFC_ABA.ALT_ABA';
      raise vError;
    end if;
    
    vIdeCiu := TFC_BOT.OBT_IDECIU(P_IDEBOT, RSP);
    If (RSP <> TFC_UTL.RSP_OK) then 
      vProces_Error := 'TFC_BOT.OBT_IDECIU';
      raise vError;
    end if;

    TFC_ACA.ALT_ACA  (vNumAny, vIdeCiu, P_BENNET, RSP);
    If (RSP <> TFC_UTL.RSP_OK) then 
      vProces_Error := 'TFC_ACA.ALT_ACA';
      raise vError;
    end if;
    
    TFC_AHA.ALT_AHA  (vNumAny, P_HORA, P_QNTVEN, RSP);
    If (RSP <> TFC_UTL.RSP_OK) then 
      vProces_Error := 'TFC_AHA.ALT_AHA';
      raise vError;
    end if;
    
    TFC_ADMA.ALT_ADMA(vNumAny, to_number(to_number(TO_CHAR(P_DATA, 'DD'))), P_QNTVEN, RSP);
    If (RSP <> TFC_UTL.RSP_OK) then 
      vProces_Error := 'TFC_ADMA.ALT_ADMA';
      raise vError;
    end if;
  EXCEPTION -- Tractament d' excepcions 
    WHEN others THEN 
      If (vProces_Error is null) then -- Si vProces_Error es null, es tracta d' un error no controlat 
        RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
      else
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Error executant el procs '||vProces_Error||':');
      end if;
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
  END;
  
  PROCEDURE INDICADORS_AGR_MOD(P_DATA        TFC_VENDES.TEM_DATA%type,
                               P_HORA        TFC_VENDES.TEM_HORA%type,
                               P_IDEBOT      TFC_VENDES.BOT_IDEBOT%type,
                               P_CODEAN13    TFC_VENDES.PRO_CODEAN13%type,
                               P_QNTVEN_OLD  TFC_VENDES.QNTVEN%type,
                               P_BENNET_OLD  TFC_VENDES.BENNET%type,
                               P_QNTVEN_NEW  TFC_VENDES.QNTVEN%type,
                               P_BENNET_NEW  TFC_VENDES.BENNET%type,
                               RSP        out varchar2) IS
    vNumAny       number := to_number(TO_CHAR(P_DATA, 'YYYY'));
    vError        Exception;
    vProces_Error varchar2(100);
    vIdeCiu       TFC_CIUTATS.IDECIU%Type;
  BEGIN
    TFC_APA.MOD_APA  (vNumAny, P_CODEAN13, (P_QNTVEN_NEW - P_QNTVEN_OLD), RSP);
    If (RSP <> TFC_UTL.RSP_OK) then 
      vProces_Error := 'TFC_APA.MOD_APA';
      raise vError;
    end if;
    
    TFC_ABA.MOD_ABA  (vNumAny, P_IDEBOT, (P_BENNET_NEW - P_BENNET_OLD), RSP);
    If (RSP <> TFC_UTL.RSP_OK) then 
      vProces_Error := 'TFC_ABA.MOD_ABA';
      raise vError;
    end if;
    
    vIdeCiu := TFC_BOT.OBT_IDECIU(P_IDEBOT, RSP);
    If (RSP <> TFC_UTL.RSP_OK) then 
      vProces_Error := 'TFC_BOT.OBT_IDECIU';
      raise vError;
    end if;

    TFC_ACA.MOD_ACA  (vNumAny, vIdeCiu, (P_BENNET_NEW - P_BENNET_OLD), RSP);
    If (RSP <> TFC_UTL.RSP_OK) then 
      vProces_Error := 'TFC_ACA.MOD_ACA';
      raise vError;
    end if;
    
    TFC_AHA.MOD_AHA  (vNumAny, P_HORA, (P_QNTVEN_NEW - P_QNTVEN_OLD), RSP);
    If (RSP <> TFC_UTL.RSP_OK) then 
      vProces_Error := 'TFC_AHA.MOD_AHA';
      raise vError;
    end if;
    
    TFC_ADMA.MOD_ADMA(vNumAny, to_number(to_number(TO_CHAR(P_DATA, 'DD'))), (P_QNTVEN_NEW - P_QNTVEN_OLD), RSP);
    If (RSP <> TFC_UTL.RSP_OK) then 
      vProces_Error := 'TFC_ADMA.MOD_ADMA';
      raise vError;
    end if;
  EXCEPTION -- Tractament d' excepcions 
    WHEN others THEN 
      If (vProces_Error is null) then -- Si vProces_Error es null, es tracta d' un error no controlat 
        RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
      else
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Error executant el procs '||vProces_Error||':');
      end if;
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
  END;

  PROCEDURE INDICADORS_AGR_SUP(P_DATA     TFC_VENDES.TEM_DATA%type,
                               P_HORA     TFC_VENDES.TEM_HORA%type,
                               P_IDEBOT   TFC_VENDES.BOT_IDEBOT%type,
                               P_CODEAN13 TFC_VENDES.PRO_CODEAN13%type,
                               P_QNTVEN_OLD  TFC_VENDES.QNTVEN%type,
                               P_BENNET_OLD  TFC_VENDES.BENNET%type,
                               P_QNTVEN_NEW  TFC_VENDES.QNTVEN%type,
                               P_BENNET_NEW  TFC_VENDES.BENNET%type,
                               RSP        out varchar2) IS
    vNumAny       number := to_number(TO_CHAR(P_DATA, 'YYYY'));
    vError        Exception;
    vProces_Error varchar2(100);
    vIdeCiu       TFC_CIUTATS.IDECIU%Type;
  BEGIN
    TFC_APA.SUP_APA  (vNumAny, P_CODEAN13, (P_QNTVEN_NEW - P_QNTVEN_OLD), RSP);
    If (RSP <> TFC_UTL.RSP_OK) then 
      vProces_Error := 'TFC_APA.SUP_APA';
      raise vError;
    end if;
    
    TFC_ABA.SUP_ABA  (vNumAny, P_IDEBOT, (P_BENNET_NEW - P_BENNET_OLD), RSP);
    If (RSP <> TFC_UTL.RSP_OK) then 
      vProces_Error := 'TFC_ABA.SUP_ABA';
      raise vError;
    end if;
    
    vIdeCiu := TFC_BOT.OBT_IDECIU(P_IDEBOT, RSP);
    If (RSP <> TFC_UTL.RSP_OK) then 
      vProces_Error := 'TFC_BOT.OBT_IDECIU';
      raise vError;
    end if;

    TFC_ACA.SUP_ACA  (vNumAny, vIdeCiu, (P_BENNET_NEW - P_BENNET_OLD), RSP);
    If (RSP <> TFC_UTL.RSP_OK) then 
      vProces_Error := 'TFC_ACA.SUP_ACA';
      raise vError;
    end if;
    
    TFC_AHA.SUP_AHA  (vNumAny, P_HORA, (P_QNTVEN_NEW - P_QNTVEN_OLD), RSP);
    If (RSP <> TFC_UTL.RSP_OK) then 
      vProces_Error := 'TFC_AHA.SUP_AHA';
      raise vError;
    end if;
    
    TFC_ADMA.SUP_ADMA(vNumAny, to_number(to_number(TO_CHAR(P_DATA, 'DD'))), (P_QNTVEN_NEW - P_QNTVEN_OLD), RSP);
    If (RSP <> TFC_UTL.RSP_OK) then 
      vProces_Error := 'TFC_ADMA.SUP_ADMA';
      raise vError;
    end if;
  EXCEPTION -- Tractament d' excepcions 
    WHEN others THEN 
      If (vProces_Error is null) then -- Si vProces_Error es null, es tracta d' un error no controlat 
        RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
      else
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Error executant el procs '||vProces_Error||':');
      end if;
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
  END;

  PROCEDURE INDICADORS_AGR_SUP(P_DATA     TFC_VENDES.TEM_DATA%type,
                               P_HORA     TFC_VENDES.TEM_HORA%type,
                               P_IDEBOT   TFC_VENDES.BOT_IDEBOT%type,
                               P_CODEAN13 TFC_VENDES.PRO_CODEAN13%type,
                               RSP        out varchar2) IS
    vNumAny       number := to_number(TO_CHAR(P_DATA, 'YYYY'));
    vError        Exception;
    vProces_Error varchar2(100);
    vIdeCiu       TFC_CIUTATS.IDECIU%Type;
  BEGIN
    TFC_APA.SUP_APA  (vNumAny, P_CODEAN13, RSP);
    If (RSP <> TFC_UTL.RSP_OK) then 
      vProces_Error := 'TFC_APA.SUP_APA';
      raise vError;
    end if;
    
    TFC_ABA.SUP_ABA  (vNumAny, P_IDEBOT, RSP);
    If (RSP <> TFC_UTL.RSP_OK) then 
      vProces_Error := 'TFC_ABA.SUP_ABA';
      raise vError;
    end if;
    
    vIdeCiu := TFC_BOT.OBT_IDECIU(P_IDEBOT, RSP);
    If (RSP <> TFC_UTL.RSP_OK) then 
      vProces_Error := 'TFC_BOT.OBT_IDECIU';
      raise vError;
    end if;

    TFC_ACA.SUP_ACA  (vNumAny, vIdeCiu, RSP);
    If (RSP <> TFC_UTL.RSP_OK) then 
      vProces_Error := 'TFC_ACA.SUP_ACA';
      raise vError;
    end if;
    
    TFC_AHA.SUP_AHA  (vNumAny, P_HORA, RSP);
    If (RSP <> TFC_UTL.RSP_OK) then 
      vProces_Error := 'TFC_AHA.SUP_AHA';
      raise vError;
    end if;
    
    TFC_ADMA.SUP_ADMA(vNumAny, to_number(to_number(TO_CHAR(P_DATA, 'DD'))), RSP);
    If (RSP <> TFC_UTL.RSP_OK) then 
      vProces_Error := 'TFC_ADMA.SUP_ADMA';
      raise vError;
    end if;
  EXCEPTION -- Tractament d' excepcions 
    WHEN others THEN 
      If (vProces_Error is null) then -- Si vProces_Error es null, es tracta d' un error no controlat 
        RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
      else
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Error executant el procs '||vProces_Error||':');
      end if;
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
  END;

  -- Procediment que inserta un registre a la taula TFC_VENDES 
  -- Parmetres d' entrada:
  --   P_DATA     => Data de la venda 
  --   P_HORA     => Hora de la venda 
  --   P_IDEBOT   => Identificador de la botiga que ha fet la venda 
  --   P_CODEAN13 => Codi del producte 
  --   P_QNTVEN   => Quantitat venuda 
  --   P_BENNET   => Benefici net total 
  --   P_IMPBRUT  => Import brut 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE ALT_VENDA(P_DATA     TFC_VENDES.TEM_DATA%type,
                      P_HORA     TFC_VENDES.TEM_HORA%type,
                      P_IDEBOT   TFC_VENDES.BOT_IDEBOT%type,
                      P_CODEAN13 TFC_VENDES.PRO_CODEAN13%type,
                      P_QNTVEN   TFC_VENDES.QNTVEN%type,
                      P_BENNET   TFC_VENDES.BENNET%type,
                      P_IMPBRUT  TFC_VENDES.IMPBRUT%type,
                      RSP        OUT VARCHAR2) IS 
    vNomProces TFC_RASTRE_PROCES.PROCES%Type := 'TFC_VEN.ALT_VENDA';
    vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
    vError             Exception;
    vProces_Error      varchar2(100);
    vError_FK          exception;
    PRAGMA EXCEPTION_INIT(vError_FK, -02291);
  BEGIN
    If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
      TFC_RAP.INICIAR_RASTRE(vNomProces);
    end if;
    
    -- Es graven els parmetres a la taula de rastre
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_DATA='    ||TO_CHAR(P_DATA,'DD-MM-YYYY'));
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_HORA='    ||P_HORA);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_IDEBOT='  ||P_IDEBOT);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_CODEAN13='||P_CODEAN13);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_QNTVEN='  ||P_QNTVEN);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_BENNET='  ||P_BENNET);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_IMPBRUT=' ||P_IMPBRUT);
    
    -- Es verifica el particionament de la taula 
    VERIFICAR_PARTICIONAMENT(to_number(to_char(P_DATA, 'YYYY')));
    
    -- Es verifica que existeix el registre pare a la taula TFC_TEMPS 
    -- En cs que no existeixi es dna d' alta.
    If (not TFC_TEM.EXISTEIX_TEMPS(P_DATA, P_HORA, RSP)) then 
      If (RSP <> TFC_UTL.RSP_OK) then 
        vProces_Error := 'TFC_TEM.EXISTEIX_TEMPS';
        raise vError;
      end if;
      
      TFC_TEM.ALT_TEMPS(P_DATA, P_HORA, RSP);
      
      If (RSP <> TFC_UTL.RSP_OK) then 
        vProces_Error := 'TFC_TEM.ALT_TEMPS';
        raise vError;
      end if;
    end if;
    
    Update TFC_VENDES ven
       set ven.QNTVEN         = ven.QNTVEN  + nvl(P_QNTVEN, 0),
           ven.BENNET         = ven.BENNET  + nvl(P_BENNET, 0),
           ven.IMPBRUT        = ven.IMPBRUT + nvl(P_IMPBRUT, 0)
       where ven.TEM_DATA     = P_DATA and 
             ven.TEM_HORA     = P_HORA and 
             ven.BOT_IDEBOT   = P_IDEBOT and 
             ven.PRO_CODEAN13 = P_CODEAN13;
    -- Si no existeix el registre a la taula TFC_VENDES es dna d' alta...
    If (sql%RowCount = 0) then    
      begin
        Insert into TFC_VENDES
          (TEM_DATA,
           TEM_HORA,
           BOT_IDEBOT,
           PRO_CODEAN13,
           QNTVEN,
           BENNET,
           IMPBRUT)
        values 
          (trunc(P_DATA),
           P_HORA,
           P_IDEBOT,
           P_CODEAN13,
           P_QNTVEN,
           P_BENNET,
           P_IMPBRUT);
          
        If (SQL%RowCount > 0) then 
          TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre insertat.');
        else
          TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre no insertat.');
        end if; 
      exception
        WHEN DUP_VAL_ON_INDEX THEN -- Si es captura aquesta excepci vol dir que el registre ja existeix.
          -- En aquest cas s' actualitzaran els valors del registre existent.
          RSP := 'Ja existeix una venda per la data = '||to_char(P_DATA, 'DD-MM-YYYY')||', hora = '||P_HORA||', id. de botiga = '||P_IDEBOT||', producte = '||P_CODEAN13;
      end;
    end if;
    
    -- Arribats a aquest punt, tot ha anat b.
    If (RSP is null) then 
      RSP := TFC_UTL.RSP_OK;
    end if;
    
    -- Es calculen els indicadors 
    IF ((RSP = TFC_UTL.RSP_OK) and (TFC_UTL.CALCULAR_IND_PER_CADA_VENDA)) THEN 
      If (TFC_UTL.TIPUS_CALCUL_INDICADORS = TFC_UTL.CALCUL_INC) THEN 
        -- Es calculen les taules d' agregats 
        IF (RSP = TFC_UTL.RSP_OK) then 
          TFC_RAP.REGISTRAR_RASTRE_LINIA('Es calculen els agregats (INDICADORS_AGR_ALT).');
          INDICADORS_AGR_ALT(P_DATA,
                             P_HORA,
                             P_IDEBOT,
                             P_CODEAN13,
                             P_QNTVEN,
                             P_BENNET,
                             RSP);                      
        end if;

        IF (RSP = TFC_UTL.RSP_OK) then 
          TFC_INA.CALC_INA(TO_NUMBER(TO_CHAR(P_DATA, 'YYYY')), RSP);
        END IF;
      ELSIF (TFC_UTL.TIPUS_CALCUL_INDICADORS = TFC_UTL.CALCUL_TOT) THEN 
        TFC_INA.CALCULAR_INDICADORS_TOT(TO_NUMBER(TO_CHAR(P_DATA, 'YYYY')), RSP);
      END IF;
    END IF; 

    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);

    If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
      If (RSP = TFC_UTL.RSP_OK) then 
        TFC_RAP.FINALITZAR_RASTRE_OK;
      else
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    end if;
  EXCEPTION -- Tractament d' excepcions 
    WHEN vError_FK THEN 
      If    (SQLERRM like '%VEN_BOT_FK%') then 
        RSP := 'No existeix cap botiga amb l'' identificador '||P_IDEBOT;
      elsIf (SQLERRM like '%VEN_PRO_FK%') then 
        RSP := 'No existeix cap producte amb el codi '||P_CODEAN13;
      elsIf (SQLERRM like '%VEN_TEM_FK%') then 
        RSP := 'No existeix cap data i hora per '||TO_CHAR(P_DATA, 'DD-MM-YYYY')||' '||P_HORA||':00:00';
      end if;
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    WHEN others THEN 
      If (vProces_Error is null) then -- Si vProces_Error es null, es tracta d' un error no controlat 
        RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
      else
        TFC_RAP.REGISTRAR_RASTRE_LINIA('Error executant el procs '||vProces_Error||':');
      end if;
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
  END;

  -- Procediment que modifica un registre de la taula TFC_VENDES 
  -- Parmetres d' entrada:
  --   P_DATA     => Data de la venda 
  --   P_HORA     => Hora de la venda 
  --   P_IDEBOT   => Identificador de la botiga que ha fet la venda 
  --   P_CODEAN13 => Codi del producte 
  --   P_QNTVEN   => Quantitat venuda 
  --   P_BENNET   => Benefici net total 
  --   P_IMPBRUT  => Import brut 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE MOD_VENDA(P_DATA     TFC_VENDES.TEM_DATA%type,
                      P_HORA     TFC_VENDES.TEM_HORA%type,
                      P_IDEBOT   TFC_VENDES.BOT_IDEBOT%type,
                      P_CODEAN13 TFC_VENDES.PRO_CODEAN13%type,
                      P_QNTVEN   TFC_VENDES.QNTVEN%type,
                      P_BENNET   TFC_VENDES.BENNET%type,
                      P_IMPBRUT  TFC_VENDES.IMPBRUT%type,
                      RSP        OUT VARCHAR2) IS
    vNomProces TFC_RASTRE_PROCES.PROCES%Type := 'TFC_VEN.MOD_VENDA';
    vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
    vQNTVEN_old TFC_VENDES.QNTVEN%Type;
    vBENNET_old TFC_VENDES.BENNET%Type;
    vIMPBRUT_old TFC_VENDES.IMPBRUT%Type;
  BEGIN
    If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
      TFC_RAP.INICIAR_RASTRE(vNomProces);
    end if;
    
    -- Es graven els parmetres a la taula de rastre
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_DATA='    ||TO_CHAR(P_DATA,'DD-MM-YYYY'));
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_HORA='    ||P_HORA);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_IDEBOT='  ||P_IDEBOT);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_CODEAN13='||P_CODEAN13);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_QNTVEN='  ||P_QNTVEN);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_BENNET='  ||P_BENNET);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_IMPBRUT=' ||P_IMPBRUT);
    
    -- Es verifica el particionament de la taula 
    VERIFICAR_PARTICIONAMENT(to_number(to_char(P_DATA, 'YYYY')));
    
    begin
      Select ven.QNTVEN,
             ven.BENNET,
             ven.IMPBRUT
        into vQNTVEN_OLD,
             vBENNET_OLD,
             vIMPBRUT_OLD
        from TFC_VENDES ven
        where ven.TEM_DATA     = P_DATA and 
              ven.TEM_HORA     = P_HORA and 
              ven.BOT_IDEBOT   = P_IDEBOT and 
              ven.PRO_CODEAN13 = P_CODEAN13;
    exception 
      when no_data_found then 
        null;
    end;
    
    Update TFC_VENDES ven
       set ven.QNTVEN         = P_QNTVEN,
           ven.BENNET         = P_BENNET,
           ven.IMPBRUT        = P_IMPBRUT
       where ven.TEM_DATA     = P_DATA and 
             ven.TEM_HORA     = P_HORA and 
             ven.BOT_IDEBOT   = P_IDEBOT and 
             ven.PRO_CODEAN13 = P_CODEAN13;
             
    If (SQL%RowCount > 0) then 
      RSP := TFC_UTL.RSP_OK;
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre modificat.');
    else
      RSP := 'No existeix cap venda per la data = '||to_char(P_DATA, 'DD-MM-YYYY')||', hora = '||P_HORA||', id. de botiga = '||P_IDEBOT||', producte = '||P_CODEAN13;
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre no modificat.');
    end if;
    
    -- Arribats a aquest punt, tot ha anat b.
    If (RSP is null) then 
      RSP := TFC_UTL.RSP_OK;
    end if;

    -- Es calculen els indicadors 
    IF ((RSP = TFC_UTL.RSP_OK) and (TFC_UTL.CALCULAR_IND_PER_CADA_VENDA)) THEN 
      If (TFC_UTL.TIPUS_CALCUL_INDICADORS = TFC_UTL.CALCUL_INC) THEN 
        -- Es calculen les taules d' agregats 
        IF (RSP = TFC_UTL.RSP_OK) then 
          TFC_RAP.REGISTRAR_RASTRE_LINIA('Es calculen els agregats (INDICADORS_AGR_MOD).');
          INDICADORS_AGR_MOD(P_DATA,
                             P_HORA,
                             P_IDEBOT,
                             P_CODEAN13,
                             vQNTVEN_OLD,
                             vBENNET_OLD,
                             P_QNTVEN,
                             P_BENNET,
                             RSP);
        end if;
      
        IF (RSP = TFC_UTL.RSP_OK) then 
          TFC_INA.CALC_INA(TO_NUMBER(TO_CHAR(P_DATA, 'YYYY')), RSP);
        END IF;
      ELSIF (TFC_UTL.TIPUS_CALCUL_INDICADORS = TFC_UTL.CALCUL_TOT) THEN 
        TFC_INA.CALCULAR_INDICADORS_TOT(TO_NUMBER(TO_CHAR(P_DATA, 'YYYY')), RSP);
      END IF;
    END IF; 

    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);

    If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
      If (RSP = TFC_UTL.RSP_OK) then 
        TFC_RAP.FINALITZAR_RASTRE_OK;
      else
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    end if;
  EXCEPTION -- Tractament d' excepcions 
    WHEN others THEN 
      RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);
      
      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);
      
      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
  END;

  -- Procediment que elimina un registre de la taula TFC_VENDES 
  -- Parmetres d' entrada:
  --   P_DATA     => Data de la venda 
  --   P_HORA     => Hora de la venda 
  --   P_IDEBOT   => Identificador de la botiga que ha fet la venda 
  --   P_CODEAN13 => Codi del producte 
  -- Parmetres de sortida:
  --   P_RSP    => Indica si l' operaci ha tingut xit o no:
  --     'OK'   => Valor indicatiu que el procs ha anat b.
  --     ERROR  => En cas que el procs no vagi b, retornar l' error produt.
  PROCEDURE SUP_VENDA(P_DATA     TFC_VENDES.TEM_DATA%type,
                      P_HORA     TFC_VENDES.TEM_HORA%type,
                      P_IDEBOT   TFC_VENDES.BOT_IDEBOT%type,
                      P_CODEAN13 TFC_VENDES.PRO_CODEAN13%type,
                      RSP        OUT VARCHAR2) IS
    vNomProces         TFC_RASTRE_PROCES.PROCES%Type := 'TFC_VEN.SUP_VENDA';
    vRastre_Ja_Iniciat boolean := TFC_RAP.RASTRE_INICIAT;
  BEGIN
    If (NOT vRastre_Ja_Iniciat) then -- Si no s' ha iniciat el rastre es fa en aquest punt 
      TFC_RAP.INICIAR_RASTRE(vNomProces);
    end if;
    
    -- Es graven els parmetres a la taula de rastre
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Entrant al procs '||vNomProces);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres d'' entrada:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_DATA='    ||TO_CHAR(P_DATA,'DD-MM-YYYY'));
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_HORA='    ||P_HORA);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_IDEBOT='  ||P_IDEBOT);
    TFC_RAP.REGISTRAR_RASTRE_LINIA('P_CODEAN13='||P_CODEAN13);
    
    -- Es verifica el particionament de la taula 
    VERIFICAR_PARTICIONAMENT(to_number(to_char(P_DATA, 'YYYY')));
    
    Delete 
      from TFC_VENDES ven
      where ven.TEM_DATA     = P_DATA and 
            ven.TEM_HORA     = P_HORA and 
            ven.BOT_IDEBOT   = P_IDEBOT and 
            ven.PRO_CODEAN13 = P_CODEAN13;
            
    If (SQL%RowCount > 0) then 
      RSP := TFC_UTL.RSP_OK;
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre eliminat.');
    else
      RSP := 'No existeix cap venda per la data = '||to_char(P_DATA, 'DD-MM-YYYY')||', hora = '||P_HORA||', id. de botiga = '||P_IDEBOT||', producte = '||P_CODEAN13;
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Registre no eliminat.');
    end if;
                        
    -- Arribats a aquest punt, tot ha anat b.
    If (RSP is null) then 
      RSP := TFC_UTL.RSP_OK;
    end if;

    -- Es calculen els indicadors 
    IF ((RSP = TFC_UTL.RSP_OK) and (TFC_UTL.CALCULAR_IND_PER_CADA_VENDA)) THEN 
      If (TFC_UTL.TIPUS_CALCUL_INDICADORS = TFC_UTL.CALCUL_INC) THEN 
        -- Es calculen les taules d' agregats 
        IF (RSP = TFC_UTL.RSP_OK) then 
          TFC_RAP.REGISTRAR_RASTRE_LINIA('Es calculen els agregats (INDICADORS_AGR_MOD).');
          INDICADORS_AGR_SUP(P_DATA,
                             P_HORA,
                             P_IDEBOT,
                             P_CODEAN13,
                             RSP);                       
        end if;
        
        IF (RSP = TFC_UTL.RSP_OK) then 
          TFC_INA.CALC_INA(TO_NUMBER(TO_CHAR(P_DATA, 'YYYY')), RSP);
        END IF;
      ELSIF (TFC_UTL.TIPUS_CALCUL_INDICADORS = TFC_UTL.CALCUL_TOT) THEN 
        TFC_INA.CALCULAR_INDICADORS_TOT(TO_NUMBER(TO_CHAR(P_DATA, 'YYYY')), RSP);
      END IF;
    END IF; 

    -- Es graven els parmetres a la taula de rastre 
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
    TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
    TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);

    If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza.
      If (RSP = TFC_UTL.RSP_OK) then 
        TFC_RAP.FINALITZAR_RASTRE_OK;
      else
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
    end if;
  EXCEPTION -- Tractament d' excepcions  
    WHEN others THEN 
      RSP := SubStr(SQLCODE||' - '||SQLERRM, 1, 200);
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Error no controlat:');
      
      -- Es registre l' error a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA(RSP);

      -- Es graven els parmetres a la taula de rastre 
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Valors dels parmetres de sortida:');
      TFC_RAP.REGISTRAR_RASTRE_LINIA('RSP='||RSP);    
      TFC_RAP.REGISTRAR_RASTRE_LINIA('Sortint del procs '||vNomProces);

      If (NOT vRastre_Ja_Iniciat) then -- En cas que s' hagi iniciat el rastre, es finalitza amb error.
        TFC_RAP.FINALITZAR_RASTRE_ERR;
      end if;
  END;
end TFC_VEN;
/
